@@ -16,6 +16,7 @@
 
 #include "jstypes.h"
 
+#include "js/AllocPolicy.h"
 #include "js/RootingAPI.h"
 #include "js/TypeDecls.h"
 #include "js/Value.h"
@@ -388,9 +389,9 @@ class SharedArrayRawBufferRefs {
   SharedArrayRawBufferRefs& operator=(SharedArrayRawBufferRefs&& other);
   ~SharedArrayRawBufferRefs();
 
-  MOZ_MUST_USE bool acquire(JSContext* cx, SharedArrayRawBuffer* rawbuf);
-  MOZ_MUST_USE bool acquireAll(JSContext* cx,
-                               const SharedArrayRawBufferRefs& that);
+  [[nodiscard]] bool acquire(JSContext* cx, SharedArrayRawBuffer* rawbuf);
+  [[nodiscard]] bool acquireAll(JSContext* cx,
+                                const SharedArrayRawBufferRefs& that);
   void takeOwnership(SharedArrayRawBufferRefs&&);
   void releaseAll();
 
@@ -456,7 +457,7 @@ class MOZ_NON_MEMMOVABLE JS_PUBLIC_API JSStructuredCloneData {
                               JS::StructuredCloneScope::Unassigned) {}
   JSStructuredCloneData(JSStructuredCloneData&& other) = default;
   JSStructuredCloneData& operator=(JSStructuredCloneData&& other) = default;
-  ~JSStructuredCloneData() { discardTransferables(); }
+  ~JSStructuredCloneData();
 
   void setCallbacks(const JSStructuredCloneCallbacks* callbacks, void* closure,
                     OwnTransferablePolicy policy) {
@@ -465,7 +466,7 @@ class MOZ_NON_MEMMOVABLE JS_PUBLIC_API JSStructuredCloneData {
     ownTransferables_ = policy;
   }
 
-  MOZ_MUST_USE bool Init(size_t initialCapacity = 0) {
+  [[nodiscard]] bool Init(size_t initialCapacity = 0) {
     return bufList_.Init(0, initialCapacity);
   }
 
@@ -495,24 +496,25 @@ class MOZ_NON_MEMMOVABLE JS_PUBLIC_API JSStructuredCloneData {
 
   const Iterator Start() const { return bufList_.Iter(); }
 
-  MOZ_MUST_USE bool Advance(Iterator& iter, size_t distance) const {
+  [[nodiscard]] bool Advance(Iterator& iter, size_t distance) const {
     return iter.AdvanceAcrossSegments(bufList_, distance);
   }
 
-  MOZ_MUST_USE bool ReadBytes(Iterator& iter, char* buffer, size_t size) const {
+  [[nodiscard]] bool ReadBytes(Iterator& iter, char* buffer,
+                               size_t size) const {
     return bufList_.ReadBytes(iter, buffer, size);
   }
 
   // Append new data to the end of the buffer.
-  MOZ_MUST_USE bool AppendBytes(const char* data, size_t size) {
+  [[nodiscard]] bool AppendBytes(const char* data, size_t size) {
     MOZ_ASSERT(scope() != JS::StructuredCloneScope::Unassigned);
     return bufList_.WriteBytes(data, size);
   }
 
   // Update data stored within the existing buffer. There must be at least
   // 'size' bytes between the position of 'iter' and the end of the buffer.
-  MOZ_MUST_USE bool UpdateBytes(Iterator& iter, const char* data,
-                                size_t size) const {
+  [[nodiscard]] bool UpdateBytes(Iterator& iter, const char* data,
+                                 size_t size) const {
     MOZ_ASSERT(scope() != JS::StructuredCloneScope::Unassigned);
     while (size > 0) {
       size_t remaining = iter.RemainingInSegment();
@@ -562,7 +564,7 @@ class MOZ_NON_MEMMOVABLE JS_PUBLIC_API JSStructuredCloneData {
   }
 
   // Append the entire contents of other's bufList_ to our own.
-  MOZ_MUST_USE bool Append(const JSStructuredCloneData& other) {
+  [[nodiscard]] bool Append(const JSStructuredCloneData& other) {
     MOZ_ASSERT(scope() == other.scope());
     return other.ForEachDataChunk(
         [&](const char* data, size_t size) { return AppendBytes(data, size); });
