@@ -22,8 +22,11 @@
 #include "jit/AtomicOperations.h"
 #include "jit/InlinableNatives.h"
 #include "js/Conversions.h"
+#include "js/experimental/TypedData.h"  // JS_NewDataView
+#include "js/friend/ErrorMessages.h"    // js::GetErrorMessage, JSMSG_*
 #include "js/PropertySpec.h"
 #include "js/Wrapper.h"
+#include "util/DifferentialTesting.h"
 #include "util/Windows.h"
 #include "vm/ArrayBufferObject.h"
 #include "vm/GlobalObject.h"
@@ -31,6 +34,7 @@
 #include "vm/JSContext.h"
 #include "vm/JSObject.h"
 #include "vm/SharedMem.h"
+#include "vm/Uint8Clamped.h"
 #include "vm/WrapperObject.h"
 
 #include "gc/Nursery-inl.h"
@@ -46,7 +50,7 @@ using mozilla::AssertedCast;
 using mozilla::WrapToSigned;
 
 DataViewObject* DataViewObject::create(
-    JSContext* cx, uint32_t byteOffset, uint32_t byteLength,
+    JSContext* cx, size_t byteOffset, size_t byteLength,
     Handle<ArrayBufferObjectMaybeShared*> arrayBuffer, HandleObject proto) {
   if (arrayBuffer->isDetached()) {
     JS_ReportErrorNumberASCII(cx, GetErrorMessage, nullptr,
@@ -68,17 +72,16 @@ DataViewObject* DataViewObject::create(
 bool DataViewObject::getAndCheckConstructorArgs(JSContext* cx,
                                                 HandleObject bufobj,
                                                 const CallArgs& args,
-                                                uint32_t* byteOffsetPtr,
-                                                uint32_t* byteLengthPtr) {
+                                                size_t* byteOffsetPtr,
+                                                size_t* byteLengthPtr) {
   // Step 3.
-  if (!IsArrayBufferMaybeShared(bufobj)) {
+  if (!bufobj->is<ArrayBufferObjectMaybeShared>()) {
     JS_ReportErrorNumberASCII(cx, GetErrorMessage, nullptr,
                               JSMSG_NOT_EXPECTED_TYPE, "DataView",
                               "ArrayBuffer", bufobj->getClass()->name);
     return false;
   }
-  Rooted<ArrayBufferObjectMaybeShared*> buffer(
-      cx, &AsArrayBufferMaybeShared(bufobj));
+  auto buffer = bufobj.as<ArrayBufferObjectMaybeShared>();
 
   // Step 4.
   uint64_t offset;
@@ -94,7 +97,7 @@ bool DataViewObject::getAndCheckConstructorArgs(JSContext* cx,
   }
 
   // Step 6.
-  uint32_t bufferByteLength = buffer->byteLength();
+  size_t bufferByteLength = buffer->byteLength();
 
   // Step 7.
   if (offset > bufferByteLength) {
@@ -102,7 +105,7 @@ bool DataViewObject::getAndCheckConstructorArgs(JSContext* cx,
                               JSMSG_OFFSET_OUT_OF_BUFFER);
     return false;
   }
-  MOZ_ASSERT(offset <= INT32_MAX);
+  MOZ_ASSERT(offset <= ArrayBufferObject::maxBufferByteLength());
 
   // Step 8.a
   uint64_t viewByteLength = bufferByteLength - offset;
@@ -123,10 +126,10 @@ bool DataViewObject::getAndCheckConstructorArgs(JSContext* cx,
       return false;
     }
   }
-  MOZ_ASSERT(viewByteLength <= INT32_MAX);
+  MOZ_ASSERT(viewByteLength <= ArrayBufferObject::maxBufferByteLength());
 
-  *byteOffsetPtr = AssertedCast<uint32_t>(offset);
-  *byteLengthPtr = AssertedCast<uint32_t>(viewByteLength);
+  *byteOffsetPtr = offset;
+  *byteLengthPtr = viewByteLength;
   return true;
 }
 
@@ -136,7 +139,8 @@ bool DataViewObject::constructSameCompartment(JSContext* cx,
   MOZ_ASSERT(args.isConstructing());
   cx->check(bufobj);
 
-  uint32_t byteOffset, byteLength;
+  size_t byteOffset = 0;
+  size_t byteLength = 0;
   if (!getAndCheckConstructorArgs(cx, bufobj, args, &byteOffset, &byteLength)) {
     return false;
   }
@@ -146,8 +150,7 @@ bool DataViewObject::constructSameCompartment(JSContext* cx,
     return false;
   }
 
-  Rooted<ArrayBufferObjectMaybeShared*> buffer(
-      cx, &AsArrayBufferMaybeShared(bufobj));
+  auto buffer = bufobj.as<ArrayBufferObjectMaybeShared>();
   JSObject* obj =
       DataViewObject::create(cx, byteOffset, byteLength, buffer, proto);
   if (!obj) {
@@ -182,7 +185,8 @@ bool DataViewObject::constructWrapped(JSContext* cx, HandleObject bufobj,
   }
 
   // NB: This entails the IsArrayBuffer check
-  uint32_t byteOffset, byteLength;
+  size_t byteOffset = 0;
+  size_t byteLength = 0;
   if (!getAndCheckConstructorArgs(cx, unwrapped, args, &byteOffset,
                                   &byteLength)) {
     return false;
@@ -249,21 +253,13 @@ bool DataViewObject::construct(JSContext* cx, unsigned argc, Value* vp) {
 }
 
 template <typename NativeType>
-/* static */
-SharedMem<uint8_t*> DataViewObject::getDataPointer(JSContext* cx,
-                                                   Handle<DataViewObject*> obj,
-                                                   uint64_t offset,
+SharedMem<uint8_t*> DataViewObject::getDataPointer(uint64_t offset,
                                                    bool* isSharedMemory) {
-  const size_t TypeSize = sizeof(NativeType);
-  if (offset > UINT32_MAX - TypeSize || offset + TypeSize > obj->byteLength()) {
-    JS_ReportErrorNumberASCII(cx, GetErrorMessage, nullptr,
-                              JSMSG_OFFSET_OUT_OF_DATAVIEW);
-    return SharedMem<uint8_t*>::unshared(nullptr);
-  }
+  MOZ_ASSERT(offsetIsInBounds<NativeType>(offset));
 
-  MOZ_ASSERT(offset < UINT32_MAX);
-  *isSharedMemory = obj->isSharedMemory();
-  return obj->dataPointerEither().cast<uint8_t*>() + uint32_t(offset);
+  MOZ_ASSERT(offset < SIZE_MAX);
+  *isSharedMemory = this->isSharedMemory();
+  return dataPointerEither().cast<uint8_t*>() + size_t(offset);
 }
 
 template <typename T>
@@ -321,45 +317,61 @@ struct DataViewIO {
   }
 };
 
+template <typename NativeType>
+NativeType DataViewObject::read(uint64_t offset, bool isLittleEndian) {
+  bool isSharedMemory;
+  SharedMem<uint8_t*> data =
+      getDataPointer<NativeType>(offset, &isSharedMemory);
+  MOZ_ASSERT(data);
+
+  NativeType val = 0;
+  if (isSharedMemory) {
+    DataViewIO<NativeType, SharedMem<uint8_t*>>::fromBuffer(&val, data,
+                                                            isLittleEndian);
+  } else {
+    DataViewIO<NativeType, uint8_t*>::fromBuffer(&val, data.unwrapUnshared(),
+                                                 isLittleEndian);
+  }
+
+  return val;
+}
+
+template uint32_t DataViewObject::read(uint64_t offset, bool isLittleEndian);
+
+// https://tc39.github.io/ecma262/#sec-getviewvalue
+// GetViewValue ( view, requestIndex, isLittleEndian, type )
 template <typename NativeType>
 /* static */
 bool DataViewObject::read(JSContext* cx, Handle<DataViewObject*> obj,
                           const CallArgs& args, NativeType* val) {
-  // Steps 1-2. done by the caller
-  // Step 3. unnecessary assert
+  // Step 1. done by the caller
+  // Step 2. unnecessary assert
 
-  // Step 4.
+  // Step 3.
   uint64_t getIndex;
   if (!ToIndex(cx, args.get(0), &getIndex)) {
     return false;
   }
 
-  // Step 5.
+  // Step 4.
   bool isLittleEndian = args.length() >= 2 && ToBoolean(args[1]);
 
-  // Steps 6-7.
+  // Steps 5-6.
   if (obj->hasDetachedBuffer()) {
     JS_ReportErrorNumberASCII(cx, GetErrorMessage, nullptr,
                               JSMSG_TYPED_ARRAY_DETACHED);
     return false;
   }
 
-  // Steps 8-12.
-  bool isSharedMemory;
-  SharedMem<uint8_t*> data = DataViewObject::getDataPointer<NativeType>(
-      cx, obj, getIndex, &isSharedMemory);
-  if (!data) {
+  // Steps 7-10.
+  if (!obj->offsetIsInBounds<NativeType>(getIndex)) {
+    JS_ReportErrorNumberASCII(cx, GetErrorMessage, nullptr,
+                              JSMSG_OFFSET_OUT_OF_DATAVIEW);
     return false;
   }
 
-  // Step 13.
-  if (isSharedMemory) {
-    DataViewIO<NativeType, SharedMem<uint8_t*>>::fromBuffer(val, data,
-                                                            isLittleEndian);
-  } else {
-    DataViewIO<NativeType, uint8_t*>::fromBuffer(val, data.unwrapUnshared(),
-                                                 isLittleEndian);
-  }
+  // Steps 11-12.
+  *val = obj->read<NativeType>(getIndex, isLittleEndian);
   return true;
 }
 
@@ -427,27 +439,25 @@ template <typename NativeType>
 /* static */
 bool DataViewObject::write(JSContext* cx, Handle<DataViewObject*> obj,
                            const CallArgs& args) {
-  // Steps 1-2. done by the caller
-  // Step 3. unnecessary assert
+  // Step 1. done by the caller
+  // Step 2. unnecessary assert
 
-  // Step 4.
+  // Step 3.
   uint64_t getIndex;
   if (!ToIndex(cx, args.get(0), &getIndex)) {
     return false;
   }
 
-  // Step 5. Extended by the BigInt proposal to call either ToBigInt or ToNumber
+  // Steps 4-5. Call ToBigInt(value) or ToNumber(value) depending on the type.
   NativeType value;
   if (!WebIDLCast(cx, args.get(1), &value)) {
     return false;
   }
 
-#ifdef JS_MORE_DETERMINISTIC
   // See the comment in ElementSpecific::doubleToNative.
-  if (TypeIsFloatingPoint<NativeType>()) {
+  if (js::SupportDifferentialTesting() && TypeIsFloatingPoint<NativeType>()) {
     value = JS::CanonicalizeNaN(value);
   }
-#endif
 
   // Step 6.
   bool isLittleEndian = args.length() >= 3 && ToBoolean(args[2]);
@@ -459,15 +469,19 @@ bool DataViewObject::write(JSContext* cx, Handle<DataViewObject*> obj,
     return false;
   }
 
-  // Steps 9-13.
-  bool isSharedMemory;
-  SharedMem<uint8_t*> data = DataViewObject::getDataPointer<NativeType>(
-      cx, obj, getIndex, &isSharedMemory);
-  if (!data) {
+  // Steps 9-12.
+  if (!obj->offsetIsInBounds<NativeType>(getIndex)) {
+    JS_ReportErrorNumberASCII(cx, GetErrorMessage, nullptr,
+                              JSMSG_OFFSET_OUT_OF_DATAVIEW);
     return false;
   }
 
-  // Step 14.
+  // Steps 13-14.
+  bool isSharedMemory;
+  SharedMem<uint8_t*> data =
+      obj->getDataPointer<NativeType>(getIndex, &isSharedMemory);
+  MOZ_ASSERT(data);
+
   if (isSharedMemory) {
     DataViewIO<NativeType, SharedMem<uint8_t*>>::toBuffer(data, &value,
                                                           isLittleEndian);
@@ -871,7 +885,7 @@ bool DataViewObject::fun_setFloat64(JSContext* cx, unsigned argc, Value* vp) {
 bool DataViewObject::bufferGetterImpl(JSContext* cx, const CallArgs& args) {
   Rooted<DataViewObject*> thisView(
       cx, &args.thisv().toObject().as<DataViewObject>());
-  args.rval().set(DataViewObject::bufferValue(thisView));
+  args.rval().set(thisView->bufferValue());
   return true;
 }
 
@@ -892,7 +906,7 @@ bool DataViewObject::byteLengthGetterImpl(JSContext* cx, const CallArgs& args) {
   }
 
   // Step 7.
-  args.rval().set(DataViewObject::byteLengthValue(thisView));
+  args.rval().set(thisView->byteLengthValue());
   return true;
 }
 
@@ -913,7 +927,7 @@ bool DataViewObject::byteOffsetGetterImpl(JSContext* cx, const CallArgs& args) {
   }
 
   // Step 7.
-  args.rval().set(DataViewObject::byteOffsetValue(thisView));
+  args.rval().set(thisView->byteOffsetValue());
   return true;
 }
 
@@ -922,11 +936,6 @@ bool DataViewObject::byteOffsetGetter(JSContext* cx, unsigned argc, Value* vp) {
   return CallNonGenericMethod<is, byteOffsetGetterImpl>(cx, args);
 }
 
-JSObject* DataViewObject::CreatePrototype(JSContext* cx, JSProtoKey key) {
-  return GlobalObject::createBlankPrototype(cx, cx->global(),
-                                            &DataViewObject::protoClass_);
-}
-
 static const JSClassOps DataViewObjectClassOps = {
     nullptr,                       // addProperty
     nullptr,                       // delProperty
@@ -958,7 +967,7 @@ const JSClass DataViewObject::class_ = {
     &DataViewObjectClassOps, &DataViewObject::classSpec_};
 
 const JSClass DataViewObject::protoClass_ = {
-    js_Object_str, JSCLASS_HAS_CACHED_PROTO(JSProto_DataView),
+    "DataView.prototype", JSCLASS_HAS_CACHED_PROTO(JSProto_DataView),
     JS_NULL_CLASS_OPS, &DataViewObject::classSpec_};
 
 const JSFunctionSpec DataViewObject::methods[] = {
@@ -1010,9 +1019,8 @@ const JSPropertySpec DataViewObject::properties[] = {
     JS_PSG("byteOffset", DataViewObject::byteOffsetGetter, 0),
     JS_STRING_SYM_PS(toStringTag, "DataView", JSPROP_READONLY), JS_PS_END};
 
-JS_FRIEND_API JSObject* JS_NewDataView(JSContext* cx, HandleObject buffer,
-                                       uint32_t byteOffset,
-                                       int32_t byteLength) {
+JS_PUBLIC_API JSObject* JS_NewDataView(JSContext* cx, HandleObject buffer,
+                                       size_t byteOffset, size_t byteLength) {
   JSProtoKey key = JSProto_DataView;
   RootedObject constructor(cx, GlobalObject::getOrCreateConstructor(cx, key));
   if (!constructor) {
@@ -1023,7 +1031,7 @@ JS_FRIEND_API JSObject* JS_NewDataView(JSContext* cx, HandleObject buffer,
 
   cargs[0].setObject(*buffer);
   cargs[1].setNumber(byteOffset);
-  cargs[2].setInt32(byteLength);
+  cargs[2].setNumber(byteLength);
 
   RootedValue fun(cx, ObjectValue(*constructor));
   RootedObject obj(cx);
