@@ -7,6 +7,8 @@
 #ifndef builtin_MapObject_h
 #define builtin_MapObject_h
 
+#include "mozilla/MemoryReporting.h"
+
 #include "builtin/SelfHostingDefines.h"
 #include "vm/GlobalObject.h"
 #include "vm/JSObject.h"
@@ -49,13 +51,16 @@ class HashableValue {
 
   HashableValue() : value(UndefinedValue()) {}
 
-  MOZ_MUST_USE bool setValue(JSContext* cx, HandleValue v);
+  [[nodiscard]] bool setValue(JSContext* cx, HandleValue v);
   HashNumber hash(const mozilla::HashCodeScrambler& hcs) const;
   bool operator==(const HashableValue& other) const;
   HashableValue trace(JSTracer* trc) const;
   Value get() const { return value.get(); }
 
   void trace(JSTracer* trc) { TraceEdge(trc, &value, "HashableValue"); }
+
+  // Clear the value without invoking the pre-barrier.
+  void unbarrieredClear() { value.unbarrieredSet(UndefinedValue()); }
 };
 
 template <typename Wrapper>
@@ -68,7 +73,7 @@ template <typename Wrapper>
 class MutableWrappedPtrOperations<HashableValue, Wrapper>
     : public WrappedPtrOperations<HashableValue, Wrapper> {
  public:
-  MOZ_MUST_USE bool setValue(JSContext* cx, HandleValue v) {
+  [[nodiscard]] bool setValue(JSContext* cx, HandleValue v) {
     return static_cast<Wrapper*>(this)->get().setValue(cx, v);
   }
 };
@@ -110,29 +115,29 @@ class MapObject : public NativeObject {
 
   enum { NurseryKeysSlot, HasNurseryMemorySlot, SlotCount };
 
-  static MOZ_MUST_USE bool getKeysAndValuesInterleaved(
+  [[nodiscard]] static bool getKeysAndValuesInterleaved(
       HandleObject obj, JS::MutableHandle<GCVector<JS::Value>> entries);
-  static MOZ_MUST_USE bool entries(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static bool entries(JSContext* cx, unsigned argc, Value* vp);
   static MapObject* create(JSContext* cx, HandleObject proto = nullptr);
 
   // Publicly exposed Map calls for JSAPI access (webidl maplike/setlike
   // interfaces, etc.)
   static uint32_t size(JSContext* cx, HandleObject obj);
-  static MOZ_MUST_USE bool get(JSContext* cx, HandleObject obj, HandleValue key,
-                               MutableHandleValue rval);
-  static MOZ_MUST_USE bool has(JSContext* cx, HandleObject obj, HandleValue key,
-                               bool* rval);
-  static MOZ_MUST_USE bool delete_(JSContext* cx, HandleObject obj,
-                                   HandleValue key, bool* rval);
+  [[nodiscard]] static bool get(JSContext* cx, HandleObject obj,
+                                HandleValue key, MutableHandleValue rval);
+  [[nodiscard]] static bool has(JSContext* cx, HandleObject obj,
+                                HandleValue key, bool* rval);
+  [[nodiscard]] static bool delete_(JSContext* cx, HandleObject obj,
+                                    HandleValue key, bool* rval);
 
   // Set call for public JSAPI exposure. Does not actually return map object
   // as stated in spec, expects caller to return a value. for instance, with
   // webidl maplike/setlike, should return interface object.
-  static MOZ_MUST_USE bool set(JSContext* cx, HandleObject obj, HandleValue key,
-                               HandleValue val);
-  static MOZ_MUST_USE bool clear(JSContext* cx, HandleObject obj);
-  static MOZ_MUST_USE bool iterator(JSContext* cx, IteratorKind kind,
-                                    HandleObject obj, MutableHandleValue iter);
+  [[nodiscard]] static bool set(JSContext* cx, HandleObject obj,
+                                HandleValue key, HandleValue val);
+  [[nodiscard]] static bool clear(JSContext* cx, HandleObject obj);
+  [[nodiscard]] static bool iterator(JSContext* cx, IteratorKind kind,
+                                     HandleObject obj, MutableHandleValue iter);
 
   using UnbarrieredTable =
       OrderedHashMap<Value, Value, UnbarrieredHashPolicy, ZoneAllocPolicy>;
@@ -140,6 +145,8 @@ class MapObject : public NativeObject {
 
   static void sweepAfterMinorGC(JSFreeOp* fop, MapObject* mapobj);
 
+  size_t sizeOfData(mozilla::MallocSizeOf mallocSizeOf);
+
  private:
   static const ClassSpec classSpec_;
   static const JSClassOps classOps_;
@@ -147,36 +154,39 @@ class MapObject : public NativeObject {
   static const JSPropertySpec properties[];
   static const JSFunctionSpec methods[];
   static const JSPropertySpec staticProperties[];
+
+  static bool finishInit(JSContext* cx, HandleObject ctor, HandleObject proto);
+
   ValueMap* getData() { return static_cast<ValueMap*>(getPrivate()); }
   static ValueMap& extract(HandleObject o);
   static ValueMap& extract(const CallArgs& args);
   static void trace(JSTracer* trc, JSObject* obj);
   static void finalize(JSFreeOp* fop, JSObject* obj);
-  static MOZ_MUST_USE bool construct(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static bool construct(JSContext* cx, unsigned argc, Value* vp);
 
   static bool is(HandleValue v);
   static bool is(HandleObject o);
 
-  static MOZ_MUST_USE bool iterator_impl(JSContext* cx, const CallArgs& args,
-                                         IteratorKind kind);
+  [[nodiscard]] static bool iterator_impl(JSContext* cx, const CallArgs& args,
+                                          IteratorKind kind);
 
-  static MOZ_MUST_USE bool size_impl(JSContext* cx, const CallArgs& args);
-  static MOZ_MUST_USE bool size(JSContext* cx, unsigned argc, Value* vp);
-  static MOZ_MUST_USE bool get_impl(JSContext* cx, const CallArgs& args);
-  static MOZ_MUST_USE bool get(JSContext* cx, unsigned argc, Value* vp);
-  static MOZ_MUST_USE bool has_impl(JSContext* cx, const CallArgs& args);
-  static MOZ_MUST_USE bool has(JSContext* cx, unsigned argc, Value* vp);
-  static MOZ_MUST_USE bool set_impl(JSContext* cx, const CallArgs& args);
-  static MOZ_MUST_USE bool set(JSContext* cx, unsigned argc, Value* vp);
-  static MOZ_MUST_USE bool delete_impl(JSContext* cx, const CallArgs& args);
-  static MOZ_MUST_USE bool delete_(JSContext* cx, unsigned argc, Value* vp);
-  static MOZ_MUST_USE bool keys_impl(JSContext* cx, const CallArgs& args);
-  static MOZ_MUST_USE bool keys(JSContext* cx, unsigned argc, Value* vp);
-  static MOZ_MUST_USE bool values_impl(JSContext* cx, const CallArgs& args);
-  static MOZ_MUST_USE bool values(JSContext* cx, unsigned argc, Value* vp);
-  static MOZ_MUST_USE bool entries_impl(JSContext* cx, const CallArgs& args);
-  static MOZ_MUST_USE bool clear_impl(JSContext* cx, const CallArgs& args);
-  static MOZ_MUST_USE bool clear(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static bool size_impl(JSContext* cx, const CallArgs& args);
+  [[nodiscard]] static bool size(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static bool get_impl(JSContext* cx, const CallArgs& args);
+  [[nodiscard]] static bool get(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static bool has_impl(JSContext* cx, const CallArgs& args);
+  [[nodiscard]] static bool has(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static bool set_impl(JSContext* cx, const CallArgs& args);
+  [[nodiscard]] static bool set(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static bool delete_impl(JSContext* cx, const CallArgs& args);
+  [[nodiscard]] static bool delete_(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static bool keys_impl(JSContext* cx, const CallArgs& args);
+  [[nodiscard]] static bool keys(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static bool values_impl(JSContext* cx, const CallArgs& args);
+  [[nodiscard]] static bool values(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static bool entries_impl(JSContext* cx, const CallArgs& args);
+  [[nodiscard]] static bool clear_impl(JSContext* cx, const CallArgs& args);
+  [[nodiscard]] static bool clear(JSContext* cx, unsigned argc, Value* vp);
 };
 
 class MapIteratorObject : public NativeObject {
@@ -207,8 +217,8 @@ class MapIteratorObject : public NativeObject {
     initFixedSlot(KindSlot, JS::Int32Value(int32_t(kind)));
   }
 
-  static MOZ_MUST_USE bool next(Handle<MapIteratorObject*> mapIterator,
-                                HandleArrayObject resultPairObj, JSContext* cx);
+  [[nodiscard]] static bool next(MapIteratorObject* mapIterator,
+                                 ArrayObject* resultPairObj);
 
   static JSObject* createResultPair(JSContext* cx);
 
@@ -236,23 +246,23 @@ class SetObject : public NativeObject {
 
   enum { NurseryKeysSlot, HasNurseryMemorySlot, SlotCount };
 
-  static MOZ_MUST_USE bool keys(JSContext* cx, HandleObject obj,
-                                JS::MutableHandle<GCVector<JS::Value>> keys);
-  static MOZ_MUST_USE bool values(JSContext* cx, unsigned argc, Value* vp);
-  static MOZ_MUST_USE bool add(JSContext* cx, HandleObject obj,
-                               HandleValue key);
+  [[nodiscard]] static bool keys(JSContext* cx, HandleObject obj,
+                                 JS::MutableHandle<GCVector<JS::Value>> keys);
+  [[nodiscard]] static bool values(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static bool add(JSContext* cx, HandleObject obj,
+                                HandleValue key);
 
   // Publicly exposed Set calls for JSAPI access (webidl maplike/setlike
   // interfaces, etc.)
   static SetObject* create(JSContext* cx, HandleObject proto = nullptr);
   static uint32_t size(JSContext* cx, HandleObject obj);
-  static MOZ_MUST_USE bool has(JSContext* cx, HandleObject obj, HandleValue key,
-                               bool* rval);
-  static MOZ_MUST_USE bool clear(JSContext* cx, HandleObject obj);
-  static MOZ_MUST_USE bool iterator(JSContext* cx, IteratorKind kind,
-                                    HandleObject obj, MutableHandleValue iter);
-  static MOZ_MUST_USE bool delete_(JSContext* cx, HandleObject obj,
-                                   HandleValue key, bool* rval);
+  [[nodiscard]] static bool has(JSContext* cx, HandleObject obj,
+                                HandleValue key, bool* rval);
+  [[nodiscard]] static bool clear(JSContext* cx, HandleObject obj);
+  [[nodiscard]] static bool iterator(JSContext* cx, IteratorKind kind,
+                                     HandleObject obj, MutableHandleValue iter);
+  [[nodiscard]] static bool delete_(JSContext* cx, HandleObject obj,
+                                    HandleValue key, bool* rval);
 
   using UnbarrieredTable =
       OrderedHashSet<Value, UnbarrieredHashPolicy, ZoneAllocPolicy>;
@@ -260,6 +270,8 @@ class SetObject : public NativeObject {
 
   static void sweepAfterMinorGC(JSFreeOp* fop, SetObject* setobj);
 
+  size_t sizeOfData(mozilla::MallocSizeOf mallocSizeOf);
+
  private:
   static const ClassSpec classSpec_;
   static const JSClassOps classOps_;
@@ -268,6 +280,8 @@ class SetObject : public NativeObject {
   static const JSFunctionSpec methods[];
   static const JSPropertySpec staticProperties[];
 
+  static bool finishInit(JSContext* cx, HandleObject ctor, HandleObject proto);
+
   ValueSet* getData() { return static_cast<ValueSet*>(getPrivate()); }
   static ValueSet& extract(HandleObject o);
   static ValueSet& extract(const CallArgs& args);
@@ -280,22 +294,22 @@ class SetObject : public NativeObject {
 
   static bool isBuiltinAdd(HandleValue add);
 
-  static MOZ_MUST_USE bool iterator_impl(JSContext* cx, const CallArgs& args,
-                                         IteratorKind kind);
+  [[nodiscard]] static bool iterator_impl(JSContext* cx, const CallArgs& args,
+                                          IteratorKind kind);
 
-  static MOZ_MUST_USE bool size_impl(JSContext* cx, const CallArgs& args);
-  static MOZ_MUST_USE bool size(JSContext* cx, unsigned argc, Value* vp);
-  static MOZ_MUST_USE bool has_impl(JSContext* cx, const CallArgs& args);
-  static MOZ_MUST_USE bool has(JSContext* cx, unsigned argc, Value* vp);
-  static MOZ_MUST_USE bool add_impl(JSContext* cx, const CallArgs& args);
-  static MOZ_MUST_USE bool add(JSContext* cx, unsigned argc, Value* vp);
-  static MOZ_MUST_USE bool delete_impl(JSContext* cx, const CallArgs& args);
-  static MOZ_MUST_USE bool delete_(JSContext* cx, unsigned argc, Value* vp);
-  static MOZ_MUST_USE bool values_impl(JSContext* cx, const CallArgs& args);
-  static MOZ_MUST_USE bool entries_impl(JSContext* cx, const CallArgs& args);
-  static MOZ_MUST_USE bool entries(JSContext* cx, unsigned argc, Value* vp);
-  static MOZ_MUST_USE bool clear_impl(JSContext* cx, const CallArgs& args);
-  static MOZ_MUST_USE bool clear(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static bool size_impl(JSContext* cx, const CallArgs& args);
+  [[nodiscard]] static bool size(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static bool has_impl(JSContext* cx, const CallArgs& args);
+  [[nodiscard]] static bool has(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static bool add_impl(JSContext* cx, const CallArgs& args);
+  [[nodiscard]] static bool add(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static bool delete_impl(JSContext* cx, const CallArgs& args);
+  [[nodiscard]] static bool delete_(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static bool values_impl(JSContext* cx, const CallArgs& args);
+  [[nodiscard]] static bool entries_impl(JSContext* cx, const CallArgs& args);
+  [[nodiscard]] static bool entries(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static bool clear_impl(JSContext* cx, const CallArgs& args);
+  [[nodiscard]] static bool clear(JSContext* cx, unsigned argc, Value* vp);
 };
 
 class SetIteratorObject : public NativeObject {
@@ -326,8 +340,8 @@ class SetIteratorObject : public NativeObject {
     initFixedSlot(KindSlot, JS::Int32Value(int32_t(kind)));
   }
 
-  static MOZ_MUST_USE bool next(Handle<SetIteratorObject*> setIterator,
-                                HandleArrayObject resultObj, JSContext* cx);
+  [[nodiscard]] static bool next(SetIteratorObject* setIterator,
+                                 ArrayObject* resultObj);
 
   static JSObject* createResult(JSContext* cx);
 
@@ -340,10 +354,10 @@ using SetInitGetPrototypeOp = NativeObject* (*)(JSContext*,
 using SetInitIsBuiltinOp = bool (*)(HandleValue);
 
 template <SetInitGetPrototypeOp getPrototypeOp, SetInitIsBuiltinOp isBuiltinOp>
-static MOZ_MUST_USE bool IsOptimizableInitForSet(JSContext* cx,
-                                                 HandleObject setObject,
-                                                 HandleValue iterable,
-                                                 bool* optimized) {
+[[nodiscard]] static bool IsOptimizableInitForSet(JSContext* cx,
+                                                  HandleObject setObject,
+                                                  HandleValue iterable,
+                                                  bool* optimized) {
   MOZ_ASSERT(!*optimized);
 
   if (!iterable.isObject()) {
@@ -367,13 +381,13 @@ static MOZ_MUST_USE bool IsOptimizableInitForSet(JSContext* cx,
   }
 
   // Look up the 'add' value on the prototype object.
-  Shape* addShape = setProto->lookup(cx, cx->names().add);
-  if (!addShape || !addShape->isDataProperty()) {
+  mozilla::Maybe<PropertyInfo> addProp = setProto->lookup(cx, cx->names().add);
+  if (addProp.isNothing() || !addProp->isDataProperty()) {
     return true;
   }
 
   // Get the referred value, ensure it holds the canonical add function.
-  RootedValue add(cx, setProto->getSlot(addShape->slot()));
+  RootedValue add(cx, setProto->getSlot(addProp->slot()));
   if (!isBuiltinOp(add)) {
     return true;
   }
