@@ -25,19 +25,19 @@ class WeakSetObject : public WeakCollectionObject {
   static const JSFunctionSpec methods[];
 
   static WeakSetObject* create(JSContext* cx, HandleObject proto = nullptr);
-  static MOZ_MUST_USE bool construct(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static bool construct(JSContext* cx, unsigned argc, Value* vp);
 
-  static MOZ_MUST_USE MOZ_ALWAYS_INLINE bool is(HandleValue v);
+  [[nodiscard]] static MOZ_ALWAYS_INLINE bool is(HandleValue v);
 
-  static MOZ_MUST_USE MOZ_ALWAYS_INLINE bool add_impl(JSContext* cx,
-                                                      const CallArgs& args);
-  static MOZ_MUST_USE bool add(JSContext* cx, unsigned argc, Value* vp);
-  static MOZ_MUST_USE MOZ_ALWAYS_INLINE bool delete_impl(JSContext* cx,
-                                                         const CallArgs& args);
-  static MOZ_MUST_USE bool delete_(JSContext* cx, unsigned argc, Value* vp);
-  static MOZ_MUST_USE MOZ_ALWAYS_INLINE bool has_impl(JSContext* cx,
-                                                      const CallArgs& args);
-  static MOZ_MUST_USE bool has(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static MOZ_ALWAYS_INLINE bool add_impl(JSContext* cx,
+                                                       const CallArgs& args);
+  [[nodiscard]] static bool add(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static MOZ_ALWAYS_INLINE bool delete_impl(JSContext* cx,
+                                                          const CallArgs& args);
+  [[nodiscard]] static bool delete_(JSContext* cx, unsigned argc, Value* vp);
+  [[nodiscard]] static MOZ_ALWAYS_INLINE bool has_impl(JSContext* cx,
+                                                       const CallArgs& args);
+  [[nodiscard]] static bool has(JSContext* cx, unsigned argc, Value* vp);
 
   static bool isBuiltinAdd(HandleValue add);
 };
