@@ -32,6 +32,7 @@
 #include "vm/PlainObject.h"  // js::PlainObject
 #include "vm/Runtime.h"
 #include "vm/StringType.h"
+#include "vm/WellKnownAtom.h"  // js_*_str
 
 #include "vm/JSObject-inl.h"
 
@@ -59,7 +60,7 @@ const JSClassOps CollatorObject::classOps_ = {
 };
 
 const JSClass CollatorObject::class_ = {
-    js_Object_str,
+    "Intl.Collator",
     JSCLASS_HAS_RESERVED_SLOTS(CollatorObject::SLOT_COUNT) |
         JSCLASS_HAS_CACHED_PROTO(JSProto_Collator) |
         JSCLASS_FOREGROUND_FINALIZE,
@@ -84,7 +85,7 @@ static const JSFunctionSpec collator_methods[] = {
 
 static const JSPropertySpec collator_properties[] = {
     JS_SELF_HOSTED_GET("compare", "$Intl_Collator_compare_get", 0),
-    JS_STRING_SYM_PS(toStringTag, "Object", JSPROP_READONLY), JS_PS_END};
+    JS_STRING_SYM_PS(toStringTag, "Intl.Collator", JSPROP_READONLY), JS_PS_END};
 
 static bool Collator(JSContext* cx, unsigned argc, Value* vp);
 
