@@ -216,14 +216,18 @@ class MOZ_STACK_CLASS LanguageTag final {
   static bool complexLanguageMapping(const LanguageSubtag& language);
 
  private:
+  static bool scriptMapping(ScriptSubtag& script);
   static bool regionMapping(RegionSubtag& region);
   static bool complexRegionMapping(const RegionSubtag& region);
 
   void performComplexLanguageMappings();
   void performComplexRegionMappings();
-  MOZ_MUST_USE bool performVariantMappings(JSContext* cx);
+  [[nodiscard]] bool performVariantMappings(JSContext* cx);
 
-  MOZ_MUST_USE bool updateGrandfatheredMappings(JSContext* cx);
+  [[nodiscard]] bool updateLegacyMappings(JSContext* cx);
+
+  static bool signLanguageMapping(LanguageSubtag& language,
+                                  const RegionSubtag& region);
 
   static const char* replaceTransformExtensionType(
       mozilla::Span<const char> key, mozilla::Span<const char> type);
@@ -345,19 +349,8 @@ class MOZ_STACK_CLASS LanguageTag final {
     privateuse_ = std::move(privateuse);
   }
 
- private:
-  enum class DuplicateVariants { Reject, Accept };
-
-  bool canonicalizeBaseName(JSContext* cx, DuplicateVariants duplicateVariants);
-
- public:
-  /**
-   * Canonicalize the base-name subtags, that means the language, script,
-   * region, and variant subtags.
-   */
-  bool canonicalizeBaseName(JSContext* cx) {
-    return canonicalizeBaseName(cx, DuplicateVariants::Reject);
-  }
+  /** Canonicalize the base-name (language, script, region, variant) subtags. */
+  bool canonicalizeBaseName(JSContext* cx);
 
   /**
    * Canonicalize all extension subtags.
@@ -479,10 +472,10 @@ class MOZ_STACK_CLASS LanguageTagParser final {
     size_t length = tok.length();
     if (locale_.is<const JS::Latin1Char*>()) {
       using T = const JS::Latin1Char;
-      subtag.set(mozilla::MakeSpan(locale_.as<T*>() + index, length));
+      subtag.set(mozilla::Span(locale_.as<T*>() + index, length));
     } else {
       using T = const char16_t;
-      subtag.set(mozilla::MakeSpan(locale_.as<T*>() + index, length));
+      subtag.set(mozilla::Span(locale_.as<T*>() + index, length));
     }
   }
 
@@ -725,22 +718,22 @@ MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS(LanguageTagParser::TokenKind)
  * Parse a string as a standalone |language| tag. If |str| is a standalone
  * language tag, store it in |result| and return true. Otherwise return false.
  */
-MOZ_MUST_USE bool ParseStandaloneLanguageTag(JS::Handle<JSLinearString*> str,
-                                             LanguageSubtag& result);
+[[nodiscard]] bool ParseStandaloneLanguageTag(JS::Handle<JSLinearString*> str,
+                                              LanguageSubtag& result);
 
 /**
  * Parse a string as a standalone |script| tag. If |str| is a standalone script
  * tag, store it in |result| and return true. Otherwise return false.
  */
-MOZ_MUST_USE bool ParseStandaloneScriptTag(JS::Handle<JSLinearString*> str,
-                                           ScriptSubtag& result);
+[[nodiscard]] bool ParseStandaloneScriptTag(JS::Handle<JSLinearString*> str,
+                                            ScriptSubtag& result);
 
 /**
  * Parse a string as a standalone |region| tag. If |str| is a standalone region
  * tag, store it in |result| and return true. Otherwise return false.
  */
-MOZ_MUST_USE bool ParseStandaloneRegionTag(JS::Handle<JSLinearString*> str,
-                                           RegionSubtag& result);
+[[nodiscard]] bool ParseStandaloneRegionTag(JS::Handle<JSLinearString*> str,
+                                            RegionSubtag& result);
 
 /**
  * Parse a string as an ISO-639 language code. Return |nullptr| in the result if
@@ -768,7 +761,7 @@ class UnicodeExtensionKeyword final {
   void trace(JSTracer* trc);
 };
 
-extern MOZ_MUST_USE bool ApplyUnicodeExtensionToTag(
+[[nodiscard]] extern bool ApplyUnicodeExtensionToTag(
     JSContext* cx, LanguageTag& tag,
     JS::HandleVector<UnicodeExtensionKeyword> keywords);
 
