@@ -8,7 +8,7 @@
 #define debugger_Debugger_h
 
 #include "mozilla/Assertions.h"        // for MOZ_ASSERT_HELPER1
-#include "mozilla/Attributes.h"        // for MOZ_MUST_USE, MOZ_RAII
+#include "mozilla/Attributes.h"        // for MOZ_RAII
 #include "mozilla/DoublyLinkedList.h"  // for DoublyLinkedListElement
 #include "mozilla/HashTable.h"         // for HashSet, DefaultHasher (ptr only)
 #include "mozilla/LinkedList.h"        // for LinkedList (ptr only)
@@ -54,7 +54,7 @@
 class JS_PUBLIC_API JSFunction;
 
 namespace JS {
-class JS_FRIEND_API AutoStableStringChars;
+class JS_PUBLIC_API AutoStableStringChars;
 class JS_PUBLIC_API Compartment;
 class JS_PUBLIC_API Realm;
 class JS_PUBLIC_API Zone;
@@ -68,12 +68,10 @@ class DebuggerEnvironment;
 class PromiseObject;
 namespace gc {
 struct Cell;
-}
+} /* namespace gc */
 namespace wasm {
 class Instance;
-}
-template <typename T>
-struct GCManagedDeletePolicy;
+} /* namespace wasm */
 } /* namespace js */
 
 /*
@@ -395,13 +393,8 @@ class DebuggerWeakMap : private WeakMap<HeapPtr<Referent*>, HeapPtr<Wrapper*>> {
  public:
   void traceCrossCompartmentEdges(JSTracer* tracer) {
     for (Enum e(*this); !e.empty(); e.popFront()) {
+      TraceEdge(tracer, &e.front().mutableKey(), "Debugger WeakMap key");
       e.front().value()->trace(tracer);
-      Key key = e.front().key();
-      TraceEdge(tracer, &key, "Debugger WeakMap key");
-      if (key != e.front().key()) {
-        e.rekeyFront(key);
-      }
-      key.unsafeSet(nullptr);
     }
   }
 
@@ -426,15 +419,17 @@ class MOZ_RAII EvalOptions {
   ~EvalOptions() = default;
   const char* filename() const { return filename_.get(); }
   unsigned lineno() const { return lineno_; }
-  MOZ_MUST_USE bool setFilename(JSContext* cx, const char* filename);
+  [[nodiscard]] bool setFilename(JSContext* cx, const char* filename);
   void setLineno(unsigned lineno) { lineno_ = lineno; }
 };
 
 /*
- * Env is the type of what ES5 calls "lexical environments" (runtime activations
- * of lexical scopes). This is currently just JSObject, and is implemented by
- * CallObject, LexicalEnvironmentObject, and WithEnvironmentObject, among
- * others--but environments and objects are really two different concepts.
+ * Env is the type of what ECMA-262 calls "lexical environments" (the records
+ * that represent scopes and bindings). See vm/EnvironmentObject.h.
+ *
+ * This is JSObject rather than js::EnvironmentObject because GlobalObject and
+ * some proxies, despite not being in the EnvironmentObject class hierarchy,
+ * can be in environment chains.
  */
 using Env = JSObject;
 
@@ -484,7 +479,7 @@ class MOZ_RAII DebuggerList {
   DebuggerList(JSContext* cx, HookIsEnabledFun hookIsEnabled)
       : debuggers(cx), hookIsEnabled(hookIsEnabled) {}
 
-  MOZ_MUST_USE bool init(JSContext* cx);
+  [[nodiscard]] bool init(JSContext* cx);
 
   bool empty() { return debuggers.empty(); }
 
@@ -495,7 +490,7 @@ class MOZ_RAII DebuggerList {
   void dispatchQuietHook(JSContext* cx, FireHookFun fireHook);
 
   template <typename FireHookFun /* bool (Debugger*, ResumeMode&, MutableHandleValue) */>
-  MOZ_MUST_USE bool dispatchResumptionHook(JSContext* cx,
+  [[nodiscard]] bool dispatchResumptionHook(JSContext* cx,
                                            AbstractFramePtr frame,
                                            FireHookFun fireHook);
 };
@@ -518,6 +513,7 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
   template <typename>
   friend class DebuggerList;
   friend struct JSRuntime::GlobalObjectWatchersLinkAccess<Debugger>;
+  friend struct JSRuntime::GarbageCollectionWatchersLinkAccess<Debugger>;
   friend class SavedStacks;
   friend class ScriptedOnStepHandler;
   friend class ScriptedOnPopHandler;
@@ -586,12 +582,10 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
 
   struct AllocationsLogEntry {
     AllocationsLogEntry(HandleObject frame, mozilla::TimeStamp when,
-                        const char* className, HandleAtom ctorName, size_t size,
-                        bool inNursery)
+                        const char* className, size_t size, bool inNursery)
         : frame(frame),
           when(when),
           className(className),
-          ctorName(ctorName),
           size(size),
           inNursery(inNursery) {
       MOZ_ASSERT_IF(frame, UncheckedUnwrap(frame)->is<SavedFrame>() ||
@@ -601,32 +595,20 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
     HeapPtr<JSObject*> frame;
     mozilla::TimeStamp when;
     const char* className;
-    HeapPtr<JSAtom*> ctorName;
     size_t size;
     bool inNursery;
 
     void trace(JSTracer* trc) {
       TraceNullableEdge(trc, &frame, "Debugger::AllocationsLogEntry::frame");
-      TraceNullableEdge(trc, &ctorName,
-                        "Debugger::AllocationsLogEntry::ctorName");
     }
   };
 
-  // Barrier methods so we can have WeakHeapPtr<Debugger*>.
-  static void readBarrier(Debugger* dbg) {
-    InternalBarrierMethods<JSObject*>::readBarrier(dbg->object);
-  }
-  static void writeBarrierPost(Debugger** vp, Debugger* prev, Debugger* next) {}
-#ifdef DEBUG
-  static void assertThingIsNotGray(Debugger* dbg) { return; }
-#endif
-
  private:
-  GCPtrNativeObject object; /* The Debugger object. Strong reference. */
+  HeapPtrNativeObject object; /* The Debugger object. Strong reference. */
   WeakGlobalObjectSet
       debuggees; /* Debuggee globals. Cross-compartment weak references. */
   JS::ZoneSet debuggeeZones; /* Set of zones that we have debuggees in. */
-  js::GCPtrObject uncaughtExceptionHook; /* Strong reference. */
+  HeapPtrObject uncaughtExceptionHook; /* Strong reference. */
   bool allowUnobservedAsmJS;
 
   // Whether to enable code coverage on the Debuggee.
@@ -661,9 +643,9 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
 
   static const size_t DEFAULT_MAX_LOG_LENGTH = 5000;
 
-  MOZ_MUST_USE bool appendAllocationSite(JSContext* cx, HandleObject obj,
-                                         HandleSavedFrame frame,
-                                         mozilla::TimeStamp when);
+  [[nodiscard]] bool appendAllocationSite(JSContext* cx, HandleObject obj,
+                                          HandleSavedFrame frame,
+                                          mozilla::TimeStamp when);
 
   /*
    * Recompute the set of debuggee zones based on the set of debuggee globals.
@@ -682,7 +664,7 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
    * debuggee's compartment. The given debuggee global must be observed by at
    * least one Debugger that is tracking allocations.
    */
-  static MOZ_MUST_USE bool addAllocationsTracking(
+  [[nodiscard]] static bool addAllocationsTracking(
       JSContext* cx, Handle<GlobalObject*> debuggee);
 
   /*
@@ -695,7 +677,7 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
   /*
    * Add or remove allocations tracking for all debuggees.
    */
-  MOZ_MUST_USE bool addAllocationsTrackingForAllDebuggees(JSContext* cx);
+  [[nodiscard]] bool addAllocationsTrackingForAllDebuggees(JSContext* cx);
   void removeAllocationsTrackingForAllDebuggees();
 
   /*
@@ -705,6 +687,13 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
    */
   mozilla::DoublyLinkedListElement<Debugger> onNewGlobalObjectWatchersLink;
 
+  /*
+   * If this Debugger has a onGarbageCollection handler, then
+   * this link is inserted into the list headed by
+   * JSRuntime::onGarbageCollectionWatchers.
+   */
+  mozilla::DoublyLinkedListElement<Debugger> onGarbageCollectionWatchersLink;
+
   /*
    * Map from stack frames that are currently on the stack to Debugger.Frame
    * instances.
@@ -799,7 +788,8 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
 
   enum class FromSweep { No, Yes };
 
-  MOZ_MUST_USE bool addDebuggeeGlobal(JSContext* cx, Handle<GlobalObject*> obj);
+  [[nodiscard]] bool addDebuggeeGlobal(JSContext* cx,
+                                       Handle<GlobalObject*> obj);
   void removeDebuggeeGlobal(JSFreeOp* fop, GlobalObject* global,
                             WeakGlobalObjectSet::Enum* debugEnum,
                             FromSweep fromSweep);
@@ -830,12 +820,11 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
    *     anything else - Make a new TypeError the pending exception and
    *         attempt to handle it with the uncaught exception handler.
    */
-  MOZ_MUST_USE bool processHandlerResult(JSContext* cx, bool success,
-                                         HandleValue rv, AbstractFramePtr frame,
-                                         jsbytecode* pc, ResumeMode& resultMode,
-                                         MutableHandleValue vp);
+  [[nodiscard]] bool processHandlerResult(
+      JSContext* cx, bool success, HandleValue rv, AbstractFramePtr frame,
+      jsbytecode* pc, ResumeMode& resultMode, MutableHandleValue vp);
 
-  MOZ_MUST_USE bool processParsedHandlerResult(
+  [[nodiscard]] bool processParsedHandlerResult(
       JSContext* cx, AbstractFramePtr frame, jsbytecode* pc, bool success,
       ResumeMode resumeMode, HandleValue value, ResumeMode& resultMode,
       MutableHandleValue vp);
@@ -844,16 +833,16 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
    * Given a resumption return value from a hook, parse and validate it based
    * on the given frame, and split the result into a ResumeMode and Value.
    */
-  MOZ_MUST_USE bool prepareResumption(JSContext* cx, AbstractFramePtr frame,
-                                      jsbytecode* pc, ResumeMode& resumeMode,
-                                      MutableHandleValue vp);
+  [[nodiscard]] bool prepareResumption(JSContext* cx, AbstractFramePtr frame,
+                                       jsbytecode* pc, ResumeMode& resumeMode,
+                                       MutableHandleValue vp);
 
   /**
    * If there is a pending exception and a handler, call the handler with the
    * exception so that it can attempt to resolve the error.
    */
-  MOZ_MUST_USE bool callUncaughtExceptionHandler(JSContext* cx,
-                                                 MutableHandleValue vp);
+  [[nodiscard]] bool callUncaughtExceptionHandler(JSContext* cx,
+                                                  MutableHandleValue vp);
 
   /**
    * If the context has a pending exception, report it to the current global.
@@ -864,14 +853,13 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
    * Call the uncaught exception handler if there is one, returning true
    * if it handled the error, or false otherwise.
    */
-  MOZ_MUST_USE bool handleUncaughtException(JSContext* cx);
+  [[nodiscard]] bool handleUncaughtException(JSContext* cx);
 
   GlobalObject* unwrapDebuggeeArgument(JSContext* cx, const Value& v);
 
   static void traceObject(JSTracer* trc, JSObject* obj);
 
   void trace(JSTracer* trc);
-  friend struct js::GCManagedDeletePolicy<Debugger>;
 
   void traceForMovingGC(JSTracer* trc);
   void traceCrossCompartmentEdges(JSTracer* tracer);
@@ -880,10 +868,17 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
   template <typename F>
   void forEachWeakMap(const F& f);
 
-  static MOZ_MUST_USE bool getHookImpl(JSContext* cx, const CallArgs& args,
-                                       Debugger& dbg, Hook which);
-  static MOZ_MUST_USE bool setHookImpl(JSContext* cx, const CallArgs& args,
-                                       Debugger& dbg, Hook which);
+  [[nodiscard]] static bool getHookImpl(JSContext* cx, const CallArgs& args,
+                                        Debugger& dbg, Hook which);
+  [[nodiscard]] static bool setHookImpl(JSContext* cx, const CallArgs& args,
+                                        Debugger& dbg, Hook which);
+
+  [[nodiscard]] static bool getGarbageCollectionHook(JSContext* cx,
+                                                     const CallArgs& args,
+                                                     Debugger& dbg);
+  [[nodiscard]] static bool setGarbageCollectionHook(JSContext* cx,
+                                                     const CallArgs& args,
+                                                     Debugger& dbg);
 
   static bool isCompilableUnit(JSContext* cx, unsigned argc, Value* vp);
   static bool recordReplayProcessKind(JSContext* cx, unsigned argc, Value* vp);
@@ -895,9 +890,35 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
   static const JSFunctionSpec methods[];
   static const JSFunctionSpec static_methods[];
 
-  static void removeFromFrameMapsAndClearBreakpointsIn(JSContext* cx,
-                                                       AbstractFramePtr frame,
-                                                       bool suspending = false);
+  /**
+   * Suspend the DebuggerFrame, clearing on-stack data but leaving it linked
+   * with the AbstractGeneratorObject so it can be re-used later.
+   */
+  static void suspendGeneratorDebuggerFrames(JSContext* cx,
+                                             AbstractFramePtr frame);
+
+  /**
+   * Terminate the DebuggerFrame, clearing all data associated with the frame
+   * so that it cannot be used to introspect stack frame data.
+   */
+  static void terminateDebuggerFrames(JSContext* cx, AbstractFramePtr frame);
+
+  /**
+   * Terminate a given DebuggerFrame, removing all internal state and all
+   * references to the frame from the Debugger itself. If the frame is being
+   * terminated while 'frames' or 'generatorFrames' are being iterated, pass a
+   * pointer to the iteration Enum to remove the entry and ensure that iteration
+   * behaves properly.
+   *
+   * The AbstractFramePtr may be omited in a call so long as it is either
+   * called again later with the correct 'frame', or the frame itself has never
+   * had on-stack data or a 'frames' entry and has never had an onStep handler.
+   */
+  static void terminateDebuggerFrame(
+      JSFreeOp* fop, Debugger* dbg, DebuggerFrame* dbgFrame,
+      AbstractFramePtr frame, FrameMap::Enum* maybeFramesEnum = nullptr,
+      GeneratorWeakMap::Enum* maybeGeneratorFramesEnum = nullptr);
+
   static bool updateExecutionObservabilityOfFrames(
       JSContext* cx, const DebugAPI::ExecutionObservableSet& obs,
       IsObserving observing);
@@ -908,8 +929,12 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
       JSContext* cx, DebugAPI::ExecutionObservableSet& obs,
       IsObserving observing);
 
-  template <typename FrameFn /* void (DebuggerFrame*) */>
-  static void forEachDebuggerFrame(AbstractFramePtr frame, FrameFn fn);
+  template <typename FrameFn /* void (Debugger*, DebuggerFrame*) */>
+  static void forEachOnStackDebuggerFrame(AbstractFramePtr frame, FrameFn fn);
+  template <typename FrameFn /* void (Debugger*, DebuggerFrame*) */>
+  static void forEachOnStackOrSuspendedDebuggerFrame(JSContext* cx,
+                                                     AbstractFramePtr frame,
+                                                     FrameFn fn);
 
   /*
    * Return a vector containing all Debugger.Frame instances referring to
@@ -917,12 +942,12 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
    * compute it ourselves from |frame|.
    */
   using DebuggerFrameVector = GCVector<DebuggerFrame*>;
-  static MOZ_MUST_USE bool getDebuggerFrames(
+  [[nodiscard]] static bool getDebuggerFrames(
       AbstractFramePtr frame, MutableHandle<DebuggerFrameVector> frames);
 
  public:
   // Public for DebuggerScript::setBreakpoint.
-  static MOZ_MUST_USE bool ensureExecutionObservabilityOfScript(
+  [[nodiscard]] static bool ensureExecutionObservabilityOfScript(
       JSContext* cx, JSScript* script);
 
   // Whether the Debugger instance needs to observe all non-AOT JS
@@ -941,17 +966,17 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
   IsObserving observesNativeCalls() const;
 
  private:
-  static MOZ_MUST_USE bool ensureExecutionObservabilityOfFrame(
+  [[nodiscard]] static bool ensureExecutionObservabilityOfFrame(
       JSContext* cx, AbstractFramePtr frame);
-  static MOZ_MUST_USE bool ensureExecutionObservabilityOfRealm(
+  [[nodiscard]] static bool ensureExecutionObservabilityOfRealm(
       JSContext* cx, JS::Realm* realm);
 
   static bool hookObservesAllExecution(Hook which);
 
-  MOZ_MUST_USE bool updateObservesAllExecutionOnDebuggees(
+  [[nodiscard]] bool updateObservesAllExecutionOnDebuggees(
       JSContext* cx, IsObserving observing);
-  MOZ_MUST_USE bool updateObservesCoverageOnDebuggees(JSContext* cx,
-                                                      IsObserving observing);
+  [[nodiscard]] bool updateObservesCoverageOnDebuggees(JSContext* cx,
+                                                       IsObserving observing);
   void updateObservesAsmJSOnDebuggees(IsObserving observing);
 
   JSObject* getHook(Hook hook) const;
@@ -967,12 +992,12 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
                                 FireHookFun fireHook);
   template <
       typename HookIsEnabledFun /* bool (Debugger*) */, typename FireHookFun /* bool (Debugger*, ResumeMode&, MutableHandleValue) */>
-  static MOZ_MUST_USE bool dispatchResumptionHook(
+  [[nodiscard]] static bool dispatchResumptionHook(
       JSContext* cx, AbstractFramePtr frame, HookIsEnabledFun hookIsEnabled,
       FireHookFun fireHook);
 
   template <typename RunImpl /* bool () */>
-  MOZ_MUST_USE bool enterDebuggerHook(JSContext* cx, RunImpl runImpl) {
+  [[nodiscard]] bool enterDebuggerHook(JSContext* cx, RunImpl runImpl) {
     if (!isHookCallAllowed(cx)) {
       return true;
     }
@@ -993,20 +1018,21 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
     return true;
   }
 
-  MOZ_MUST_USE bool fireDebuggerStatement(JSContext* cx, ResumeMode& resumeMode,
-                                          MutableHandleValue vp);
-  MOZ_MUST_USE bool fireExceptionUnwind(JSContext* cx, HandleValue exc,
-                                        ResumeMode& resumeMode,
-                                        MutableHandleValue vp);
-  MOZ_MUST_USE bool fireEnterFrame(JSContext* cx, ResumeMode& resumeMode,
-                                   MutableHandleValue vp);
-  MOZ_MUST_USE bool fireNativeCall(JSContext* cx, const CallArgs& args,
-                                   CallReason reason, ResumeMode& resumeMode,
-                                   MutableHandleValue vp);
-  MOZ_MUST_USE bool fireNewGlobalObject(JSContext* cx,
-                                        Handle<GlobalObject*> global);
-  MOZ_MUST_USE bool firePromiseHook(JSContext* cx, Hook hook,
-                                    HandleObject promise);
+  [[nodiscard]] bool fireDebuggerStatement(JSContext* cx,
+                                           ResumeMode& resumeMode,
+                                           MutableHandleValue vp);
+  [[nodiscard]] bool fireExceptionUnwind(JSContext* cx, HandleValue exc,
+                                         ResumeMode& resumeMode,
+                                         MutableHandleValue vp);
+  [[nodiscard]] bool fireEnterFrame(JSContext* cx, ResumeMode& resumeMode,
+                                    MutableHandleValue vp);
+  [[nodiscard]] bool fireNativeCall(JSContext* cx, const CallArgs& args,
+                                    CallReason reason, ResumeMode& resumeMode,
+                                    MutableHandleValue vp);
+  [[nodiscard]] bool fireNewGlobalObject(JSContext* cx,
+                                         Handle<GlobalObject*> global);
+  [[nodiscard]] bool firePromiseHook(JSContext* cx, Hook hook,
+                                     HandleObject promise);
 
   DebuggerScript* newVariantWrapper(JSContext* cx,
                                     Handle<DebuggerScriptReferent> referent) {
@@ -1052,35 +1078,34 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
    * Receive a "new script" event from the engine. A new script was compiled
    * or deserialized.
    */
-  MOZ_MUST_USE bool fireNewScript(
+  [[nodiscard]] bool fireNewScript(
       JSContext* cx, Handle<DebuggerScriptReferent> scriptReferent);
 
   /*
    * Receive a "garbage collection" event from the engine. A GC cycle with the
    * given data was recently completed.
    */
-  MOZ_MUST_USE bool fireOnGarbageCollectionHook(
+  [[nodiscard]] bool fireOnGarbageCollectionHook(
       JSContext* cx, const JS::dbg::GarbageCollectionEvent::Ptr& gcData);
 
   inline Breakpoint* firstBreakpoint() const;
 
-  static MOZ_MUST_USE bool replaceFrameGuts(JSContext* cx,
-                                            AbstractFramePtr from,
-                                            AbstractFramePtr to,
-                                            ScriptFrameIter& iter);
+  [[nodiscard]] static bool replaceFrameGuts(JSContext* cx,
+                                             AbstractFramePtr from,
+                                             AbstractFramePtr to,
+                                             ScriptFrameIter& iter);
 
  public:
   Debugger(JSContext* cx, NativeObject* dbg);
   ~Debugger();
 
-  inline const js::GCPtrNativeObject& toJSObject() const;
-  inline js::GCPtrNativeObject& toJSObjectRef();
+  inline const js::HeapPtrNativeObject& toJSObject() const;
+  inline js::HeapPtrNativeObject& toJSObjectRef();
   static inline Debugger* fromJSObject(const JSObject* obj);
 
 #ifdef DEBUG
   static bool isChildJSObject(JSObject* obj);
 #endif
-  static Debugger* fromChildJSObject(JSObject* obj);
 
   Zone* zone() const { return toJSObject()->zone(); }
 
@@ -1089,7 +1114,6 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
 
   WeakGlobalObjectSet::Range allDebuggees() const { return debuggees.all(); }
 
-  static void detachAllDebuggersFromGlobal(JSFreeOp* fop, GlobalObject* global);
 #ifdef DEBUG
   static bool isDebuggerCrossCompartmentEdge(JSObject* obj,
                                              const js::gc::Cell* cell);
@@ -1113,10 +1137,10 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
    * or create a Debugger.Environment object for the given Env. On success,
    * store the Environment object in *vp and return true.
    */
-  MOZ_MUST_USE bool wrapEnvironment(JSContext* cx, Handle<Env*> env,
-                                    MutableHandleValue vp);
-  MOZ_MUST_USE bool wrapEnvironment(JSContext* cx, Handle<Env*> env,
-                                    MutableHandleDebuggerEnvironment result);
+  [[nodiscard]] bool wrapEnvironment(JSContext* cx, Handle<Env*> env,
+                                     MutableHandleValue vp);
+  [[nodiscard]] bool wrapEnvironment(JSContext* cx, Handle<Env*> env,
+                                     MutableHandleDebuggerEnvironment result);
 
   /*
    * Like cx->compartment()->wrap(cx, vp), but for the debugger realm.
@@ -1130,7 +1154,7 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
    * If *vp is a magic JS_OPTIMIZED_OUT value, this produces a plain object
    * of the form { optimizedOut: true }.
    *
-   * If *vp is a magic JS_OPTIMIZED_ARGUMENTS value signifying missing
+   * If *vp is a magic JS_MISSING_ARGUMENTS value signifying missing
    * arguments, this produces a plain object of the form { missingArguments:
    * true }.
    *
@@ -1138,10 +1162,10 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
    * unaccessible uninitialized binding, this produces a plain object of the
    * form { uninitialized: true }.
    */
-  MOZ_MUST_USE bool wrapDebuggeeValue(JSContext* cx, MutableHandleValue vp);
-  MOZ_MUST_USE bool wrapDebuggeeObject(JSContext* cx, HandleObject obj,
-                                       MutableHandleDebuggerObject result);
-  MOZ_MUST_USE bool wrapNullableDebuggeeObject(
+  [[nodiscard]] bool wrapDebuggeeValue(JSContext* cx, MutableHandleValue vp);
+  [[nodiscard]] bool wrapDebuggeeObject(JSContext* cx, HandleObject obj,
+                                        MutableHandleDebuggerObject result);
+  [[nodiscard]] bool wrapNullableDebuggeeObject(
       JSContext* cx, HandleObject obj, MutableHandleDebuggerObject result);
 
   /*
@@ -1171,10 +1195,10 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
    * debugger compartment--mirror symmetry. But compartment wrapping always
    * happens in the target compartment--rotational symmetry.)
    */
-  MOZ_MUST_USE bool unwrapDebuggeeValue(JSContext* cx, MutableHandleValue vp);
-  MOZ_MUST_USE bool unwrapDebuggeeObject(JSContext* cx,
-                                         MutableHandleObject obj);
-  MOZ_MUST_USE bool unwrapPropertyDescriptor(
+  [[nodiscard]] bool unwrapDebuggeeValue(JSContext* cx, MutableHandleValue vp);
+  [[nodiscard]] bool unwrapDebuggeeObject(JSContext* cx,
+                                          MutableHandleObject obj);
+  [[nodiscard]] bool unwrapPropertyDescriptor(
       JSContext* cx, HandleObject obj, MutableHandle<PropertyDescriptor> desc);
 
   /*
@@ -1184,14 +1208,14 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
    * `iter` points to, a new Frame object is created, and `iter`'s private
    * data is copied into it.
    */
-  MOZ_MUST_USE bool getFrame(JSContext* cx, const FrameIter& iter,
-                             MutableHandleValue vp);
-  MOZ_MUST_USE bool getFrame(JSContext* cx, MutableHandleDebuggerFrame result);
-  MOZ_MUST_USE bool getFrame(JSContext* cx, const FrameIter& iter,
-                             MutableHandleDebuggerFrame result);
-  MOZ_MUST_USE bool getFrame(JSContext* cx,
-                             Handle<AbstractGeneratorObject*> genObj,
-                             MutableHandleDebuggerFrame result);
+  [[nodiscard]] bool getFrame(JSContext* cx, const FrameIter& iter,
+                              MutableHandleValue vp);
+  [[nodiscard]] bool getFrame(JSContext* cx, MutableHandleDebuggerFrame result);
+  [[nodiscard]] bool getFrame(JSContext* cx, const FrameIter& iter,
+                              MutableHandleDebuggerFrame result);
+  [[nodiscard]] bool getFrame(JSContext* cx,
+                              Handle<AbstractGeneratorObject*> genObj,
+                              MutableHandleDebuggerFrame result);
 
   /*
    * Return the Debugger.Script object for |script|, or create a new one if
@@ -1233,6 +1257,22 @@ class Debugger : private mozilla::LinkedListElement<Debugger> {
   Debugger& operator=(const Debugger&) = delete;
 };
 
+// Specialize InternalBarrierMethods so we can have WeakHeapPtr<Debugger*>.
+template <>
+struct InternalBarrierMethods<Debugger*> {
+  static bool isMarkable(Debugger* dbg) { return dbg->toJSObject(); }
+
+  static void postBarrier(Debugger** vp, Debugger* prev, Debugger* next) {}
+
+  static void readBarrier(Debugger* dbg) {
+    InternalBarrierMethods<JSObject*>::readBarrier(dbg->toJSObject());
+  }
+
+#ifdef DEBUG
+  static void assertThingIsNotGray(Debugger* dbg) {}
+#endif
+};
+
 /**
  * This class exists for one specific reason. If a given Debugger object is in
  * a state where:
@@ -1523,12 +1563,12 @@ Breakpoint* Debugger::firstBreakpoint() const {
   return &(*breakpoints.begin());
 }
 
-const js::GCPtrNativeObject& Debugger::toJSObject() const {
+const js::HeapPtrNativeObject& Debugger::toJSObject() const {
   MOZ_ASSERT(object);
   return object;
 }
 
-js::GCPtrNativeObject& Debugger::toJSObjectRef() {
+js::HeapPtrNativeObject& Debugger::toJSObjectRef() {
   MOZ_ASSERT(object);
   return object;
 }
@@ -1546,11 +1586,12 @@ bool Debugger::observesGlobal(GlobalObject* global) const {
   return debuggees.has(debuggee);
 }
 
-MOZ_MUST_USE bool ReportObjectRequired(JSContext* cx);
+[[nodiscard]] bool ReportObjectRequired(JSContext* cx);
 
 JSObject* IdVectorToArray(JSContext* cx, Handle<IdVector> ids);
 bool IsInterpretedNonSelfHostedFunction(JSFunction* fun);
 JSScript* GetOrCreateFunctionScript(JSContext* cx, HandleFunction fun);
+ArrayObject* GetFunctionParameterNamesArray(JSContext* cx, HandleFunction fun);
 bool ValueToIdentifier(JSContext* cx, HandleValue v, MutableHandleId id);
 bool ValueToStableChars(JSContext* cx, const char* fnname, HandleValue value,
                         JS::AutoStableStringChars& stableChars);
@@ -1576,12 +1617,4 @@ bool ParseResumptionValue(JSContext* cx, HandleValue rval,
 
 } /* namespace js */
 
-namespace JS {
-
-template <>
-struct DeletePolicy<js::Debugger>
-    : public js::GCManagedDeletePolicy<js::Debugger> {};
-
-} /* namespace JS */
-
 #endif /* debugger_Debugger_h */
