@@ -11,6 +11,8 @@
 
 #if defined(XP_WIN)
 #  include "util/Windows.h"
+#elif defined(__wasi__)
+// Nothing.
 #elif defined(XP_UNIX) && !defined(XP_DARWIN)
 #  include <signal.h>
 #  include <sys/types.h>
@@ -35,8 +37,7 @@ namespace js {
 
 // Memory protection occurs at non-deterministic points when
 // recording/replaying.
-static mozilla::Atomic<bool, mozilla::SequentiallyConsistent,
-                       mozilla::recordreplay::Behavior::DontPreserve>
+static mozilla::Atomic<bool, mozilla::SequentiallyConsistent>
     sProtectedRegionsInit(false);
 
 /*
@@ -122,6 +123,8 @@ bool MemoryProtectionExceptionHandler::isDisabled() {
 #elif !defined(MOZ_DIAGNOSTIC_ASSERT_ENABLED)
   // Disable the exception handler for Beta and Release builds.
   return true;
+#elif defined(__wasi__)
+  return true;
 #else
   return false;
 #endif
@@ -233,6 +236,12 @@ void MemoryProtectionExceptionHandler::uninstall() {
   }
 }
 
+#elif defined(__wasi__)
+
+bool MemoryProtectionExceptionHandler::install() { return true; }
+
+void MemoryProtectionExceptionHandler::uninstall() {}
+
 #elif defined(XP_UNIX) && !defined(XP_DARWIN)
 
 static struct sigaction sPrevSEGVHandler = {};
@@ -530,6 +539,7 @@ static ExceptionHandlerState* sMachExceptionState = nullptr;
  * previously installed handler (which will likely terminate the process).
  */
 static void MachExceptionHandler() {
+  ThisThread::SetName("JS MachExceptionHandler");
   kern_return_t ret;
   MachExceptionParameters& current = sMachExceptionState->current;
   MachExceptionParameters& previous = sMachExceptionState->previous;
