@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
  * vim: set ts=8 sw=2 et tw=0 ft=c:
  *
  * This Source Code Form is subject to the terms of the Mozilla Public
@@ -8,14 +8,16 @@
 #ifndef frontend_ObjLiteral_h
 #define frontend_ObjLiteral_h
 
-#include "mozilla/EndianUtils.h"
+#include "mozilla/BloomFilter.h"  // mozilla::BitBloomFilter
 #include "mozilla/EnumSet.h"
 #include "mozilla/Span.h"
 
+#include "frontend/ParserAtom.h"  // ParserAtomsTable, TaggedParserAtomIndex
 #include "js/AllocPolicy.h"
 #include "js/GCPolicyAPI.h"
 #include "js/Value.h"
 #include "js/Vector.h"
+#include "util/EnumFlags.h"
 
 /*
  * [SMDOC] ObjLiteral (Object Literal) Handling
@@ -74,10 +76,6 @@
  *   with a defined set of properties. The given object is used as a
  *   *template*.
  *
- * - JSOp::NewObjectWithGroup (added as part of this ObjLiteral work), same as
- *   above but uses the ObjectGroup of the template object for the new object,
- *   rather than trying to apply a set of heuristics to choose a group.
- *
  * - JSOp::Object, with an object as argument, instructs the runtime to
  *   literally return the object argument as the result. This is thus only an
  *   "allocation" in the sense that the object was originally allocated when
@@ -85,19 +83,12 @@
  *   sure that the script, and this program point within the script, will run
  *   *once*. (See the `treatAsRunOnce` flag on JSScript.)
  *
- * Before we go further, we also define "singleton context" and "singleton
- * group". An operation occurs in a "singleton context", according to the
- * parser, if it will only ever execute once. In particular, this happens when
- * (i) the script is a "run-once" script, which is usually the case for e.g.
- * top-level scripts of web-pages (they run on page load, but no function or
- * handle wraps or refers to the script so it can't be invoked again),
- * and (ii) the operation itself is not within a loop or function in that
- * run-once script. "Singleton group", on the other hand, refers to an
- * ObjectGroup (used by Type Inference) that represents only one object, and
- * has a special flag set to mark it as such. Usually we want to give singleton
- * groups to object allocations that happen in a singleton context (because
- * there will only ever be one of the object), hence the connection between
- * these terms.
+ * An operation occurs in a "singleton context", according to the parser, if it
+ * will only ever execute once. In particular, this happens when (i) the script
+ * is a "run-once" script, which is usually the case for e.g. top-level scripts
+ * of web-pages (they run on page load, but no function or handle wraps or
+ * refers to the script so it can't be invoked again), and (ii) the operation
+ * itself is not within a loop or function in that run-once script.
  *
  * When we encounter an object literal, we decide which opcode to use, and we
  * construct the ObjLiteral and the bytecode using its result appropriately:
@@ -105,40 +96,25 @@
  * - If in a singleton context, and if we support the values, we use
  *   JSOp::Object and we build the ObjLiteral instructions with values.
  * - Otherwise, if we support the keys but not the values, or if we are not
- *   in a singleton context, we use JSOp::NewObject or JSOp::NewObjectWithGroup,
- *   depending on the "inner singleton" status (see below). In this case, the
- *   initial opcode only creates an object with empty values, so
- *   BytecodeEmitter then generates bytecode to set the values
- *   appropriately.
+ *   in a singleton context, we use JSOp::NewObject. In this case, the initial
+ *   opcode only creates an object with empty values, so BytecodeEmitter then
+ *   generates bytecode to set the values appropriately.
  * - Otherwise, we generate JSOp::NewInit and bytecode to add properties one at
  *   a time. This will always work, but is the slowest and least
  *   memory-efficient option.
- *
- * We need to take special care to ensure that the ObjectGroup of the resulting
- * object is chosen "correctly". Failure to do so can result in all sorts
- * of performance and/or memory regressions. In brief, we want to use a
- * singleton group whenever an object is allocated in a singleton context.
- * However, there is a special "inner singleton" context that deserves special
- * mention. When a program has a nested tree of objects, the old
- * (pre-ObjLiteral) world would perform a group lookup by shape (list of
- * property IDs) for all non-root objects, so in the following snippet, the
- * inner objects would share a group:
- *
- *     var list = [{a: 0}, {a: 1}];
- *     var obj = { first: {a: 0}, second: {a: 1} };
- *
- * In the generated bytecode, the inner objects are created first, then placed
- * in the relevant properties of the outer objects/arrays using
- * INITPROP/INITELEM. Thus to a naïve analysis, it appears that the inner
- * objects are singletons. But heuristically it is better if they are not. So
- * we pass down an `isInner` boolean while recursively traversing the
- * parse-node tree and generating bytecode. If we encounter an object literal
- * that is in singleton (run-once) context but also `isInner`, then we set
- * special flags to ensure its shape is looked up based on properties instead.
  */
 
 namespace js {
 
+class LifoAlloc;
+class JSONPrinter;
+
+namespace frontend {
+struct CompilationAtomCache;
+struct CompilationStencil;
+class StencilXDR;
+}  // namespace frontend
+
 // Object-literal instruction opcodes. An object literal is constructed by a
 // straight-line sequence of these ops, each adding one property to the
 // object.
@@ -159,31 +135,19 @@ enum class ObjLiteralOpcode : uint8_t {
 // (These become bitflags by wrapping with EnumSet below.)
 enum class ObjLiteralFlag : uint8_t {
   // If set, this object is an array.
-  Array = 1,
+  Array = 1 << 0,
 
-  // If set, the created object will be created with an object group either
-  // freshly allocated or determined by property names by calling
-  // `ObjectGroup::newPlainObject`.
-  SpecificGroup = 2,
-  // If set, the created object will be created with newType == SingletonObject
-  // rather than TenuredObject.
-  Singleton = 3,
-  // If set, the created array will be created as a COW array rather than a
-  // normal array.
-  ArrayCOW = 4,
+  // If set, this is an object literal in a singleton context and property
+  // values are included. See also JSOp::Object.
+  Singleton = 1 << 1,
 
-  // No values are provided; the object is meant as a template object.
-  NoValues = 5,
-
-  // This object is inside a top-level singleton, and so prior to ObjLiteral,
-  // would have been allocated at parse time, but is now allocated in bytecode.
-  // We do special things to get the right group on the template object; this
-  // flag indicates that if JSOp::NewObject copies the object, it should retain
-  // its group.
-  IsInnerSingleton = 6,
+  // If set, this object contains index property, or duplicate non-index
+  // property.
+  // This flag is valid only if Array flag isn't set.
+  HasIndexOrDuplicatePropName = 1 << 2,
 };
 
-using ObjLiteralFlags = mozilla::EnumSet<ObjLiteralFlag>;
+using ObjLiteralFlags = EnumFlags<ObjLiteralFlag>;
 
 inline bool ObjLiteralOpcodeHasValueArg(ObjLiteralOpcode op) {
   return op == ObjLiteralOpcode::ConstValue;
@@ -195,7 +159,7 @@ inline bool ObjLiteralOpcodeHasAtomArg(ObjLiteralOpcode op) {
 
 struct ObjLiteralReaderBase;
 
-// Property name (as an atom index) or an integer index.  Only used for
+// Property name (as TaggedParserAtomIndex) or an integer index.  Only used for
 // object-type literals; array literals do not require the index (the sequence
 // is always dense, with no holes, so the index is implicit). For the latter
 // case, we have a `None` placeholder.
@@ -220,8 +184,8 @@ struct ObjLiteralKey {
       : ObjLiteralKey(value, isArrayIndex ? ArrayIndex : AtomIndex) {}
   ObjLiteralKey(const ObjLiteralKey& other) = default;
 
-  static ObjLiteralKey fromPropName(uint32_t atomIndex) {
-    return ObjLiteralKey(atomIndex, false);
+  static ObjLiteralKey fromPropName(frontend::TaggedParserAtomIndex atomIndex) {
+    return ObjLiteralKey(atomIndex.rawData(), false);
   }
   static ObjLiteralKey fromArrayIndex(uint32_t index) {
     return ObjLiteralKey(index, true);
@@ -232,9 +196,9 @@ struct ObjLiteralKey {
   bool isAtomIndex() const { return type_ == AtomIndex; }
   bool isArrayIndex() const { return type_ == ArrayIndex; }
 
-  uint32_t getAtomIndex() const {
+  frontend::TaggedParserAtomIndex getAtomIndex() const {
     MOZ_ASSERT(isAtomIndex());
-    return value_;
+    return frontend::TaggedParserAtomIndex::fromRaw(value_);
   }
   uint32_t getArrayIndex() const {
     MOZ_ASSERT(isArrayIndex());
@@ -247,22 +211,34 @@ struct ObjLiteralKey {
 struct ObjLiteralWriterBase {
  protected:
   friend struct ObjLiteralReaderBase;  // for access to mask and shift.
-  static const uint32_t ATOM_INDEX_MASK = 0x007fffff;
+  static const uint32_t ATOM_INDEX_MASK = 0x7fffffff;
   // If set, the atom index field is an array index, not an atom index.
-  static const uint32_t INDEXED_PROP = 0x00800000;
-  static const int OP_SHIFT = 24;
-
- protected:
-  Vector<uint8_t, 64> code_;
+  static const uint32_t INDEXED_PROP = 0x80000000;
 
  public:
-  explicit ObjLiteralWriterBase(JSContext* cx) : code_(cx) {}
+  using CodeVector = Vector<uint8_t, 64, js::SystemAllocPolicy>;
+
+ protected:
+  CodeVector code_;
+
+ public:
+  ObjLiteralWriterBase() = default;
 
   uint32_t curOffset() const { return code_.length(); }
 
-  MOZ_MUST_USE bool prepareBytes(size_t len, uint8_t** p) {
+ private:
+  [[nodiscard]] bool pushByte(JSContext* cx, uint8_t data) {
+    if (!code_.append(data)) {
+      js::ReportOutOfMemory(cx);
+      return false;
+    }
+    return true;
+  }
+
+  [[nodiscard]] bool prepareBytes(JSContext* cx, size_t len, uint8_t** p) {
     size_t offset = code_.length();
     if (!code_.growByUninitialized(len)) {
+      js::ReportOutOfMemory(cx);
       return false;
     }
     *p = &code_[offset];
@@ -270,32 +246,33 @@ struct ObjLiteralWriterBase {
   }
 
   template <typename T>
-  MOZ_MUST_USE bool pushRawData(T data) {
+  [[nodiscard]] bool pushRawData(JSContext* cx, T data) {
     uint8_t* p = nullptr;
-    if (!prepareBytes(sizeof(T), &p)) {
+    if (!prepareBytes(cx, sizeof(T), &p)) {
       return false;
     }
-    mozilla::NativeEndian::copyAndSwapToLittleEndian(reinterpret_cast<void*>(p),
-                                                     &data, 1);
+    memcpy(p, &data, sizeof(T));
     return true;
   }
 
-  MOZ_MUST_USE bool pushOpAndName(ObjLiteralOpcode op, ObjLiteralKey key) {
-    uint32_t data = (key.rawIndex() & ATOM_INDEX_MASK) |
-                    (key.isArrayIndex() ? INDEXED_PROP : 0) |
-                    (static_cast<uint8_t>(op) << OP_SHIFT);
-    return pushRawData(data);
+ protected:
+  [[nodiscard]] bool pushOpAndName(JSContext* cx, ObjLiteralOpcode op,
+                                   ObjLiteralKey key) {
+    uint8_t opdata = static_cast<uint8_t>(op);
+    uint32_t data = key.rawIndex() | (key.isArrayIndex() ? INDEXED_PROP : 0);
+    return pushByte(cx, opdata) && pushRawData(cx, data);
   }
 
-  MOZ_MUST_USE bool pushValueArg(const JS::Value& value) {
+  [[nodiscard]] bool pushValueArg(JSContext* cx, const JS::Value& value) {
     MOZ_ASSERT(value.isNumber() || value.isNullOrUndefined() ||
                value.isBoolean());
     uint64_t data = value.asRawBits();
-    return pushRawData(data);
+    return pushRawData(cx, data);
   }
 
-  MOZ_MUST_USE bool pushAtomArg(uint32_t atomIndex) {
-    return pushRawData(atomIndex);
+  [[nodiscard]] bool pushAtomArg(JSContext* cx,
+                                 frontend::TaggedParserAtomIndex atomIndex) {
+    return pushRawData(cx, atomIndex.rawData());
   }
 };
 
@@ -306,64 +283,126 @@ struct ObjLiteralWriterBase {
 // within the writer.
 struct ObjLiteralWriter : private ObjLiteralWriterBase {
  public:
-  explicit ObjLiteralWriter(JSContext* cx)
-      : ObjLiteralWriterBase(cx), flags_() {}
+  ObjLiteralWriter() = default;
 
   void clear() { code_.clear(); }
 
+  using CodeVector = typename ObjLiteralWriterBase::CodeVector;
+
+  bool checkForDuplicatedNames(JSContext* cx);
   mozilla::Span<const uint8_t> getCode() const { return code_; }
   ObjLiteralFlags getFlags() const { return flags_; }
+  uint32_t getPropertyCount() const { return propertyCount_; }
 
   void beginObject(ObjLiteralFlags flags) { flags_ = flags; }
-  void setPropName(uint32_t propName) {
+  bool setPropName(JSContext* cx, frontend::ParserAtomsTable& parserAtoms,
+                   const frontend::TaggedParserAtomIndex propName) {
     // Only valid in object-mode.
-    MOZ_ASSERT(!flags_.contains(ObjLiteralFlag::Array));
-    MOZ_ASSERT(propName <= ATOM_INDEX_MASK);
+    setPropNameNoDuplicateCheck(parserAtoms, propName);
+
+    if (flags_.hasFlag(ObjLiteralFlag::HasIndexOrDuplicatePropName)) {
+      return true;
+    }
+
+    // OK to early return if we've already discovered a potential duplicate.
+    if (mightContainDuplicatePropertyNames_) {
+      return true;
+    }
+
+    // Check bloom filter for duplicate, and add if not already represented.
+    if (propNamesFilter_.mightContain(propName.rawData())) {
+      mightContainDuplicatePropertyNames_ = true;
+    } else {
+      propNamesFilter_.add(propName.rawData());
+    }
+    return true;
+  }
+  void setPropNameNoDuplicateCheck(
+      frontend::ParserAtomsTable& parserAtoms,
+      const frontend::TaggedParserAtomIndex propName) {
+    // Only valid in object-mode.
+    MOZ_ASSERT(!flags_.hasFlag(ObjLiteralFlag::Array));
+    parserAtoms.markUsedByStencil(propName);
     nextKey_ = ObjLiteralKey::fromPropName(propName);
   }
   void setPropIndex(uint32_t propIndex) {
     // Only valid in object-mode.
-    MOZ_ASSERT(!flags_.contains(ObjLiteralFlag::Array));
+    MOZ_ASSERT(!flags_.hasFlag(ObjLiteralFlag::Array));
     MOZ_ASSERT(propIndex <= ATOM_INDEX_MASK);
     nextKey_ = ObjLiteralKey::fromArrayIndex(propIndex);
+    flags_.setFlag(ObjLiteralFlag::HasIndexOrDuplicatePropName);
   }
   void beginDenseArrayElements() {
     // Only valid in array-mode.
-    MOZ_ASSERT(flags_.contains(ObjLiteralFlag::Array));
+    MOZ_ASSERT(flags_.hasFlag(ObjLiteralFlag::Array));
     // Dense array element sequences do not use the keys; the indices are
     // implicit.
     nextKey_ = ObjLiteralKey::none();
   }
 
-  MOZ_MUST_USE bool propWithConstNumericValue(const JS::Value& value) {
+  [[nodiscard]] bool propWithConstNumericValue(JSContext* cx,
+                                               const JS::Value& value) {
+    propertyCount_++;
     MOZ_ASSERT(value.isNumber());
-    return pushOpAndName(ObjLiteralOpcode::ConstValue, nextKey_) &&
-           pushValueArg(value);
+    return pushOpAndName(cx, ObjLiteralOpcode::ConstValue, nextKey_) &&
+           pushValueArg(cx, value);
   }
-  MOZ_MUST_USE bool propWithAtomValue(uint32_t value) {
-    return pushOpAndName(ObjLiteralOpcode::ConstAtom, nextKey_) &&
-           pushAtomArg(value);
+  [[nodiscard]] bool propWithAtomValue(
+      JSContext* cx, frontend::ParserAtomsTable& parserAtoms,
+      const frontend::TaggedParserAtomIndex value) {
+    propertyCount_++;
+    parserAtoms.markUsedByStencil(value);
+    return pushOpAndName(cx, ObjLiteralOpcode::ConstAtom, nextKey_) &&
+           pushAtomArg(cx, value);
   }
-  MOZ_MUST_USE bool propWithNullValue() {
-    return pushOpAndName(ObjLiteralOpcode::Null, nextKey_);
+  [[nodiscard]] bool propWithNullValue(JSContext* cx) {
+    propertyCount_++;
+    return pushOpAndName(cx, ObjLiteralOpcode::Null, nextKey_);
   }
-  MOZ_MUST_USE bool propWithUndefinedValue() {
-    return pushOpAndName(ObjLiteralOpcode::Undefined, nextKey_);
+  [[nodiscard]] bool propWithUndefinedValue(JSContext* cx) {
+    propertyCount_++;
+    return pushOpAndName(cx, ObjLiteralOpcode::Undefined, nextKey_);
   }
-  MOZ_MUST_USE bool propWithTrueValue() {
-    return pushOpAndName(ObjLiteralOpcode::True, nextKey_);
+  [[nodiscard]] bool propWithTrueValue(JSContext* cx) {
+    propertyCount_++;
+    return pushOpAndName(cx, ObjLiteralOpcode::True, nextKey_);
   }
-  MOZ_MUST_USE bool propWithFalseValue() {
-    return pushOpAndName(ObjLiteralOpcode::False, nextKey_);
+  [[nodiscard]] bool propWithFalseValue(JSContext* cx) {
+    propertyCount_++;
+    return pushOpAndName(cx, ObjLiteralOpcode::False, nextKey_);
   }
 
   static bool arrayIndexInRange(int32_t i) {
     return i >= 0 && static_cast<uint32_t>(i) <= ATOM_INDEX_MASK;
   }
 
+#if defined(DEBUG) || defined(JS_JITSPEW)
+  void dump() const;
+  void dump(JSONPrinter& json,
+            const frontend::CompilationStencil* stencil) const;
+  void dumpFields(JSONPrinter& json,
+                  const frontend::CompilationStencil* stencil) const;
+#endif
+
  private:
+  // Set to true if we've found possible duplicate names while building.
+  // This field is placed next to `flags_` field, to reduce padding.
+  bool mightContainDuplicatePropertyNames_ = false;
+
   ObjLiteralFlags flags_;
   ObjLiteralKey nextKey_;
+  uint32_t propertyCount_ = 0;
+
+  // Duplicate property names detection is performed in the following way:
+  //   * while emitting code, add each property names with
+  //     `propNamesFilter_`
+  //   * if possible duplicate property name is detected, set
+  //     `mightContainDuplicatePropertyNames_` to true
+  //   * in `checkForDuplicatedNames` method,
+  //     if `mightContainDuplicatePropertyNames_` is true,
+  //     check the duplicate property names with `HashSet`, and if it exists,
+  //     set HasIndexOrDuplicatePropName flag.
+  mozilla::BitBloomFilter<12, frontend::TaggedParserAtomIndex> propNamesFilter_;
 };
 
 struct ObjLiteralReaderBase {
@@ -371,7 +410,16 @@ struct ObjLiteralReaderBase {
   mozilla::Span<const uint8_t> data_;
   size_t cursor_;
 
-  MOZ_MUST_USE bool readBytes(size_t size, const uint8_t** p) {
+  [[nodiscard]] bool readByte(uint8_t* b) {
+    if (cursor_ + 1 > data_.Length()) {
+      return false;
+    }
+    *b = *data_.From(cursor_).data();
+    cursor_ += 1;
+    return true;
+  }
+
+  [[nodiscard]] bool readBytes(size_t size, const uint8_t** p) {
     if (cursor_ + size > data_.Length()) {
       return false;
     }
@@ -381,13 +429,12 @@ struct ObjLiteralReaderBase {
   }
 
   template <typename T>
-  MOZ_MUST_USE bool readRawData(T* data) {
+  [[nodiscard]] bool readRawData(T* data) {
     const uint8_t* p = nullptr;
     if (!readBytes(sizeof(T), &p)) {
       return false;
     }
-    mozilla::NativeEndian::copyAndSwapFromLittleEndian(
-        data, reinterpret_cast<const void*>(p), 1);
+    memcpy(data, p, sizeof(T));
     return true;
   }
 
@@ -395,24 +442,27 @@ struct ObjLiteralReaderBase {
   explicit ObjLiteralReaderBase(mozilla::Span<const uint8_t> data)
       : data_(data), cursor_(0) {}
 
-  MOZ_MUST_USE bool readOpAndKey(ObjLiteralOpcode* op, ObjLiteralKey* key) {
-    uint32_t data;
-    if (!readRawData(&data)) {
+  [[nodiscard]] bool readOpAndKey(ObjLiteralOpcode* op, ObjLiteralKey* key) {
+    uint8_t opbyte;
+    if (!readByte(&opbyte)) {
       return false;
     }
-    uint8_t opbyte =
-        static_cast<uint8_t>(data >> ObjLiteralWriterBase::OP_SHIFT);
     if (MOZ_UNLIKELY(opbyte > static_cast<uint8_t>(ObjLiteralOpcode::MAX))) {
       return false;
     }
     *op = static_cast<ObjLiteralOpcode>(opbyte);
+
+    uint32_t data;
+    if (!readRawData(&data)) {
+      return false;
+    }
     bool isArray = data & ObjLiteralWriterBase::INDEXED_PROP;
-    uint32_t rawIndex = data & ObjLiteralWriterBase::ATOM_INDEX_MASK;
+    uint32_t rawIndex = data & ~ObjLiteralWriterBase::INDEXED_PROP;
     *key = ObjLiteralKey(rawIndex, isArray);
     return true;
   }
 
-  MOZ_MUST_USE bool readValueArg(JS::Value* value) {
+  [[nodiscard]] bool readValueArg(JS::Value* value) {
     uint64_t data;
     if (!readRawData(&data)) {
       return false;
@@ -421,9 +471,11 @@ struct ObjLiteralReaderBase {
     return true;
   }
 
-  MOZ_MUST_USE bool readAtomArg(uint32_t* atomIndex) {
-    return readRawData(atomIndex);
+  [[nodiscard]] bool readAtomArg(frontend::TaggedParserAtomIndex* atomIndex) {
+    return readRawData(atomIndex->rawDataRef());
   }
+
+  size_t cursor() const { return cursor_; }
 };
 
 // A single object-literal instruction, creating one property on an object.
@@ -435,7 +487,7 @@ struct ObjLiteralInsn {
     explicit Arg(uint64_t raw_) : raw(raw_) {}
 
     JS::Value constValue;
-    uint32_t atomIndex;
+    frontend::TaggedParserAtomIndex atomIndex;
     uint64_t raw;
   } arg_;
 
@@ -452,7 +504,8 @@ struct ObjLiteralInsn {
     MOZ_ASSERT(!hasAtomIndex());
     arg_.constValue = value;
   }
-  ObjLiteralInsn(ObjLiteralOpcode op, ObjLiteralKey key, uint32_t atomIndex)
+  ObjLiteralInsn(ObjLiteralOpcode op, ObjLiteralKey key,
+                 frontend::TaggedParserAtomIndex atomIndex)
       : op_(op), key_(key), arg_(0) {
     MOZ_ASSERT(!hasConstValue());
     MOZ_ASSERT(hasAtomIndex());
@@ -495,7 +548,7 @@ struct ObjLiteralInsn {
     MOZ_ASSERT(hasConstValue());
     return arg_.constValue;
   }
-  uint32_t getAtomIndex() const {
+  frontend::TaggedParserAtomIndex getAtomIndex() const {
     MOZ_ASSERT(isValid());
     MOZ_ASSERT(hasAtomIndex());
     return arg_.atomIndex;
@@ -509,7 +562,7 @@ struct ObjLiteralReader : private ObjLiteralReaderBase {
   explicit ObjLiteralReader(mozilla::Span<const uint8_t> data)
       : ObjLiteralReaderBase(data) {}
 
-  MOZ_MUST_USE bool readInsn(ObjLiteralInsn* insn) {
+  [[nodiscard]] bool readInsn(ObjLiteralInsn* insn) {
     ObjLiteralOpcode op;
     ObjLiteralKey key;
     if (!readOpAndKey(&op, &key)) {
@@ -524,7 +577,7 @@ struct ObjLiteralReader : private ObjLiteralReaderBase {
       return true;
     }
     if (ObjLiteralOpcodeHasAtomArg(op)) {
-      uint32_t atomIndex;
+      frontend::TaggedParserAtomIndex atomIndex;
       if (!readAtomArg(&atomIndex)) {
         return false;
       }
@@ -536,34 +589,105 @@ struct ObjLiteralReader : private ObjLiteralReaderBase {
   }
 };
 
-typedef Vector<JSAtom*, 4> ObjLiteralAtomVector;
-
-JSObject* InterpretObjLiteral(JSContext* cx, const ObjLiteralAtomVector& atoms,
-                              const mozilla::Span<const uint8_t> insns,
-                              ObjLiteralFlags flags);
-
-inline JSObject* InterpretObjLiteral(JSContext* cx,
-                                     const ObjLiteralAtomVector& atoms,
-                                     const ObjLiteralWriter& writer) {
-  return InterpretObjLiteral(cx, atoms, writer.getCode(), writer.getFlags());
-}
-
-class ObjLiteralCreationData {
- private:
-  ObjLiteralWriter writer_;
-  ObjLiteralAtomVector atoms_;
+// A class to modify the code, while keeping the structure.
+struct ObjLiteralModifier : private ObjLiteralReaderBase {
+  mozilla::Span<uint8_t> mutableData_;
 
  public:
-  explicit ObjLiteralCreationData(JSContext* cx) : writer_(cx), atoms_(cx) {}
+  explicit ObjLiteralModifier(mozilla::Span<uint8_t> data)
+      : ObjLiteralReaderBase(data), mutableData_(data) {}
 
-  ObjLiteralWriter& writer() { return writer_; }
-
-  bool addAtom(JSAtom* atom, uint32_t* index) {
-    *index = atoms_.length();
-    return atoms_.append(atom);
+ private:
+  // Map `atom` with `map`, and write to `atomCursor` of `mutableData_`.
+  template <typename MapT>
+  void mapOneAtom(MapT map, frontend::TaggedParserAtomIndex atom,
+                  size_t atomCursor) {
+    auto atomIndex = map(atom);
+    memcpy(mutableData_.data() + atomCursor, atomIndex.rawDataRef(),
+           sizeof(frontend::TaggedParserAtomIndex));
   }
 
-  JSObject* create(JSContext* cx) const;
+  // Map atoms in single instruction.
+  // Return true if it successfully maps.
+  // Return false if there's no more instruction.
+  template <typename MapT>
+  bool mapInsnAtom(MapT map) {
+    ObjLiteralOpcode op;
+    ObjLiteralKey key;
+
+    size_t opCursor = cursor();
+    if (!readOpAndKey(&op, &key)) {
+      return false;
+    }
+    if (key.isAtomIndex()) {
+      static constexpr size_t OpLength = 1;
+      size_t atomCursor = opCursor + OpLength;
+      mapOneAtom(map, key.getAtomIndex(), atomCursor);
+    }
+
+    if (ObjLiteralOpcodeHasValueArg(op)) {
+      JS::Value value;
+      if (!readValueArg(&value)) {
+        return false;
+      }
+    } else if (ObjLiteralOpcodeHasAtomArg(op)) {
+      size_t atomCursor = cursor();
+
+      frontend::TaggedParserAtomIndex atomIndex;
+      if (!readAtomArg(&atomIndex)) {
+        return false;
+      }
+
+      mapOneAtom(map, atomIndex, atomCursor);
+    }
+
+    return true;
+  }
+
+ public:
+  // Map TaggedParserAtomIndex inside the code in place, with given function.
+  template <typename MapT>
+  void mapAtom(MapT map) {
+    while (mapInsnAtom(map)) {
+    }
+  }
+};
+
+class ObjLiteralStencil {
+  friend class frontend::StencilXDR;
+
+  mozilla::Span<uint8_t> code_;
+  ObjLiteralFlags flags_;
+  uint32_t propertyCount_ = 0;
+
+ public:
+  ObjLiteralStencil() = default;
+
+  ObjLiteralStencil(uint8_t* code, size_t length, const ObjLiteralFlags& flags,
+                    uint32_t propertyCount)
+      : code_(mozilla::Span(code, length)),
+        flags_(flags),
+        propertyCount_(propertyCount) {}
+
+  JSObject* create(JSContext* cx,
+                   const frontend::CompilationAtomCache& atomCache) const;
+
+  mozilla::Span<const uint8_t> code() const { return code_; }
+  ObjLiteralFlags flags() const { return flags_; }
+  uint32_t propertyCount() const { return propertyCount_; }
+
+#ifdef DEBUG
+  bool isContainedIn(const LifoAlloc& alloc) const;
+#endif
+
+#if defined(DEBUG) || defined(JS_JITSPEW)
+  void dump() const;
+  void dump(JSONPrinter& json,
+            const frontend::CompilationStencil* stencil) const;
+  void dumpFields(JSONPrinter& json,
+                  const frontend::CompilationStencil* stencil) const;
+
+#endif
 };
 
 }  // namespace js
