@@ -17,7 +17,7 @@ namespace frontend {
 template <typename Tag>
 struct TypedIndex {
   TypedIndex() = default;
-  explicit TypedIndex(uint32_t index) : index(index){};
+  constexpr explicit TypedIndex(uint32_t index) : index(index){};
 
   uint32_t index = 0;
 
@@ -28,6 +28,11 @@ struct TypedIndex {
     index = idx;
     return *this;
   }
+
+  bool operator<(TypedIndex other) { return index < other.index; }
+  bool operator<=(TypedIndex other) { return index <= other.index; }
+  bool operator>(TypedIndex other) { return index > other.index; }
+  bool operator>=(TypedIndex other) { return index >= other.index; }
 };
 
 }  // namespace frontend
