@@ -5,17 +5,17 @@
 offThreadCompileScript("Error()");
 assertEq(!!runOffThreadScript().stack.match(/^@<string>:1:1\n/), true);
 
- offThreadCompileScript("Error()", { fileName: "candelabra", lineNumber: 6502 });
+offThreadCompileScript("Error()", { fileName: "candelabra", lineNumber: 6502 });
 assertEq(!!runOffThreadScript().stack.match(/^@candelabra:6502:1\n/), true);
 
 var element = {};
-offThreadCompileScript("Error()", { element }); // shouldn't crash
-runOffThreadScript();
+var job = offThreadCompileScript("Error()"); // shouldn't crash
+runOffThreadScript(job, { element });
 
 var elementAttributeName = "molybdenum";
 elementAttributeName +=
   elementAttributeName + elementAttributeName + elementAttributeName;
-offThreadCompileScript("Error()", {
+job = offThreadCompileScript("Error()"); // shouldn't crash
+runOffThreadScript(job, {
   elementAttributeName,
-}); // shouldn't crash
-runOffThreadScript();
+});
