@@ -19,7 +19,7 @@ runOffThreadScript();
 
 var msg = getLastWarning().message;
 assertEq(msg === "asm.js type error: Disabled by debugger" ||
-         msg === "asm.js type error: Disabled by lack of a JIT compiler" ||
+         msg === "asm.js type error: Disabled because no suitable wasm compiler is available" ||
          msg === "asm.js type error: Disabled by 'asmjs' runtime option" ||
          msg === "asm.js type error: Disabled by lack of compiler support",
          true);
