@@ -1,9 +1,5 @@
-let moduleRepo = {};
-setModuleResolveHook(function(module, specifier) {
-        return moduleRepo[specifier];
-});
 setJitCompilerOption("ion.warmup.trigger", 50);
 s = "";
 for (i = 0; i < 1024; i++) s += "export let e" + i + "\n";
-moduleRepo['a'] = parseModule(s);
+registerModule('a', parseModule(s));
 parseModule("import * as ns from 'a'").declarationInstantiation();
