@@ -14,7 +14,7 @@ function loadModuleSrc(type, ext, offset, align, drop = false) {
           (local.get 0)
          )
          ${maybeDrop}
-       ) (export "" 0))`;
+       ) (export "" (func 0)))`;
 }
 function loadModule(type, ext, offset, align, drop = false) {
     return wasmEvalText(loadModuleSrc(type, ext, offset, align, drop)).exports[""];
@@ -33,14 +33,14 @@ function storeModuleSrc(type, ext, offset, align) {
           (local.get 0)
           (local.get 1)
          )
-       ) (export "store" 0)
+       ) (export "store" (func 0))
        (func $load (param i32) (result ${type})
         (${type}.load${load_ext}
          offset=${offset}
          ${align != 0 ? 'align=' + align : ''}
          (local.get 0)
         )
-       ) (export "load" 1))`;
+       ) (export "load" (func 1)))`;
 }
 function storeModule(type, ext, offset, align) {
     return wasmEvalText(storeModuleSrc(type, ext, offset, align)).exports;
@@ -59,14 +59,14 @@ function storeModuleCstSrc(type, ext, offset, align, value) {
           (local.get 0)
           (${type}.const ${value})
          )
-       ) (export "store" 0)
+       ) (export "store" (func 0))
        (func $load (param i32) (result ${type})
         (${type}.load${load_ext}
          offset=${offset}
          ${align != 0 ? 'align=' + align : ''}
          (local.get 0)
         )
-       ) (export "load" 1))`;
+       ) (export "load" (func 1)))`;
 }
 function storeModuleCst(type, ext, offset, align, value) {
     return wasmEvalText(storeModuleCstSrc(type, ext, offset, align, value)).exports;
@@ -113,7 +113,7 @@ function testStore(type, ext, base, offset, align, value) {
 function testStoreOOB(type, ext, base, offset, align, value) {
     if (type === 'i64') {
         assertErrorMessage(() => wasmAssert(
-            storeModuleSrc(type, ext, offset, align, value),
+            storeModuleSrc(type, ext, offset, align),
             [{type, func: '$store', args: [`i32.const ${base}`, `i64.const ${value}`]}]
         ), RuntimeError, /index out of bounds/);
     } else {
@@ -122,11 +122,11 @@ function testStoreOOB(type, ext, base, offset, align, value) {
 }
 
 function badLoadModule(type, ext) {
-    wasmFailValidateText( `(module (func (param i32) (${type}.load${ext} (local.get 0))) (export "" 0))`, /can't touch memory/);
+    wasmFailValidateText( `(module (func (param i32) (${type}.load${ext} (local.get 0))) (export "" (func 0)))`, /(can't touch memory)|(unknown memory 0)/);
 }
 
 function badStoreModule(type, ext) {
-    wasmFailValidateText(`(module (func (param i32) (${type}.store${ext} (local.get 0) (${type}.const 0))) (export "" 0))`, /can't touch memory/);
+    wasmFailValidateText(`(module (func (param i32) (${type}.store${ext} (local.get 0) (${type}.const 0))) (export "" (func 0)))`, /(can't touch memory)|(unknown memory 0)/);
 }
 
 // Can't touch memory.
@@ -266,7 +266,7 @@ for (var foldOffsets = 0; foldOffsets <= 1; foldOffsets++) {
         }
 
         // Ensure wrapping doesn't apply.
-        offset = 0x7fffffff; // maximum allowed offset that doesn't always throw.
+        offset = 0x7fffffff;
         for (let index of [0, 1, 2, 3, 0x7fffffff, 0x80000000, 0x80000001]) {
             if (align < 2) {
                 testLoadOOB('i32', '8_s', index, offset, align);
@@ -309,7 +309,7 @@ for (var foldOffsets = 0; foldOffsets <= 1; foldOffsets++) {
               (memory 1)
               (data (i32.const 0) "\\00\\01\\02\\03\\04\\05\\06\\07\\08\\09\\0a\\0b\\0c\\0d\\0e\\0f")
               (data (i32.const 16) "\\f0\\f1\\f2\\f3\\f4\\f5\\f6\\f7\\f8\\f9\\fa\\fb\\fc\\fd\\fe\\ff")
-              (func (param i32) (local i32 i32 i32 i32 f32 f64) (result i32)
+              (func (param i32) (result i32) (local i32 i32 i32 i32 f32 f64)
                (local.set 1 (i32.load8_s offset=4 (local.get 0)))
                (local.set 2 (i32.load16_s (local.get 1)))
                (i32.store8 offset=4 (local.get 0) (local.get 1))
@@ -337,7 +337,7 @@ for (var foldOffsets = 0; foldOffsets <= 1; foldOffsets++) {
                  )
                 )
                )
-              ) (export "" 0))`
+              ) (export "" (func 0)))`
             ).exports[""](1), 50464523);
         }
 
@@ -456,3 +456,17 @@ setJitCompilerOption('wasm.fold-offsets', 1);
                        WebAssembly.CompileError,
                        /memory index must be zero/);
 }
+
+// Misc syntax for data.
+
+// When memory index is present it must be zero, and the offset must be present too;
+// but it's OK for there to be neither
+new WebAssembly.Module(wasmTextToBinary(`(module (memory 1) (data 0 (i32.const 0) ""))`));
+new WebAssembly.Module(wasmTextToBinary(`(module (memory 1) (data 0 (offset (i32.const 0)) ""))`));
+new WebAssembly.Module(wasmTextToBinary(`(module (memory 1) (data ""))`));
+assertErrorMessage(() => new WebAssembly.Module(wasmTextToBinary(`(module (memory 1) (data 0 ""))`)),
+                   SyntaxError,
+                   /wasm text error/);
+assertErrorMessage(() => new WebAssembly.Module(wasmTextToBinary(`(module (memory 1) (data 1 (i32.const 0) ""))`)),
+                   WebAssembly.CompileError,
+                   /memory index must be zero/);
