@@ -5,6 +5,6 @@ let t = cacheEntry("function f() { function g() { }; return 3; };");
 evaluate(t, { sourceIsLazy: true, saveIncrementalBytecode: true });
 for (var i = 1; i < 20; ++i) {
     oomAtAllocation(i, THREAD_TYPE_PARSE);
-    offThreadDecodeScript(t);
+    offThreadDecodeScript(t, { sourceIsLazy: true });
     gc();
 }
