@@ -8,6 +8,7 @@
 #define jit_IonIC_h
 
 #include "jit/CacheIR.h"
+#include "jit/shared/Assembler-shared.h"
 
 namespace js {
 namespace jit {
@@ -27,6 +28,12 @@ class IonICStub {
   // Info about this stub.
   CacheIRStubInfo* stubInfo_;
 
+#ifndef JS_64BIT
+ protected:  // Silence Clang warning about unused private fields.
+  // Ensure stub data is 8-byte aligned on 32-bit.
+  uintptr_t padding_ = 0;
+#endif
+
  public:
   IonICStub(uint8_t* fallbackCode, CacheIRStubInfo* stubInfo)
       : nextCodeRaw_(fallbackCode), next_(nullptr), stubInfo_(stubInfo) {}
@@ -38,11 +45,11 @@ class IonICStub {
 
   uint8_t* stubDataStart();
 
-  void setNext(IonICStub* next, JitCode* nextCode) {
+  void setNext(IonICStub* next, uint8_t* nextCodeRaw) {
     MOZ_ASSERT(!next_);
-    MOZ_ASSERT(next && nextCode);
+    MOZ_ASSERT(next && nextCodeRaw);
     next_ = next;
-    nextCodeRaw_ = nextCode->raw();
+    nextCodeRaw_ = nextCodeRaw;
   }
 
   // Null out pointers when we unlink stubs, to ensure we never use
@@ -61,11 +68,14 @@ class IonGetNameIC;
 class IonBindNameIC;
 class IonGetIteratorIC;
 class IonHasOwnIC;
+class IonCheckPrivateFieldIC;
 class IonInIC;
 class IonInstanceOfIC;
 class IonCompareIC;
 class IonUnaryArithIC;
 class IonBinaryArithIC;
+class IonToPropertyKeyIC;
+class IonOptimizeSpreadCallIC;
 
 class IonIC {
   // This either points at the OOL path for the fallback path, or the code for
@@ -75,30 +85,30 @@ class IonIC {
   // The first optimized stub, or nullptr.
   IonICStub* firstStub_;
 
-  // The address stubs should jump to when done.
-  CodeLocationLabel rejoinLabel_;
-
-  // The OOL path that calls the IC's update function.
-  CodeLocationLabel fallbackLabel_;
-
-  // Location of this IC, nullptr for idempotent caches.
+  // Location of this IC.
   JSScript* script_;
   jsbytecode* pc_;
 
+  // The offset of the rejoin location in the IonScript's code (stubs jump to
+  // this location).
+  uint32_t rejoinOffset_;
+
+  // The offset of the OOL path in the IonScript's code that calls the IC's
+  // update function.
+  uint32_t fallbackOffset_;
+
   CacheKind kind_;
-  bool idempotent_ : 1;
   ICState state_;
 
  protected:
   explicit IonIC(CacheKind kind)
       : codeRaw_(nullptr),
         firstStub_(nullptr),
-        rejoinLabel_(),
-        fallbackLabel_(),
         script_(nullptr),
         pc_(nullptr),
+        rejoinOffset_(0),
+        fallbackOffset_(0),
         kind_(kind),
-        idempotent_(false),
         state_() {}
 
   void attachStub(IonICStub* newStub, JitCode* code);
@@ -120,26 +130,26 @@ class IonIC {
     return pc_;
   }
 
-  CodeLocationLabel rejoinLabel() const { return rejoinLabel_; }
-
   // Discard all stubs.
-  void discardStubs(Zone* zone);
+  void discardStubs(Zone* zone, IonScript* ionScript);
 
   // Discard all stubs and reset the ICState.
-  void reset(Zone* zone);
+  void reset(Zone* zone, IonScript* ionScript);
 
   ICState& state() { return state_; }
 
   CacheKind kind() const { return kind_; }
   uint8_t** codeRawPtr() { return &codeRaw_; }
 
-  bool idempotent() const { return idempotent_; }
-  void setIdempotent() { idempotent_ = true; }
-
-  void setFallbackLabel(CodeOffset fallbackLabel) {
-    fallbackLabel_ = fallbackLabel;
+  void setFallbackOffset(CodeOffset offset) {
+    fallbackOffset_ = offset.offset();
   }
-  void setRejoinLabel(CodeOffset rejoinLabel) { rejoinLabel_ = rejoinLabel; }
+  void setRejoinOffset(CodeOffset offset) { rejoinOffset_ = offset.offset(); }
+
+  void resetCodeRaw(IonScript* ionScript);
+
+  uint8_t* fallbackAddr(IonScript* ionScript) const;
+  uint8_t* rejoinAddr(IonScript* ionScript) const;
 
   IonGetPropertyIC* asGetPropertyIC() {
     MOZ_ASSERT(kind_ == CacheKind::GetProp || kind_ == CacheKind::GetElem);
@@ -166,10 +176,18 @@ class IonIC {
     MOZ_ASSERT(kind_ == CacheKind::GetIterator);
     return (IonGetIteratorIC*)this;
   }
+  IonOptimizeSpreadCallIC* asOptimizeSpreadCallIC() {
+    MOZ_ASSERT(kind_ == CacheKind::OptimizeSpreadCall);
+    return (IonOptimizeSpreadCallIC*)this;
+  }
   IonHasOwnIC* asHasOwnIC() {
     MOZ_ASSERT(kind_ == CacheKind::HasOwn);
     return (IonHasOwnIC*)this;
   }
+  IonCheckPrivateFieldIC* asCheckPrivateFieldIC() {
+    MOZ_ASSERT(kind_ == CacheKind::CheckPrivateField);
+    return (IonCheckPrivateFieldIC*)this;
+  }
   IonInIC* asInIC() {
     MOZ_ASSERT(kind_ == CacheKind::In);
     return (IonInIC*)this;
@@ -190,18 +208,19 @@ class IonIC {
     MOZ_ASSERT(kind_ == CacheKind::BinaryArith);
     return (IonBinaryArithIC*)this;
   }
-
-  void updateBaseAddress(JitCode* code);
+  IonToPropertyKeyIC* asToPropertyKeyIC() {
+    MOZ_ASSERT(kind_ == CacheKind::ToPropertyKey);
+    return (IonToPropertyKeyIC*)this;
+  }
 
   // Returns the Register to use as scratch when entering IC stubs. This
   // should either be an output register or a temp.
   Register scratchRegisterForEntryJump();
 
-  void trace(JSTracer* trc);
+  void trace(JSTracer* trc, IonScript* ionScript);
 
   void attachCacheIRStub(JSContext* cx, const CacheIRWriter& writer,
-                         CacheKind kind, IonScript* ionScript, bool* attached,
-                         const PropertyTypeCheckInfo* typeCheckInfo = nullptr);
+                         CacheKind kind, IonScript* ionScript, bool* attached);
 };
 
 class IonGetPropertyIC : public IonIC {
@@ -210,40 +229,26 @@ class IonGetPropertyIC : public IonIC {
 
   TypedOrValueRegister value_;
   ConstantOrRegister id_;
-  TypedOrValueRegister output_;
-  Register maybeTemp_;  // Might be InvalidReg.
-
-  GetPropertyResultFlags resultFlags_;
+  ValueOperand output_;
 
  public:
   IonGetPropertyIC(CacheKind kind, LiveRegisterSet liveRegs,
                    TypedOrValueRegister value, const ConstantOrRegister& id,
-                   TypedOrValueRegister output, Register maybeTemp,
-                   GetPropertyResultFlags resultFlags)
+                   ValueOperand output)
       : IonIC(kind),
         liveRegs_(liveRegs),
         value_(value),
         id_(id),
-        output_(output),
-        maybeTemp_(maybeTemp),
-        resultFlags_(resultFlags) {}
+        output_(output) {}
 
   TypedOrValueRegister value() const { return value_; }
   ConstantOrRegister id() const { return id_; }
-  TypedOrValueRegister output() const { return output_; }
-  Register maybeTemp() const { return maybeTemp_; }
+  ValueOperand output() const { return output_; }
   LiveRegisterSet liveRegs() const { return liveRegs_; }
-  GetPropertyResultFlags resultFlags() const { return resultFlags_; }
-  bool monitoredResult() const {
-    return resultFlags_ & GetPropertyResultFlags::Monitored;
-  }
-  bool allowDoubleResult() const {
-    return resultFlags_ & GetPropertyResultFlags::AllowDouble;
-  }
 
-  static MOZ_MUST_USE bool update(JSContext* cx, HandleScript outerScript,
-                                  IonGetPropertyIC* ic, HandleValue val,
-                                  HandleValue idVal, MutableHandleValue res);
+  [[nodiscard]] static bool update(JSContext* cx, HandleScript outerScript,
+                                   IonGetPropertyIC* ic, HandleValue val,
+                                   HandleValue idVal, MutableHandleValue res);
 };
 
 class IonGetPropSuperIC : public IonIC {
@@ -252,12 +257,12 @@ class IonGetPropSuperIC : public IonIC {
   Register object_;
   TypedOrValueRegister receiver_;
   ConstantOrRegister id_;
-  TypedOrValueRegister output_;
+  ValueOperand output_;
 
  public:
   IonGetPropSuperIC(CacheKind kind, LiveRegisterSet liveRegs, Register object,
                     TypedOrValueRegister receiver, const ConstantOrRegister& id,
-                    TypedOrValueRegister output)
+                    ValueOperand output)
       : IonIC(kind),
         liveRegs_(liveRegs),
         object_(object),
@@ -268,13 +273,13 @@ class IonGetPropSuperIC : public IonIC {
   Register object() const { return object_; }
   TypedOrValueRegister receiver() const { return receiver_; }
   ConstantOrRegister id() const { return id_; }
-  TypedOrValueRegister output() const { return output_; }
+  ValueOperand output() const { return output_; }
   LiveRegisterSet liveRegs() const { return liveRegs_; }
 
-  static MOZ_MUST_USE bool update(JSContext* cx, HandleScript outerScript,
-                                  IonGetPropSuperIC* ic, HandleObject obj,
-                                  HandleValue receiver, HandleValue idVal,
-                                  MutableHandleValue res);
+  [[nodiscard]] static bool update(JSContext* cx, HandleScript outerScript,
+                                   IonGetPropSuperIC* ic, HandleObject obj,
+                                   HandleValue receiver, HandleValue idVal,
+                                   MutableHandleValue res);
 };
 
 class IonSetPropertyIC : public IonIC {
@@ -282,34 +287,21 @@ class IonSetPropertyIC : public IonIC {
 
   Register object_;
   Register temp_;
-  FloatRegister maybeTempDouble_;
-  FloatRegister maybeTempFloat32_;
   ConstantOrRegister id_;
   ConstantOrRegister rhs_;
   bool strict_ : 1;
-  bool needsPostBarrier_ : 1;
-  bool needsTypeBarrier_ : 1;
-  bool guardHoles_ : 1;
 
  public:
   IonSetPropertyIC(CacheKind kind, LiveRegisterSet liveRegs, Register object,
-                   Register temp, FloatRegister maybeTempDouble,
-                   FloatRegister maybeTempFloat32, const ConstantOrRegister& id,
-                   const ConstantOrRegister& rhs, bool strict,
-                   bool needsPostBarrier, bool needsTypeBarrier,
-                   bool guardHoles)
+                   Register temp, const ConstantOrRegister& id,
+                   const ConstantOrRegister& rhs, bool strict)
       : IonIC(kind),
         liveRegs_(liveRegs),
         object_(object),
         temp_(temp),
-        maybeTempDouble_(maybeTempDouble),
-        maybeTempFloat32_(maybeTempFloat32),
         id_(id),
         rhs_(rhs),
-        strict_(strict),
-        needsPostBarrier_(needsPostBarrier),
-        needsTypeBarrier_(needsTypeBarrier),
-        guardHoles_(guardHoles) {}
+        strict_(strict) {}
 
   LiveRegisterSet liveRegs() const { return liveRegs_; }
   Register object() const { return object_; }
@@ -317,17 +309,12 @@ class IonSetPropertyIC : public IonIC {
   ConstantOrRegister rhs() const { return rhs_; }
 
   Register temp() const { return temp_; }
-  FloatRegister maybeTempDouble() const { return maybeTempDouble_; }
-  FloatRegister maybeTempFloat32() const { return maybeTempFloat32_; }
 
   bool strict() const { return strict_; }
-  bool needsPostBarrier() const { return needsPostBarrier_; }
-  bool needsTypeBarrier() const { return needsTypeBarrier_; }
-  bool guardHoles() const { return guardHoles_; }
 
-  static MOZ_MUST_USE bool update(JSContext* cx, HandleScript outerScript,
-                                  IonSetPropertyIC* ic, HandleObject obj,
-                                  HandleValue idVal, HandleValue rhs);
+  [[nodiscard]] static bool update(JSContext* cx, HandleScript outerScript,
+                                   IonSetPropertyIC* ic, HandleObject obj,
+                                   HandleValue idVal, HandleValue rhs);
 };
 
 class IonGetNameIC : public IonIC {
@@ -351,9 +338,9 @@ class IonGetNameIC : public IonIC {
   Register temp() const { return temp_; }
   LiveRegisterSet liveRegs() const { return liveRegs_; }
 
-  static MOZ_MUST_USE bool update(JSContext* cx, HandleScript outerScript,
-                                  IonGetNameIC* ic, HandleObject envChain,
-                                  MutableHandleValue res);
+  [[nodiscard]] static bool update(JSContext* cx, HandleScript outerScript,
+                                   IonGetNameIC* ic, HandleObject envChain,
+                                   MutableHandleValue res);
 };
 
 class IonBindNameIC : public IonIC {
@@ -408,6 +395,31 @@ class IonGetIteratorIC : public IonIC {
                           IonGetIteratorIC* ic, HandleValue value);
 };
 
+class IonOptimizeSpreadCallIC : public IonIC {
+  LiveRegisterSet liveRegs_;
+  ValueOperand value_;
+  Register output_;
+  Register temp_;
+
+ public:
+  IonOptimizeSpreadCallIC(LiveRegisterSet liveRegs, ValueOperand value,
+                          Register output, Register temp)
+      : IonIC(CacheKind::OptimizeSpreadCall),
+        liveRegs_(liveRegs),
+        value_(value),
+        output_(output),
+        temp_(temp) {}
+
+  ValueOperand value() const { return value_; }
+  Register output() const { return output_; }
+  Register temp() const { return temp_; }
+  LiveRegisterSet liveRegs() const { return liveRegs_; }
+
+  static bool update(JSContext* cx, HandleScript outerScript,
+                     IonOptimizeSpreadCallIC* ic, HandleValue value,
+                     bool* result);
+};
+
 class IonHasOwnIC : public IonIC {
   LiveRegisterSet liveRegs_;
 
@@ -429,9 +441,35 @@ class IonHasOwnIC : public IonIC {
   Register output() const { return output_; }
   LiveRegisterSet liveRegs() const { return liveRegs_; }
 
-  static MOZ_MUST_USE bool update(JSContext* cx, HandleScript outerScript,
-                                  IonHasOwnIC* ic, HandleValue val,
-                                  HandleValue idVal, int32_t* res);
+  [[nodiscard]] static bool update(JSContext* cx, HandleScript outerScript,
+                                   IonHasOwnIC* ic, HandleValue val,
+                                   HandleValue idVal, int32_t* res);
+};
+
+class IonCheckPrivateFieldIC : public IonIC {
+  LiveRegisterSet liveRegs_;
+
+  TypedOrValueRegister value_;
+  TypedOrValueRegister id_;
+  Register output_;
+
+ public:
+  IonCheckPrivateFieldIC(LiveRegisterSet liveRegs, TypedOrValueRegister value,
+                         TypedOrValueRegister id, Register output)
+      : IonIC(CacheKind::CheckPrivateField),
+        liveRegs_(liveRegs),
+        value_(value),
+        id_(id),
+        output_(output) {}
+
+  TypedOrValueRegister value() const { return value_; }
+  TypedOrValueRegister id() const { return id_; }
+  Register output() const { return output_; }
+  LiveRegisterSet liveRegs() const { return liveRegs_; }
+
+  [[nodiscard]] static bool update(JSContext* cx, HandleScript outerScript,
+                                   IonCheckPrivateFieldIC* ic, HandleValue val,
+                                   HandleValue idVal, bool* res);
 };
 
 class IonInIC : public IonIC {
@@ -458,9 +496,9 @@ class IonInIC : public IonIC {
   Register temp() const { return temp_; }
   LiveRegisterSet liveRegs() const { return liveRegs_; }
 
-  static MOZ_MUST_USE bool update(JSContext* cx, HandleScript outerScript,
-                                  IonInIC* ic, HandleValue key,
-                                  HandleObject obj, bool* res);
+  [[nodiscard]] static bool update(JSContext* cx, HandleScript outerScript,
+                                   IonInIC* ic, HandleValue key,
+                                   HandleObject obj, bool* res);
 };
 
 class IonInstanceOfIC : public IonIC {
@@ -485,9 +523,9 @@ class IonInstanceOfIC : public IonIC {
   Register output() const { return output_; }
 
   // This signature mimics that of TryAttachInstanceOfStub in baseline
-  static MOZ_MUST_USE bool update(JSContext* cx, HandleScript outerScript,
-                                  IonInstanceOfIC* ic, HandleValue lhs,
-                                  HandleObject rhs, bool* attached);
+  [[nodiscard]] static bool update(JSContext* cx, HandleScript outerScript,
+                                   IonInstanceOfIC* ic, HandleValue lhs,
+                                   HandleObject rhs, bool* attached);
 };
 
 class IonCompareIC : public IonIC {
@@ -511,9 +549,9 @@ class IonCompareIC : public IonIC {
   TypedOrValueRegister rhs() const { return rhs_; }
   Register output() const { return output_; }
 
-  static MOZ_MUST_USE bool update(JSContext* cx, HandleScript outerScript,
-                                  IonCompareIC* stub, HandleValue lhs,
-                                  HandleValue rhs, bool* res);
+  [[nodiscard]] static bool update(JSContext* cx, HandleScript outerScript,
+                                   IonCompareIC* stub, HandleValue lhs,
+                                   HandleValue rhs, bool* res);
 };
 
 class IonUnaryArithIC : public IonIC {
@@ -534,9 +572,31 @@ class IonUnaryArithIC : public IonIC {
   TypedOrValueRegister input() const { return input_; }
   ValueOperand output() const { return output_; }
 
-  static MOZ_MUST_USE bool update(JSContext* cx, HandleScript outerScript,
-                                  IonUnaryArithIC* stub, HandleValue val,
-                                  MutableHandleValue res);
+  [[nodiscard]] static bool update(JSContext* cx, HandleScript outerScript,
+                                   IonUnaryArithIC* stub, HandleValue val,
+                                   MutableHandleValue res);
+};
+
+class IonToPropertyKeyIC : public IonIC {
+  LiveRegisterSet liveRegs_;
+  ValueOperand input_;
+  ValueOperand output_;
+
+ public:
+  IonToPropertyKeyIC(LiveRegisterSet liveRegs, ValueOperand input,
+                     ValueOperand output)
+      : IonIC(CacheKind::ToPropertyKey),
+        liveRegs_(liveRegs),
+        input_(input),
+        output_(output) {}
+
+  LiveRegisterSet liveRegs() const { return liveRegs_; }
+  ValueOperand input() const { return input_; }
+  ValueOperand output() const { return output_; }
+
+  [[nodiscard]] static bool update(JSContext* cx, HandleScript outerScript,
+                                   IonToPropertyKeyIC* ic, HandleValue val,
+                                   MutableHandleValue res);
 };
 
 class IonBinaryArithIC : public IonIC {
@@ -560,9 +620,9 @@ class IonBinaryArithIC : public IonIC {
   TypedOrValueRegister rhs() const { return rhs_; }
   ValueOperand output() const { return output_; }
 
-  static MOZ_MUST_USE bool update(JSContext* cx, HandleScript outerScript,
-                                  IonBinaryArithIC* stub, HandleValue lhs,
-                                  HandleValue rhs, MutableHandleValue res);
+  [[nodiscard]] static bool update(JSContext* cx, HandleScript outerScript,
+                                   IonBinaryArithIC* stub, HandleValue lhs,
+                                   HandleValue rhs, MutableHandleValue res);
 };
 
 }  // namespace jit
