@@ -196,7 +196,11 @@ class LUse : public LAllocation {
   static const uint32_t POLICY_BITS = 3;
   static const uint32_t POLICY_SHIFT = 0;
   static const uint32_t POLICY_MASK = (1 << POLICY_BITS) - 1;
+#ifdef JS_CODEGEN_ARM64
+  static const uint32_t REG_BITS = 7;
+#else
   static const uint32_t REG_BITS = 6;
+#endif
   static const uint32_t REG_SHIFT = POLICY_SHIFT + POLICY_BITS;
   static const uint32_t REG_MASK = (1 << REG_BITS) - 1;
 
@@ -205,8 +209,14 @@ class LUse : public LAllocation {
   static const uint32_t USED_AT_START_SHIFT = REG_SHIFT + REG_BITS;
   static const uint32_t USED_AT_START_MASK = (1 << USED_AT_START_BITS) - 1;
 
+  // The REG field will hold the register code for any Register or
+  // FloatRegister, though not for an AnyRegister.
+  static_assert(std::max(Registers::Total, FloatRegisters::Total) <=
+                    REG_MASK + 1,
+                "The field must be able to represent any register code");
+
  public:
-  // Virtual registers get the remaining 19 bits.
+  // Virtual registers get the remaining bits.
   static const uint32_t VREG_BITS =
       DATA_BITS - (USED_AT_START_SHIFT + USED_AT_START_BITS);
   static const uint32_t VREG_SHIFT = USED_AT_START_SHIFT + USED_AT_START_BITS;
@@ -240,6 +250,7 @@ class LUse : public LAllocation {
   };
 
   void set(Policy policy, uint32_t reg, bool usedAtStart) {
+    MOZ_ASSERT(reg <= REG_MASK, "Register code must fit in field");
     setKindAndData(USE, (policy << POLICY_SHIFT) | (reg << REG_SHIFT) |
                             ((usedAtStart ? 1 : 0) << USED_AT_START_SHIFT));
   }
@@ -606,7 +617,6 @@ class LDefinition {
       case MIRType::Symbol:
       case MIRType::BigInt:
       case MIRType::Object:
-      case MIRType::ObjectOrNull:
       case MIRType::RefOrNull:
         return LDefinition::OBJECT;
       case MIRType::Double:
@@ -621,6 +631,7 @@ class LDefinition {
       case MIRType::Elements:
         return LDefinition::SLOTS;
       case MIRType::Pointer:
+      case MIRType::IntPtr:
         return LDefinition::GENERAL;
 #if defined(JS_PUNBOX64)
       case MIRType::Int64:
@@ -885,26 +896,18 @@ const LInstruction* LNode::toInstruction() const {
 }
 
 class LElementVisitor {
-  LNode* ins_;
+#ifdef TRACK_SNAPSHOTS
+  LInstruction* ins_ = nullptr;
+#endif
 
  protected:
-  jsbytecode* lastPC_;
-  jsbytecode* lastNotInlinedPC_;
+#ifdef TRACK_SNAPSHOTS
+  LInstruction* instruction() { return ins_; }
 
-  LNode* instruction() { return ins_; }
-
-  void setElement(LNode* ins) {
-    ins_ = ins;
-    if (ins->mirRaw()) {
-      lastPC_ = ins->mirRaw()->trackedPc();
-      if (ins->mirRaw()->trackedTree()) {
-        lastNotInlinedPC_ = ins->mirRaw()->profilerLeavePc();
-      }
-    }
-  }
-
-  LElementVisitor()
-      : ins_(nullptr), lastPC_(nullptr), lastNotInlinedPC_(nullptr) {}
+  void setElement(LInstruction* ins) { ins_ = ins; }
+#else
+  void setElement(LInstruction* ins) {}
+#endif
 };
 
 using LInstructionIterator = InlineList<LInstruction>::iterator;
@@ -966,7 +969,7 @@ class LBlock {
 
  public:
   explicit LBlock(MBasicBlock* block);
-  MOZ_MUST_USE bool init(TempAllocator& alloc);
+  [[nodiscard]] bool init(TempAllocator& alloc);
 
   void add(LInstruction* ins) {
     ins->setBlock(this);
@@ -1213,14 +1216,14 @@ class LRecoverInfo : public TempObject {
   RecoverOffset recoverOffset_;
 
   explicit LRecoverInfo(TempAllocator& alloc);
-  MOZ_MUST_USE bool init(MResumePoint* mir);
+  [[nodiscard]] bool init(MResumePoint* mir);
 
   // Fill the instruction vector such as all instructions needed for the
   // recovery are pushed before the current instruction.
   template <typename Node>
-  MOZ_MUST_USE bool appendOperands(Node* ins);
-  MOZ_MUST_USE bool appendDefinition(MDefinition* def);
-  MOZ_MUST_USE bool appendResumePoint(MResumePoint* rp);
+  [[nodiscard]] bool appendOperands(Node* ins);
+  [[nodiscard]] bool appendDefinition(MDefinition* def);
+  [[nodiscard]] bool appendResumePoint(MResumePoint* rp);
 
  public:
   static LRecoverInfo* New(MIRGenerator* gen, MResumePoint* mir);
@@ -1312,15 +1315,15 @@ class LRecoverInfo : public TempObject {
 // compressed and saved in the compiled script.
 class LSnapshot : public TempObject {
  private:
-  uint32_t numSlots_;
   LAllocation* slots_;
   LRecoverInfo* recoverInfo_;
   SnapshotOffset snapshotOffset_;
+  uint32_t numSlots_;
   BailoutId bailoutId_;
   BailoutKind bailoutKind_;
 
   LSnapshot(LRecoverInfo* recover, BailoutKind kind);
-  MOZ_MUST_USE bool init(MIRGenerator* gen);
+  [[nodiscard]] bool init(MIRGenerator* gen);
 
  public:
   static LSnapshot* New(MIRGenerator* gen, LRecoverInfo* recover,
@@ -1430,13 +1433,13 @@ class LSafepoint : public TempObject {
   // List of slots which have gcthing pointers.
   SlotList gcSlots_;
 
-  // List of slots which have Values.
-  SlotList valueSlots_;
-
 #ifdef JS_NUNBOX32
   // List of registers (in liveRegs) and slots which contain pieces of Values.
   NunboxList nunboxParts_;
 #elif JS_PUNBOX64
+  // List of slots which have Values.
+  SlotList valueSlots_;
+
   // The subset of liveRegs which have Values.
   LiveGeneralRegisterSet valueRegs_;
 #endif
@@ -1479,12 +1482,11 @@ class LSafepoint : public TempObject {
       : safepointOffset_(INVALID_SAFEPOINT_OFFSET),
         osiCallPointOffset_(0),
         gcSlots_(alloc),
-        valueSlots_(alloc)
 #ifdef JS_NUNBOX32
-        ,
-        nunboxParts_(alloc)
+        nunboxParts_(alloc),
+#else
+        valueSlots_(alloc),
 #endif
-        ,
         slotsOrElementsSlots_(alloc),
         isWasmTrap_(false),
         framePushedAtStackMapBase_(0) {
@@ -1507,7 +1509,7 @@ class LSafepoint : public TempObject {
     assertInvariants();
   }
   LiveGeneralRegisterSet gcRegs() const { return gcRegs_; }
-  MOZ_MUST_USE bool addGcSlot(bool stack, uint32_t slot) {
+  [[nodiscard]] bool addGcSlot(bool stack, uint32_t slot) {
     bool result = gcSlots_.append(SlotEntry(stack, slot));
     if (result) {
       assertInvariants();
@@ -1524,14 +1526,14 @@ class LSafepoint : public TempObject {
     slotsOrElementsRegs_.addUnchecked(reg);
     assertInvariants();
   }
-  MOZ_MUST_USE bool addSlotsOrElementsSlot(bool stack, uint32_t slot) {
+  [[nodiscard]] bool addSlotsOrElementsSlot(bool stack, uint32_t slot) {
     bool result = slotsOrElementsSlots_.append(SlotEntry(stack, slot));
     if (result) {
       assertInvariants();
     }
     return result;
   }
-  MOZ_MUST_USE bool addSlotsOrElementsPointer(LAllocation alloc) {
+  [[nodiscard]] bool addSlotsOrElementsPointer(LAllocation alloc) {
     if (alloc.isMemory()) {
       return addSlotsOrElementsSlot(alloc.isStackSlot(), alloc.memorySlot());
     }
@@ -1554,7 +1556,7 @@ class LSafepoint : public TempObject {
     return false;
   }
 
-  MOZ_MUST_USE bool addGcPointer(LAllocation alloc) {
+  [[nodiscard]] bool addGcPointer(LAllocation alloc) {
     if (alloc.isMemory()) {
       return addGcSlot(alloc.isStackSlot(), alloc.memorySlot());
     }
@@ -1591,28 +1593,9 @@ class LSafepoint : public TempObject {
     return true;
   }
 
-  MOZ_MUST_USE bool addValueSlot(bool stack, uint32_t slot) {
-    bool result = valueSlots_.append(SlotEntry(stack, slot));
-    if (result) {
-      assertInvariants();
-    }
-    return result;
-  }
-  SlotList& valueSlots() { return valueSlots_; }
-
-  bool hasValueSlot(bool stack, uint32_t slot) const {
-    for (size_t i = 0; i < valueSlots_.length(); i++) {
-      if (valueSlots_[i].stack == stack && valueSlots_[i].slot == slot) {
-        return true;
-      }
-    }
-    return false;
-  }
-
 #ifdef JS_NUNBOX32
-
-  MOZ_MUST_USE bool addNunboxParts(uint32_t typeVreg, LAllocation type,
-                                   LAllocation payload) {
+  [[nodiscard]] bool addNunboxParts(uint32_t typeVreg, LAllocation type,
+                                    LAllocation payload) {
     bool result = nunboxParts_.append(NunboxEntry(typeVreg, type, payload));
     if (result) {
       assertInvariants();
@@ -1620,7 +1603,7 @@ class LSafepoint : public TempObject {
     return result;
   }
 
-  MOZ_MUST_USE bool addNunboxType(uint32_t typeVreg, LAllocation type) {
+  [[nodiscard]] bool addNunboxType(uint32_t typeVreg, LAllocation type) {
     for (size_t i = 0; i < nunboxParts_.length(); i++) {
       if (nunboxParts_[i].type == type) {
         return true;
@@ -1641,8 +1624,8 @@ class LSafepoint : public TempObject {
     return result;
   }
 
-  MOZ_MUST_USE bool addNunboxPayload(uint32_t payloadVreg,
-                                     LAllocation payload) {
+  [[nodiscard]] bool addNunboxPayload(uint32_t payloadVreg,
+                                      LAllocation payload) {
     for (size_t i = 0; i < nunboxParts_.length(); i++) {
       if (nunboxParts_[i].payload == payload) {
         return true;
@@ -1679,10 +1662,6 @@ class LSafepoint : public TempObject {
 
 #  ifdef DEBUG
   bool hasNunboxPayload(LAllocation payload) const {
-    if (payload.isMemory() &&
-        hasValueSlot(payload.isStackSlot(), payload.memorySlot())) {
-      return true;
-    }
     for (size_t i = 0; i < nunboxParts_.length(); i++) {
       if (nunboxParts_[i].payload == payload) {
         return true;
@@ -1695,6 +1674,23 @@ class LSafepoint : public TempObject {
   NunboxList& nunboxParts() { return nunboxParts_; }
 
 #elif JS_PUNBOX64
+  [[nodiscard]] bool addValueSlot(bool stack, uint32_t slot) {
+    bool result = valueSlots_.append(SlotEntry(stack, slot));
+    if (result) {
+      assertInvariants();
+    }
+    return result;
+  }
+  SlotList& valueSlots() { return valueSlots_; }
+
+  bool hasValueSlot(bool stack, uint32_t slot) const {
+    for (size_t i = 0; i < valueSlots_.length(); i++) {
+      if (valueSlots_[i].stack == stack && valueSlots_[i].slot == slot) {
+        return true;
+      }
+    }
+    return false;
+  }
 
   void addValueRegister(Register reg) {
     valueRegs_.add(reg);
@@ -1702,7 +1698,7 @@ class LSafepoint : public TempObject {
   }
   LiveGeneralRegisterSet valueRegs() const { return valueRegs_; }
 
-  MOZ_MUST_USE bool addBoxedValue(LAllocation alloc) {
+  [[nodiscard]] bool addBoxedValue(LAllocation alloc) {
     if (alloc.isRegister()) {
       Register reg = alloc.toRegister().gpr();
       if (!valueRegs().has(reg)) {
@@ -1826,7 +1822,7 @@ class LIRGraph {
   // constantPool_ is a mozilla::Vector, not a js::Vector, because
   // js::Vector<Value> is prohibited as unsafe. This particular Vector of
   // Values is safe because it is only used within the scope of an
-  // AutoEnterAnalysis (in IonCompile), which inhibits GC.
+  // AutoSuppressGC (in IonCompile), which inhibits GC.
   mozilla::Vector<Value, 0, JitAllocPolicy> constantPool_;
   typedef HashMap<Value, uint32_t, ValueHasher, JitAllocPolicy> ConstantPoolMap;
   ConstantPoolMap constantPoolMap_;
@@ -1840,22 +1836,19 @@ class LIRGraph {
   // Number of stack slots needed for argument construction for calls.
   uint32_t argumentSlotCount_;
 
-  // Snapshot taken before any LIR has been lowered.
-  LSnapshot* entrySnapshot_;
-
   MIRGraph& mir_;
 
  public:
   explicit LIRGraph(MIRGraph* mir);
 
-  MOZ_MUST_USE bool init() {
+  [[nodiscard]] bool init() {
     return blocks_.init(mir_.alloc(), mir_.numBlocks());
   }
   MIRGraph& mir() const { return mir_; }
   size_t numBlocks() const { return blocks_.length(); }
   LBlock* getBlock(size_t i) { return &blocks_[i]; }
   uint32_t numBlockIds() const { return mir_.numBlockIds(); }
-  MOZ_MUST_USE bool initBlock(MBasicBlock* mir) {
+  [[nodiscard]] bool initBlock(MBasicBlock* mir) {
     auto* block = &blocks_[mir->id()];
     auto* lir = new (block) LBlock(mir);
     return lir->init(mir_.alloc());
@@ -1895,17 +1888,10 @@ class LIRGraph {
   uint32_t totalSlotCount() const {
     return paddedLocalSlotCount() + argumentsSize();
   }
-  MOZ_MUST_USE bool addConstantToPool(const Value& v, uint32_t* index);
+  [[nodiscard]] bool addConstantToPool(const Value& v, uint32_t* index);
   size_t numConstants() const { return constantPool_.length(); }
   Value* constantPool() { return &constantPool_[0]; }
-  void setEntrySnapshot(LSnapshot* snapshot) {
-    MOZ_ASSERT(!entrySnapshot_);
-    entrySnapshot_ = snapshot;
-  }
-  LSnapshot* entrySnapshot() const {
-    MOZ_ASSERT(entrySnapshot_);
-    return entrySnapshot_;
-  }
+
   bool noteNeedsSafepoint(LInstruction* ins);
   size_t numNonCallSafepoints() const { return nonCallSafepoints_.length(); }
   LInstruction* getNonCallSafepoint(size_t i) const {
@@ -2004,37 +1990,6 @@ LALLOC_CONST_CAST(ConstantIndex)
 
 #undef LALLOC_CAST
 
-#ifdef JS_NUNBOX32
-static inline signed OffsetToOtherHalfOfNunbox(LDefinition::Type type) {
-  MOZ_ASSERT(type == LDefinition::TYPE || type == LDefinition::PAYLOAD);
-  signed offset = (type == LDefinition::TYPE) ? PAYLOAD_INDEX - TYPE_INDEX
-                                              : TYPE_INDEX - PAYLOAD_INDEX;
-  return offset;
-}
-
-static inline void AssertTypesFormANunbox(LDefinition::Type type1,
-                                          LDefinition::Type type2) {
-  MOZ_ASSERT((type1 == LDefinition::TYPE && type2 == LDefinition::PAYLOAD) ||
-             (type2 == LDefinition::TYPE && type1 == LDefinition::PAYLOAD));
-}
-
-static inline unsigned OffsetOfNunboxSlot(LDefinition::Type type) {
-  if (type == LDefinition::PAYLOAD) {
-    return NUNBOX32_PAYLOAD_OFFSET;
-  }
-  return NUNBOX32_TYPE_OFFSET;
-}
-
-// Note that stack indexes for LStackSlot are modelled backwards, so a
-// double-sized slot starting at 2 has its next word at 1, *not* 3.
-static inline unsigned BaseOfNunboxSlot(LDefinition::Type type, unsigned slot) {
-  if (type == LDefinition::PAYLOAD) {
-    return slot + NUNBOX32_PAYLOAD_OFFSET;
-  }
-  return slot + NUNBOX32_TYPE_OFFSET;
-}
-#endif
-
 }  // namespace jit
 }  // namespace js
 
