@@ -7,19 +7,22 @@
 #ifndef jit_RegisterSets_h
 #define jit_RegisterSets_h
 
+#include "mozilla/Assertions.h"
 #include "mozilla/Attributes.h"
-#include "mozilla/MathAlgorithms.h"
 
 #include <new>
+#include <stddef.h>
+#include <stdint.h>
 
-#include "jit/JitAllocPolicy.h"
+#include "jit/IonTypes.h"
 #include "jit/Registers.h"
+#include "js/Value.h"
 
 namespace js {
 namespace jit {
 
 struct AnyRegister {
-  typedef uint8_t Code;
+  using Code = uint8_t;
 
   static const uint8_t Total = Registers::Total + FloatRegisters::Total;
   static const uint8_t Invalid = UINT8_MAX;
@@ -123,10 +126,16 @@ class ValueOperand {
 
   constexpr Register typeReg() const { return type_; }
   constexpr Register payloadReg() const { return payload_; }
+  constexpr Register64 toRegister64() const {
+    return Register64(typeReg(), payloadReg());
+  }
   constexpr bool aliases(Register reg) const {
     return type_ == reg || payload_ == reg;
   }
   constexpr Register payloadOrValueReg() const { return payloadReg(); }
+  bool hasVolatileReg() const {
+    return type_.volatile_() || payload_.volatile_();
+  }
   constexpr bool operator==(const ValueOperand& o) const {
     return type_ == o.type_ && payload_ == o.payload_;
   }
@@ -141,8 +150,10 @@ class ValueOperand {
   explicit constexpr ValueOperand(Register value) : value_(value) {}
 
   constexpr Register valueReg() const { return value_; }
+  constexpr Register64 toRegister64() const { return Register64(valueReg()); }
   constexpr bool aliases(Register reg) const { return value_ == reg; }
   constexpr Register payloadOrValueReg() const { return valueReg(); }
+  bool hasVolatileReg() const { return value_.volatile_(); }
   constexpr bool operator==(const ValueOperand& o) const {
     return value_ == o.value_;
   }
@@ -163,7 +174,16 @@ class TypedOrValueRegister {
 
   union U {
     AnyRegister::Code typed;
-    ValueOperand value;
+#if defined(JS_PUNBOX64)
+    Register::Code value;
+#elif defined(JS_NUNBOX32)
+    struct {
+      Register::Code valueType;
+      Register::Code valuePayload;
+    } s;
+#else
+#  error "Bad architecture"
+#endif
   } data;
 
  public:
@@ -175,7 +195,14 @@ class TypedOrValueRegister {
 
   MOZ_IMPLICIT TypedOrValueRegister(ValueOperand value)
       : type_(MIRType::Value) {
-    data.value = value;
+#if defined(JS_PUNBOX64)
+    data.value = value.valueReg().code();
+#elif defined(JS_NUNBOX32)
+    data.s.valueType = value.typeReg().code();
+    data.s.valuePayload = value.payloadReg().code();
+#else
+#  error "Bad architecture"
+#endif
   }
 
   MIRType type() const { return type_; }
@@ -193,7 +220,14 @@ class TypedOrValueRegister {
 
   ValueOperand valueReg() const {
     MOZ_ASSERT(hasValue());
-    return data.value;
+#if defined(JS_PUNBOX64)
+    return ValueOperand(Register::FromCode(data.value));
+#elif defined(JS_NUNBOX32)
+    return ValueOperand(Register::FromCode(data.s.valueType),
+                        Register::FromCode(data.s.valuePayload));
+#else
+#  error "Bad architecture"
+#endif
   }
 
   AnyRegister scratchReg() {
@@ -224,9 +258,9 @@ class ConstantOrRegister {
  public:
   ConstantOrRegister() = delete;
 
-  MOZ_IMPLICIT ConstantOrRegister(const Value& value) : constant_(true) {
+  MOZ_IMPLICIT ConstantOrRegister(const JS::Value& value) : constant_(true) {
     MOZ_ASSERT(constant());
-    new (&data.constant) Value(value);
+    new (&data.constant) JS::Value(value);
   }
 
   MOZ_IMPLICIT ConstantOrRegister(TypedOrValueRegister reg) : constant_(false) {
@@ -236,7 +270,7 @@ class ConstantOrRegister {
 
   bool constant() const { return constant_; }
 
-  Value value() const {
+  JS::Value value() const {
     MOZ_ASSERT(constant());
     return data.constant;
   }
@@ -250,8 +284,8 @@ class ConstantOrRegister {
 template <typename T>
 class TypedRegisterSet {
  public:
-  typedef T RegType;
-  typedef typename T::SetType SetType;
+  using RegType = T;
+  using SetType = typename T::SetType;
 
  private:
   SetType bits_;
@@ -339,8 +373,8 @@ class TypedRegisterSet {
   uint32_t getPushSizeInBytes() const { return T::GetPushSizeInBytes(*this); }
 };
 
-typedef TypedRegisterSet<Register> GeneralRegisterSet;
-typedef TypedRegisterSet<FloatRegister> FloatRegisterSet;
+using GeneralRegisterSet = TypedRegisterSet<Register>;
+using FloatRegisterSet = TypedRegisterSet<FloatRegister>;
 
 class AnyRegisterIterator;
 
@@ -351,7 +385,7 @@ class RegisterSet {
   friend class AnyRegisterIterator;
 
  public:
-  RegisterSet() {}
+  RegisterSet() = default;
   constexpr RegisterSet(const GeneralRegisterSet& gpr,
                         const FloatRegisterSet& fpu)
       : gpr_(gpr), fpu_(fpu) {}
@@ -473,9 +507,9 @@ class LiveSet;
 template <typename Set>
 class AllocatableSetAccessors {
  public:
-  typedef Set RegSet;
-  typedef typename RegSet::RegType RegType;
-  typedef typename RegSet::SetType SetType;
+  using RegSet = Set;
+  using RegType = typename RegSet::RegType;
+  using SetType = typename RegSet::SetType;
 
  protected:
   RegSet set_;
@@ -506,9 +540,9 @@ class AllocatableSetAccessors {
 template <>
 class AllocatableSetAccessors<RegisterSet> {
  public:
-  typedef RegisterSet RegSet;
-  typedef AnyRegister RegType;
-  typedef char SetType;
+  using RegSet = RegisterSet;
+  using RegType = AnyRegister;
+  using SetType = char;
 
  protected:
   RegisterSet set_;
@@ -555,9 +589,9 @@ class AllocatableSetAccessors<RegisterSet> {
 template <typename Set>
 class LiveSetAccessors {
  public:
-  typedef Set RegSet;
-  typedef typename RegSet::RegType RegType;
-  typedef typename RegSet::SetType SetType;
+  using RegSet = Set;
+  using RegType = typename RegSet::RegType;
+  using SetType = typename RegSet::SetType;
 
  protected:
   RegSet set_;
@@ -583,9 +617,9 @@ class LiveSetAccessors {
 template <>
 class LiveSetAccessors<RegisterSet> {
  public:
-  typedef RegisterSet RegSet;
-  typedef AnyRegister RegType;
-  typedef char SetType;
+  using RegSet = RegisterSet;
+  using RegType = AnyRegister;
+  using SetType = char;
 
  protected:
   RegisterSet set_;
@@ -631,7 +665,7 @@ class LiveSetAccessors<RegisterSet> {
 // |takeAny| variants.
 template <class Accessors, typename Set>
 class SpecializedRegSet : public Accessors {
-  typedef Accessors Parent;
+  using Parent = Accessors;
 
  public:
   DEFINE_ACCESSOR_CONSTRUCTORS_(SpecializedRegSet)
@@ -740,7 +774,7 @@ class SpecializedRegSet : public Accessors {
 // Specialization of the accessors for the RegisterSet aggregate.
 template <class Accessors>
 class SpecializedRegSet<Accessors, RegisterSet> : public Accessors {
-  typedef Accessors Parent;
+  using Parent = Accessors;
 
  public:
   DEFINE_ACCESSOR_CONSTRUCTORS_(SpecializedRegSet)
@@ -855,8 +889,8 @@ class SpecializedRegSet<Accessors, RegisterSet> : public Accessors {
 };
 
 // Interface which is common to all register set implementations. It overloads
-// |add|, |take| and |takeUnchecked| methods for types such as |ValueOperand|
-// and |TypedOrValueRegister|.
+// |add|, |take| and |takeUnchecked| methods for types such as |ValueOperand|,
+// |TypedOrValueRegister|, and |Register64|.
 template <class Accessors, typename Set>
 class CommonRegSet : public SpecializedRegSet<Accessors, Set> {
   typedef SpecializedRegSet<Accessors, Set> Parent;
@@ -879,6 +913,14 @@ class CommonRegSet : public SpecializedRegSet<Accessors, Set> {
     add(value.valueReg());
 #else
 #  error "Bad architecture"
+#endif
+  }
+  void add(Register64 reg) {
+#if JS_BITS_PER_WORD == 32
+    add(reg.high);
+    add(reg.low);
+#else
+    add(reg.reg);
 #endif
   }
 
@@ -891,6 +933,14 @@ class CommonRegSet : public SpecializedRegSet<Accessors, Set> {
     addUnchecked(value.valueReg());
 #else
 #  error "Bad architecture"
+#endif
+  }
+  void addUnchecked(Register64 reg) {
+#if JS_BITS_PER_WORD == 32
+    take(reg.high);
+    take(reg.low);
+#else
+    take(reg.reg);
 #endif
   }
 
@@ -920,6 +970,14 @@ class CommonRegSet : public SpecializedRegSet<Accessors, Set> {
       take(reg.typedReg());
     }
   }
+  void take(Register64 reg) {
+#if JS_BITS_PER_WORD == 32
+    take(reg.high);
+    take(reg.low);
+#else
+    take(reg.reg);
+#endif
+  }
 
   using Parent::takeUnchecked;
   void takeUnchecked(ValueOperand value) {
@@ -939,6 +997,14 @@ class CommonRegSet : public SpecializedRegSet<Accessors, Set> {
       takeUnchecked(reg.typedReg());
     }
   }
+  void takeUnchecked(Register64 reg) {
+#if JS_BITS_PER_WORD == 32
+    takeUnchecked(reg.high);
+    takeUnchecked(reg.low);
+#else
+    takeUnchecked(reg.reg);
+#endif
+  }
 };
 
 // These classes do not provide any additional members, they only use their
@@ -1007,13 +1073,13 @@ class AllocatableSet<RegisterSet>
 #undef DEFINE_ACCESSOR_CONSTRUCTORS_FOR_REGISTERSET_
 #undef DEFINE_ACCESSOR_CONSTRUCTORS_
 
-typedef AllocatableSet<GeneralRegisterSet> AllocatableGeneralRegisterSet;
-typedef AllocatableSet<FloatRegisterSet> AllocatableFloatRegisterSet;
-typedef AllocatableSet<RegisterSet> AllocatableRegisterSet;
+using AllocatableGeneralRegisterSet = AllocatableSet<GeneralRegisterSet>;
+using AllocatableFloatRegisterSet = AllocatableSet<FloatRegisterSet>;
+using AllocatableRegisterSet = AllocatableSet<RegisterSet>;
 
-typedef LiveSet<GeneralRegisterSet> LiveGeneralRegisterSet;
-typedef LiveSet<FloatRegisterSet> LiveFloatRegisterSet;
-typedef LiveSet<RegisterSet> LiveRegisterSet;
+using LiveGeneralRegisterSet = LiveSet<GeneralRegisterSet>;
+using LiveFloatRegisterSet = LiveSet<FloatRegisterSet>;
+using LiveRegisterSet = LiveSet<RegisterSet>;
 
 // iterates in whatever order happens to be convenient.
 // Use TypedRegisterBackwardIterator or TypedRegisterForwardIterator if a
@@ -1080,14 +1146,14 @@ class TypedRegisterForwardIterator {
   T operator*() const { return regset_.template getFirst<RegTypeName::Any>(); }
 };
 
-typedef TypedRegisterIterator<Register> GeneralRegisterIterator;
-typedef TypedRegisterIterator<FloatRegister> FloatRegisterIterator;
-typedef TypedRegisterBackwardIterator<Register> GeneralRegisterBackwardIterator;
-typedef TypedRegisterBackwardIterator<FloatRegister>
-    FloatRegisterBackwardIterator;
-typedef TypedRegisterForwardIterator<Register> GeneralRegisterForwardIterator;
-typedef TypedRegisterForwardIterator<FloatRegister>
-    FloatRegisterForwardIterator;
+using GeneralRegisterIterator = TypedRegisterIterator<Register>;
+using FloatRegisterIterator = TypedRegisterIterator<FloatRegister>;
+using GeneralRegisterBackwardIterator = TypedRegisterBackwardIterator<Register>;
+using FloatRegisterBackwardIterator =
+    TypedRegisterBackwardIterator<FloatRegister>;
+using GeneralRegisterForwardIterator = TypedRegisterForwardIterator<Register>;
+using FloatRegisterForwardIterator =
+    TypedRegisterForwardIterator<FloatRegister>;
 
 class AnyRegisterIterator {
   GeneralRegisterIterator geniter_;
@@ -1103,8 +1169,7 @@ class AnyRegisterIterator {
       : geniter_(set.gpr_), floatiter_(set.fpu_) {}
   explicit AnyRegisterIterator(const LiveSet<RegisterSet>& set)
       : geniter_(set.gprs()), floatiter_(set.fpus()) {}
-  AnyRegisterIterator(const AnyRegisterIterator& other)
-      : geniter_(other.geniter_), floatiter_(other.floatiter_) {}
+  AnyRegisterIterator(const AnyRegisterIterator& other) = default;
   bool more() const { return geniter_.more() || floatiter_.more(); }
   AnyRegisterIterator& operator++() {
     if (geniter_.more()) {
