@@ -11,9 +11,9 @@
 #include "jit/arm64/vixl/Debugger-vixl.h"
 #include "jit/arm64/vixl/MacroAssembler-vixl.h"
 #include "jit/AtomicOp.h"
-#include "jit/JitFrames.h"
 #include "jit/MoveResolver.h"
 #include "vm/BigIntType.h"  // JS::BigInt
+#include "wasm/WasmTypes.h"
 
 #ifdef _M_ARM64
 #  ifdef move32
@@ -261,6 +261,39 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
       Mov(GetStackPointer64(), vixl::sp);
     }
   }
+  // In debug builds only, cause a trap if PSP is active and PSP != SP
+  void assertStackPtrsSynced(uint32_t id) {
+#ifdef DEBUG
+    // The add and sub instructions below will only take a 12-bit immediate.
+    MOZ_ASSERT(id <= 0xFFF);
+    if (!GetStackPointer64().Is(vixl::sp)) {
+      Label ok;
+      // Add a marker, so we can figure out who requested the check when
+      // inspecting the generated code.  Note, a more concise way to encode
+      // the marker would be to use it as an immediate for the `brk`
+      // instruction as generated by `Unreachable()`, and removing the add/sub.
+      Add(GetStackPointer64(), GetStackPointer64(), Operand(id));
+      Sub(GetStackPointer64(), GetStackPointer64(), Operand(id));
+      Cmp(vixl::sp, GetStackPointer64());
+      B(Equal, &ok);
+      Unreachable();
+      bind(&ok);
+    }
+#endif
+  }
+  // In debug builds only, add a marker that doesn't change the machine's
+  // state.  Note these markers are x16-based, as opposed to the x28-based
+  // ones made by `assertStackPtrsSynced`.
+  void addMarker(uint32_t id) {
+#ifdef DEBUG
+    // Only 12 bits of immediate are allowed.
+    MOZ_ASSERT(id <= 0xFFF);
+    ARMRegister x16 = ARMRegister(r16, 64);
+    Add(x16, x16, Operand(id));
+    Sub(x16, x16, Operand(id));
+#endif
+  }
+
   void storeValue(ValueOperand val, const Address& dest) {
     storePtr(val.valueReg(), dest);
   }
@@ -310,6 +343,8 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   }
   void popValue(ValueOperand val) {
     vixl::MacroAssembler::Pop(ARMRegister(val.valueReg(), 64));
+    // SP may be < PSP now (that's OK).
+    // eg testcase: tests/backup-point-bug1315634.js
   }
   void pushValue(const Value& val) {
     vixl::UseScratchRegisterScope temps(this);
@@ -393,47 +428,45 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     return CodeOffset(off.getOffset());
   }
 
-  void boxValue(JSValueType type, Register src, Register dest) {
-    Orr(ARMRegister(dest, 64), ARMRegister(src, 64),
-        Operand(ImmShiftedTag(type).value));
-  }
+  void boxValue(JSValueType type, Register src, Register dest);
+
   void splitSignExtTag(Register src, Register dest) {
     sbfx(ARMRegister(dest, 64), ARMRegister(src, 64), JSVAL_TAG_SHIFT,
          (64 - JSVAL_TAG_SHIFT));
   }
-  MOZ_MUST_USE Register extractTag(const Address& address, Register scratch) {
+  [[nodiscard]] Register extractTag(const Address& address, Register scratch) {
     loadPtr(address, scratch);
     splitSignExtTag(scratch, scratch);
     return scratch;
   }
-  MOZ_MUST_USE Register extractTag(const ValueOperand& value,
-                                   Register scratch) {
+  [[nodiscard]] Register extractTag(const ValueOperand& value,
+                                    Register scratch) {
     splitSignExtTag(value.valueReg(), scratch);
     return scratch;
   }
-  MOZ_MUST_USE Register extractObject(const Address& address,
-                                      Register scratch) {
+  [[nodiscard]] Register extractObject(const Address& address,
+                                       Register scratch) {
     loadPtr(address, scratch);
     unboxObject(scratch, scratch);
     return scratch;
   }
-  MOZ_MUST_USE Register extractObject(const ValueOperand& value,
-                                      Register scratch) {
+  [[nodiscard]] Register extractObject(const ValueOperand& value,
+                                       Register scratch) {
     unboxObject(value, scratch);
     return scratch;
   }
-  MOZ_MUST_USE Register extractSymbol(const ValueOperand& value,
-                                      Register scratch) {
+  [[nodiscard]] Register extractSymbol(const ValueOperand& value,
+                                       Register scratch) {
     unboxSymbol(value, scratch);
     return scratch;
   }
-  MOZ_MUST_USE Register extractInt32(const ValueOperand& value,
-                                     Register scratch) {
+  [[nodiscard]] Register extractInt32(const ValueOperand& value,
+                                      Register scratch) {
     unboxInt32(value, scratch);
     return scratch;
   }
-  MOZ_MUST_USE Register extractBoolean(const ValueOperand& value,
-                                       Register scratch) {
+  [[nodiscard]] Register extractBoolean(const ValueOperand& value,
+                                        Register scratch) {
     unboxBoolean(value, scratch);
     return scratch;
   }
@@ -530,26 +563,55 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
 
   void convertDoubleToInt32(FloatRegister src, Register dest, Label* fail,
                             bool negativeZeroCheck = true) {
-    vixl::UseScratchRegisterScope temps(this);
-    const ARMFPRegister scratch64 = temps.AcquireD();
-
-    ARMFPRegister fsrc(src, 64);
+    ARMFPRegister fsrc64(src, 64);
     ARMRegister dest32(dest, 32);
-    ARMRegister dest64(dest, 64);
 
-    MOZ_ASSERT(!scratch64.Is(fsrc));
+    // ARMv8.3 chips support the FJCVTZS instruction, which handles
+    // exactly this logic.
+    if (CPUHas(vixl::CPUFeatures::kFP, vixl::CPUFeatures::kJSCVT)) {
+      // Convert double to integer, rounding toward zero.
+      // The Z-flag is set iff the conversion is exact. -0 unsets the Z-flag.
+      Fjcvtzs(dest32, fsrc64);
 
-    Fcvtzs(dest32, fsrc);      // Convert, rounding toward zero.
-    Scvtf(scratch64, dest32);  // Convert back, using FPCR rounding mode.
-    Fcmp(scratch64, fsrc);
-    B(fail, Assembler::NotEqual);
+      if (negativeZeroCheck) {
+        B(fail, Assembler::NonZero);
+      } else {
+        Label done;
+        B(&done, Assembler::Zero);  // If conversion was exact, go to end.
 
-    if (negativeZeroCheck) {
-      Label nonzero;
-      Cbnz(dest32, &nonzero);
-      Fmov(dest64, fsrc);
-      Cbnz(dest64, fail);
-      bind(&nonzero);
+        // The conversion was inexact, but the caller intends to allow -0.
+        vixl::UseScratchRegisterScope temps(this);
+        const ARMFPRegister scratch64 = temps.AcquireD();
+        MOZ_ASSERT(!scratch64.Is(fsrc64));
+
+        // Compare fsrc64 to 0.
+        // If fsrc64 == 0 and FJCVTZS conversion was inexact, then fsrc64 is -0.
+        Fmov(scratch64, xzr);
+        Fcmp(scratch64, fsrc64);
+        B(fail, Assembler::NotEqual);  // Pass through -0; fail otherwise.
+
+        bind(&done);
+      }
+    } else {
+      // Older processors use a significantly slower path.
+      ARMRegister dest64(dest, 64);
+
+      vixl::UseScratchRegisterScope temps(this);
+      const ARMFPRegister scratch64 = temps.AcquireD();
+      MOZ_ASSERT(!scratch64.Is(fsrc64));
+
+      Fcvtzs(dest32, fsrc64);    // Convert, rounding toward zero.
+      Scvtf(scratch64, dest32);  // Convert back, using FPCR rounding mode.
+      Fcmp(scratch64, fsrc64);
+      B(fail, Assembler::NotEqual);
+
+      if (negativeZeroCheck) {
+        Label nonzero;
+        Cbnz(dest32, &nonzero);
+        Fmov(dest64, fsrc64);
+        Cbnz(dest64, fail);
+        bind(&nonzero);
+      }
     }
   }
   void convertFloat32ToInt32(FloatRegister src, Register dest, Label* fail,
@@ -578,6 +640,32 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     And(dest64, dest64, Operand(0xffffffff));
   }
 
+  void convertDoubleToPtr(FloatRegister src, Register dest, Label* fail,
+                          bool negativeZeroCheck = true) {
+    ARMFPRegister fsrc64(src, 64);
+    ARMRegister dest64(dest, 64);
+
+    vixl::UseScratchRegisterScope temps(this);
+    const ARMFPRegister scratch64 = temps.AcquireD();
+    MOZ_ASSERT(!scratch64.Is(fsrc64));
+
+    // Note: we can't use the FJCVTZS instruction here because that only works
+    // for 32-bit values.
+
+    Fcvtzs(dest64, fsrc64);    // Convert, rounding toward zero.
+    Scvtf(scratch64, dest64);  // Convert back, using FPCR rounding mode.
+    Fcmp(scratch64, fsrc64);
+    B(fail, Assembler::NotEqual);
+
+    if (negativeZeroCheck) {
+      Label nonzero;
+      Cbnz(dest64, &nonzero);
+      Fmov(dest64, fsrc64);
+      Cbnz(dest64, fail);
+      bind(&nonzero);
+    }
+  }
+
   void floor(FloatRegister input, Register output, Label* bail) {
     Label handleZero;
     // Label handleNeg;
@@ -680,18 +768,30 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
 
   void jump(Label* label) { B(label); }
   void jump(JitCode* code) { branch(code); }
-  void jump(TrampolinePtr code) {
+  void jump(ImmPtr ptr) {
+    // It is unclear why this sync is necessary:
+    // * PSP and SP have been observed to be different in testcase
+    //   tests/asm.js/testBug1046688.js.
+    // * Removing the sync causes no failures in all of jit-tests.
+    //
+    // Also see branch(JitCode*) below. This version of jump() is called only
+    // from jump(TrampolinePtr) which is called on various very slow paths,
+    // probably only in JS.
     syncStackPtr();
     BufferOffset loc =
         b(-1,
           LabelDoc());  // The jump target will be patched by executableCopy().
-    addPendingJump(loc, ImmPtr(code.value), RelocationKind::HARDCODED);
+    addPendingJump(loc, ptr, RelocationKind::HARDCODED);
   }
-  void jump(RepatchLabel* label) { MOZ_CRASH("jump (repatchlabel)"); }
+  void jump(TrampolinePtr code) { jump(ImmPtr(code.value)); }
   void jump(Register reg) { Br(ARMRegister(reg, 64)); }
   void jump(const Address& addr) {
-    loadPtr(addr, ip0);
-    Br(vixl::ip0);
+    vixl::UseScratchRegisterScope temps(this);
+    MOZ_ASSERT(temps.IsAvailable(ScratchReg64));  // ip0
+    temps.Exclude(ScratchReg64);
+    MOZ_ASSERT(addr.base != ScratchReg64.asUnsized());
+    loadPtr(addr, ScratchReg64.asUnsized());
+    br(ScratchReg64);
   }
 
   void align(int alignment) { armbuffer_.align(alignment); }
@@ -818,6 +918,10 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     Mov(scratch32, Operand(imm.value));
     doBaseIndex(scratch32, address, vixl::STRH_w);
   }
+  template <typename S, typename T>
+  void store16Unaligned(const S& src, const T& dest) {
+    store16(src, dest);
+  }
 
   void storePtr(ImmWord imm, const Address& address) {
     vixl::UseScratchRegisterScope temps(this);
@@ -909,8 +1013,30 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     Str(scratch32, toMemOperand(address));
   }
 
+  template <typename S, typename T>
+  void store32Unaligned(const S& src, const T& dest) {
+    store32(src, dest);
+  }
+
   void store64(Register64 src, Address address) { storePtr(src.reg, address); }
 
+  void store64(Register64 src, const BaseIndex& address) {
+    storePtr(src.reg, address);
+  }
+
+  void store64(Imm64 imm, const BaseIndex& address) {
+    storePtr(ImmWord(imm.value), address);
+  }
+
+  void store64(Imm64 imm, const Address& address) {
+    storePtr(ImmWord(imm.value), address);
+  }
+
+  template <typename S, typename T>
+  void store64Unaligned(const S& src, const T& dest) {
+    store64(src, dest);
+  }
+
   // StackPointer manipulation.
   inline void addToStackPtr(Register src);
   inline void addToStackPtr(Imm32 imm);
@@ -922,7 +1048,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   inline void subStackPtrFrom(Register dest);
 
   inline void andToStackPtr(Imm32 t);
-  inline void andStackPtrTo(Register dest);
 
   inline void moveToStackPtr(Register src);
   inline void moveStackPtrTo(Register dest);
@@ -974,14 +1099,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     Ldr(scratch32, toMemOperand(lhs));
     Cmp(scratch32, Operand(ARMRegister(rhs, 32)));
   }
-  void cmp32(Register lhs, const Address& rhs) {
-    vixl::UseScratchRegisterScope temps(this);
-    const ARMRegister scratch32 = temps.AcquireW();
-    MOZ_ASSERT(scratch32.asUnsized() != rhs.base);
-    MOZ_ASSERT(scratch32.asUnsized() != lhs);
-    Ldr(scratch32, toMemOperand(rhs));
-    Cmp(scratch32, Operand(ARMRegister(lhs, 32)));
-  }
   void cmp32(const vixl::Operand& lhs, Imm32 rhs) {
     vixl::UseScratchRegisterScope temps(this);
     const ARMRegister scratch32 = temps.AcquireW();
@@ -994,12 +1111,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     Mov(scratch32, lhs);
     Cmp(scratch32, Operand(ARMRegister(rhs, 32)));
   }
-  void cmp32(Register lhs, const vixl::Operand& rhs) {
-    vixl::UseScratchRegisterScope temps(this);
-    const ARMRegister scratch32 = temps.AcquireW();
-    Mov(scratch32, rhs);
-    Cmp(scratch32, Operand(ARMRegister(lhs, 32)));
-  }
 
   void cmn32(Register lhs, Imm32 rhs) {
     Cmn(ARMRegister(lhs, 32), Operand(rhs.value));
@@ -1135,6 +1246,10 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     MOZ_CRASH("moveFloatAsDouble");
   }
 
+  void moveSimd128(FloatRegister src, FloatRegister dest) {
+    fmov(ARMFPRegister(dest, 128), ARMFPRegister(src, 128));
+  }
+
   void splitSignExtTag(const ValueOperand& operand, Register dest) {
     splitSignExtTag(operand.valueReg(), dest);
   }
@@ -1163,9 +1278,20 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     movePtr(ImmWord((uintptr_t)address.addr), scratch64.asUnsized());
     ldr(ARMRegister(dest, 32), MemOperand(scratch64));
   }
+  template <typename S>
+  void load32Unaligned(const S& src, Register dest) {
+    load32(src, dest);
+  }
   void load64(const Address& address, Register64 dest) {
     loadPtr(address, dest.reg);
   }
+  void load64(const BaseIndex& address, Register64 dest) {
+    loadPtr(address, dest.reg);
+  }
+  template <typename S>
+  void load64Unaligned(const S& src, Register64 dest) {
+    load64(src, dest);
+  }
 
   void load8SignExtend(const Address& address, Register dest) {
     Ldrsb(ARMRegister(dest, 32), toMemOperand(address));
@@ -1187,6 +1313,10 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   void load16SignExtend(const BaseIndex& src, Register dest) {
     doBaseIndex(ARMRegister(dest, 32), src, vixl::LDRSH_w);
   }
+  template <typename S>
+  void load16UnalignedSignExtend(const S& src, Register dest) {
+    load16SignExtend(src, dest);
+  }
 
   void load16ZeroExtend(const Address& address, Register dest) {
     Ldrh(ARMRegister(dest, 32), toMemOperand(address));
@@ -1194,6 +1324,10 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   void load16ZeroExtend(const BaseIndex& src, Register dest) {
     doBaseIndex(ARMRegister(dest, 32), src, vixl::LDRH_w);
   }
+  template <typename S>
+  void load16UnalignedZeroExtend(const S& src, Register dest) {
+    load16ZeroExtend(src, dest);
+  }
 
   void adds32(Register src, Register dest) {
     Adds(ARMRegister(dest, 32), ARMRegister(dest, 32),
@@ -1211,6 +1345,13 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     Adds(scratch32, scratch32, Operand(imm.value));
     Str(scratch32, toMemOperand(dest));
   }
+  void adds64(Imm32 imm, Register dest) {
+    Adds(ARMRegister(dest, 64), ARMRegister(dest, 64), Operand(imm.value));
+  }
+  void adds64(Register src, Register dest) {
+    Adds(ARMRegister(dest, 64), ARMRegister(dest, 64),
+         Operand(ARMRegister(src, 64)));
+  }
 
   void subs32(Imm32 imm, Register dest) {
     Subs(ARMRegister(dest, 32), ARMRegister(dest, 32), Operand(imm.value));
@@ -1219,6 +1360,13 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     Subs(ARMRegister(dest, 32), ARMRegister(dest, 32),
          Operand(ARMRegister(src, 32)));
   }
+  void subs64(Imm32 imm, Register dest) {
+    Subs(ARMRegister(dest, 64), ARMRegister(dest, 64), Operand(imm.value));
+  }
+  void subs64(Register src, Register dest) {
+    Subs(ARMRegister(dest, 64), ARMRegister(dest, 64),
+         Operand(ARMRegister(src, 64)));
+  }
 
   void ret() {
     pop(lr);
@@ -1226,17 +1374,27 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   }
 
   void retn(Imm32 n) {
+    vixl::UseScratchRegisterScope temps(this);
+    MOZ_ASSERT(temps.IsAvailable(ScratchReg64));  // ip0
+    temps.Exclude(ScratchReg64);
     // ip0 <- [sp]; sp += n; ret ip0
-    Ldr(vixl::ip0,
+    Ldr(ScratchReg64,
         MemOperand(GetStackPointer64(), ptrdiff_t(n.value), vixl::PostIndex));
     syncStackPtr();  // SP is always used to transmit the stack between calls.
-    Ret(vixl::ip0);
+    Ret(ScratchReg64);
   }
 
   void j(Condition cond, Label* dest) { B(dest, cond); }
 
   void branch(Condition cond, Label* label) { B(label, cond); }
   void branch(JitCode* target) {
+    // It is unclear why this sync is necessary:
+    // * PSP and SP have been observed to be different in testcase
+    //   tests/async/debugger-reject-after-fulfill.js
+    // * Removing the sync causes no failures in all of jit-tests.
+    //
+    // Also see jump() above.  This is used only to implement jump(JitCode*)
+    // and only for JS, it appears.
     syncStackPtr();
     BufferOffset loc =
         b(-1,
@@ -1244,28 +1402,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     addPendingJump(loc, ImmPtr(target->raw()), RelocationKind::JITCODE);
   }
 
-  CodeOffsetJump jumpWithPatch(RepatchLabel* label) {
-    // jumpWithPatch() is only used by IonCacheIRCompiler::emitReturnFromIC().
-    // The RepatchLabel is unbound and unused.
-    MOZ_ASSERT(!label->used());
-    MOZ_ASSERT(!label->bound());
-
-    vixl::UseScratchRegisterScope temps(this);
-    const ARMRegister scratch64 = temps.AcquireX();
-
-    ARMBuffer::PoolEntry pe;
-    BufferOffset load_bo;
-
-    // This no-op load exists for PatchJump(), in the case of a target outside
-    // the range of +/- 128 MB. If the load is used, then the branch here is
-    // overwritten with a `BR` from the loaded register.
-    load_bo = immPool64(scratch64, (uint64_t)label, &pe);
-    BufferOffset branch_bo = b(-1, LabelDoc());
-
-    label->use(branch_bo.getOffset());
-    return CodeOffsetJump(load_bo.getOffset(), pe.index());
-  }
-
   void compareDouble(DoubleCondition cond, FloatRegister lhs,
                      FloatRegister rhs) {
     Fcmp(ARMFPRegister(lhs, 64), ARMFPRegister(rhs, 64));
@@ -1276,6 +1412,19 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     Fcmp(ARMFPRegister(lhs, 32), ARMFPRegister(rhs, 32));
   }
 
+  void compareSimd128Int(Assembler::Condition cond, ARMFPRegister dest,
+                         ARMFPRegister lhs, ARMFPRegister rhs);
+  void compareSimd128Float(Assembler::Condition cond, ARMFPRegister dest,
+                           ARMFPRegister lhs, ARMFPRegister rhs);
+  void rightShiftInt8x16(FloatRegister lhs, Register rhs, FloatRegister dest,
+                         bool isUnsigned);
+  void rightShiftInt16x8(FloatRegister lhs, Register rhs, FloatRegister dest,
+                         bool isUnsigned);
+  void rightShiftInt32x4(FloatRegister lhs, Register rhs, FloatRegister dest,
+                         bool isUnsigned);
+  void rightShiftInt64x2(FloatRegister lhs, Register rhs, FloatRegister dest,
+                         bool isUnsigned);
+
   void branchNegativeZero(FloatRegister reg, Register scratch, Label* label) {
     MOZ_CRASH("branchNegativeZero");
   }
@@ -1297,7 +1446,10 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     move32(src.valueReg(), dest);
   }
   void unboxInt32(const Address& src, Register dest) { load32(src, dest); }
-  void unboxDouble(const Address& src, FloatRegister dest) {
+  void unboxInt32(const BaseIndex& src, Register dest) { load32(src, dest); }
+
+  template <typename T>
+  void unboxDouble(const T& src, FloatRegister dest) {
     loadDouble(src, dest);
   }
   void unboxDouble(const ValueOperand& src, FloatRegister dest) {
@@ -1315,6 +1467,7 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     move32(src.valueReg(), dest);
   }
   void unboxBoolean(const Address& src, Register dest) { load32(src, dest); }
+  void unboxBoolean(const BaseIndex& src, Register dest) { load32(src, dest); }
 
   void unboxMagic(const ValueOperand& src, Register dest) {
     move32(src.valueReg(), dest);
@@ -1345,14 +1498,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
         Operand(JSVAL_TYPE_TO_SHIFTED_TAG(type)));
   }
 
-  void unboxPrivate(const ValueOperand& src, Register dest) {
-#if defined(JS_UNALIGNED_PRIVATE_VALUES)
-    Mov(ARMRegister(dest, 64), ARMRegister(src.valueReg(), 64));
-#else
-    Lsl(ARMRegister(dest, 64), ARMRegister(src.valueReg(), 64), 1);
-#endif
-  }
-
   void notBoolean(const ValueOperand& val) {
     ARMRegister r(val.valueReg(), 64);
     eor(r, r, Operand(1));
@@ -1376,14 +1521,18 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   void unboxObjectOrNull(const T& src, Register dest) {
     unboxNonDouble(src, dest, JSVAL_TYPE_OBJECT);
     And(ARMRegister(dest, 64), ARMRegister(dest, 64),
-        Operand(~JSVAL_OBJECT_OR_NULL_BIT));
+        Operand(~JS::detail::ValueObjectOrNullBit));
   }
 
   // See comment in MacroAssembler-x64.h.
-  void unboxGCThingForPreBarrierTrampoline(const Address& src, Register dest) {
+  void unboxGCThingForGCBarrier(const Address& src, Register dest) {
     loadPtr(src, dest);
     And(ARMRegister(dest, 64), ARMRegister(dest, 64),
-        Operand(JSVAL_PAYLOAD_MASK_GCTHING));
+        Operand(JS::detail::ValueGCThingPayloadMask));
+  }
+  void unboxGCThingForGCBarrier(const ValueOperand& src, Register dest) {
+    And(ARMRegister(dest, 64), ARMRegister(src.valueReg(), 64),
+        Operand(JS::detail::ValueGCThingPayloadMask));
   }
 
   inline void unboxValue(const ValueOperand& src, AnyRegister dest,
@@ -1510,13 +1659,13 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   }
   Condition testNumber(Condition cond, Register tag) {
     MOZ_ASSERT(cond == Equal || cond == NotEqual);
-    cmpTag(tag, ImmTag(JSVAL_UPPER_INCL_TAG_OF_NUMBER_SET));
+    cmpTag(tag, ImmTag(JS::detail::ValueUpperInclNumberTag));
     // Requires unsigned comparison due to cmpTag internals.
     return (cond == Equal) ? BelowOrEqual : Above;
   }
   Condition testGCThing(Condition cond, Register tag) {
     MOZ_ASSERT(cond == Equal || cond == NotEqual);
-    cmpTag(tag, ImmTag(JSVAL_LOWER_INCL_TAG_OF_GCTHING_SET));
+    cmpTag(tag, ImmTag(JS::detail::ValueLowerInclGCThingTag));
     // Requires unsigned comparison due to cmpTag internals.
     return (cond == Equal) ? AboveOrEqual : Below;
   }
@@ -1527,7 +1676,7 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   }
   Condition testPrimitive(Condition cond, Register tag) {
     MOZ_ASSERT(cond == Equal || cond == NotEqual);
-    cmpTag(tag, ImmTag(JSVAL_UPPER_EXCL_TAG_OF_PRIMITIVE_SET));
+    cmpTag(tag, ImmTag(JS::detail::ValueUpperExclPrimitiveTag));
     // Requires unsigned comparison due to cmpTag internals.
     return (cond == Equal) ? Below : AboveOrEqual;
   }
@@ -1622,6 +1771,13 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     splitSignExtTag(value, scratch);
     return testMagic(cond, scratch);
   }
+  Condition testGCThing(Condition cond, const ValueOperand& value) {
+    vixl::UseScratchRegisterScope temps(this);
+    const Register scratch = temps.AcquireX().asUnsized();
+    MOZ_ASSERT(value.valueReg() != scratch);
+    splitSignExtTag(value, scratch);
+    return testGCThing(cond, scratch);
+  }
   Condition testError(Condition cond, const ValueOperand& value) {
     return testMagic(cond, value);
   }
@@ -1764,14 +1920,12 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   Condition testBigIntTruthy(bool truthy, const ValueOperand& value) {
     vixl::UseScratchRegisterScope temps(this);
     const Register scratch = temps.AcquireX().asUnsized();
-    const ARMRegister scratch64(scratch, 64);
 
     MOZ_ASSERT(value.valueReg() != scratch);
 
     unboxBigInt(value, scratch);
-    Ldr(scratch64,
-        MemOperand(scratch64, BigInt::offsetOfLengthSignAndReservedBits()));
-    Cmp(scratch64, Operand(0));
+    load32(Address(scratch, BigInt::offsetOfDigitLength()), scratch);
+    cmp32(scratch, Imm32(0));
     return truthy ? Condition::NonZero : Condition::Zero;
   }
   Condition testInt32(Condition cond, const BaseIndex& src) {
@@ -1844,7 +1998,7 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     Label join;
     testInt32(Equal, ValueOperand(src));
     B(&isInt32, Equal);
-    // is double, move teh bits as is
+    // is double, move the bits as is
     Fmov(dest, ARMRegister(src, 64));
     B(&join);
     bind(&isInt32);
@@ -1859,8 +2013,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
       MOZ_ASSERT(scratch64.asUnsized() != address.base);
       Ldr(scratch64, toMemOperand(address));
       int32OrDouble(scratch64.asUnsized(), ARMFPRegister(dest.fpu(), 64));
-    } else if (type == MIRType::ObjectOrNull) {
-      unboxObjectOrNull(address, dest.gpr());
     } else {
       unboxNonDouble(address, dest.gpr(), ValueTypeFromMIRType(type));
     }
@@ -1874,8 +2026,6 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
       MOZ_ASSERT(scratch64.asUnsized() != address.index);
       doBaseIndex(scratch64, address, vixl::LDR_x);
       int32OrDouble(scratch64.asUnsized(), ARMFPRegister(dest.fpu(), 64));
-    } else if (type == MIRType::ObjectOrNull) {
-      unboxObjectOrNull(address, dest.gpr());
     } else {
       unboxNonDouble(address, dest.gpr(), ValueTypeFromMIRType(type));
     }
@@ -1894,6 +2044,8 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
 
   // load: offset to the load instruction obtained by movePatchablePtr().
   void writeDataRelocation(ImmGCPtr ptr, BufferOffset load) {
+    // Raw GC pointer relocations and Value relocations both end up in
+    // Assembler::TraceDataRelocations.
     if (ptr.value) {
       if (gc::IsInsideNursery(ptr.value)) {
         embedsNurseryPointers_ = true;
@@ -1902,6 +2054,8 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     }
   }
   void writeDataRelocation(const Value& val, BufferOffset load) {
+    // Raw GC pointer relocations and Value relocations both end up in
+    // Assembler::TraceDataRelocations.
     if (val.isGCThing()) {
       gc::Cell* cell = val.toGCThing();
       if (cell && gc::IsInsideNursery(cell)) {
@@ -1931,24 +2085,30 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
   }
 
  public:
-  CodeOffset labelForPatch() { return CodeOffset(nextOffset().getOffset()); }
-
-  void handleFailureWithHandlerTail(void* handler, Label* profilerExitTail);
+  void handleFailureWithHandlerTail(Label* profilerExitTail);
 
   void profilerEnterFrame(Register framePtr, Register scratch);
   void profilerEnterFrame(RegisterOrSP framePtr, Register scratch);
-  void profilerExitFrame() {
-    jump(GetJitContext()->runtime->jitRuntime()->getProfilerExitFrameTail());
-  }
+  void profilerExitFrame();
   Address ToPayload(Address value) { return value; }
   Address ToType(Address value) { return value; }
 
   void wasmLoadImpl(const wasm::MemoryAccessDesc& access, Register memoryBase,
-                    Register ptr, Register ptrScratch, AnyRegister outany,
-                    Register64 out64);
+                    Register ptr, AnyRegister outany, Register64 out64);
+  void wasmLoadImpl(const wasm::MemoryAccessDesc& access, MemOperand srcAddr,
+                    AnyRegister outany, Register64 out64);
   void wasmStoreImpl(const wasm::MemoryAccessDesc& access, AnyRegister valany,
-                     Register64 val64, Register memoryBase, Register ptr,
-                     Register ptrScratch);
+                     Register64 val64, Register memoryBase, Register ptr);
+  void wasmStoreImpl(const wasm::MemoryAccessDesc& access, MemOperand destAddr,
+                     AnyRegister valany, Register64 val64);
+  // The complete address is in `address`, and `access` is used for its type
+  // attributes only; its `offset` is ignored.
+  void wasmLoadAbsolute(const wasm::MemoryAccessDesc& access,
+                        Register memoryBase, uint64_t address, AnyRegister out,
+                        Register64 out64);
+  void wasmStoreAbsolute(const wasm::MemoryAccessDesc& access,
+                         AnyRegister value, Register64 value64,
+                         Register memoryBase, uint64_t address);
 
   // Emit a BLR or NOP instruction. ToggleCall can be used to patch
   // this instruction.
@@ -1957,6 +2117,10 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     // for the debugger to match offset with Baseline's pcMappingEntries_.
     BufferOffset offset = nextOffset();
 
+    // It is unclear why this sync is necessary:
+    // * PSP and SP have been observed to be different in testcase
+    //   tests/cacheir/bug1448136.js
+    // * Removing the sync causes no failures in all of jit-tests.
     syncStackPtr();
 
     BufferOffset loadOffset;
@@ -2080,7 +2244,7 @@ class MacroAssemblerCompat : public vixl::MacroAssembler {
     // Bfxil cannot be used with the zero register as a source.
     if (src == rzr) {
       And(ARMRegister(dest, 64), ARMRegister(dest, 64),
-          Operand(JSVAL_TAG_MASK));
+          Operand(JS::detail::ValueTagMask));
     } else {
       Bfxil(ARMRegister(dest, 64), ARMRegister(src, 64), 0, JSVAL_TAG_SHIFT);
     }
