@@ -10,6 +10,7 @@
 #include "js/ContextOptions.h"
 #include "js/SourceText.h"  // JS::Source{Ownership,Text}
 #include "jsapi-tests/tests.h"
+#include "util/Text.h"
 
 static TestJSPrincipals system_principals(1);
 
@@ -67,7 +68,7 @@ BEGIN_TEST(testChromeBuffer) {
       static const char bytes[] = "return x ? 1 + trusted(x-1) : 0";
 
       JS::SourceText<mozilla::Utf8Unit> srcBuf;
-      CHECK(srcBuf.init(cx, bytes, mozilla::ArrayLength(bytes) - 1,
+      CHECK(srcBuf.init(cx, bytes, js_strlen(bytes),
                         JS::SourceOwnership::Borrowed));
 
       JS::CompileOptions options(cx);
@@ -98,7 +99,7 @@ BEGIN_TEST(testChromeBuffer) {
         "}                                          ";
 
     JS::SourceText<mozilla::Utf8Unit> srcBuf;
-    CHECK(srcBuf.init(cx, bytes, mozilla::ArrayLength(bytes) - 1,
+    CHECK(srcBuf.init(cx, bytes, js_strlen(bytes),
                       JS::SourceOwnership::Borrowed));
 
     JS::CompileOptions options(cx);
@@ -141,7 +142,7 @@ BEGIN_TEST(testChromeBuffer) {
           "}                                      ";
 
       JS::SourceText<mozilla::Utf8Unit> srcBuf;
-      CHECK(srcBuf.init(cx, bytes, mozilla::ArrayLength(bytes) - 1,
+      CHECK(srcBuf.init(cx, bytes, js_strlen(bytes),
                         JS::SourceOwnership::Borrowed));
 
       JS::CompileOptions options(cx);
@@ -168,7 +169,7 @@ BEGIN_TEST(testChromeBuffer) {
         "}                                          ";
 
     JS::SourceText<mozilla::Utf8Unit> srcBuf;
-    CHECK(srcBuf.init(cx, bytes, mozilla::ArrayLength(bytes) - 1,
+    CHECK(srcBuf.init(cx, bytes, js_strlen(bytes),
                       JS::SourceOwnership::Borrowed));
 
     JS::CompileOptions options(cx);
@@ -202,7 +203,7 @@ BEGIN_TEST(testChromeBuffer) {
       static const char bytes[] = "return 42";
 
       JS::SourceText<mozilla::Utf8Unit> srcBuf;
-      CHECK(srcBuf.init(cx, bytes, mozilla::ArrayLength(bytes) - 1,
+      CHECK(srcBuf.init(cx, bytes, js_strlen(bytes),
                         JS::SourceOwnership::Borrowed));
 
       JS::CompileOptions options(cx);
@@ -230,7 +231,7 @@ BEGIN_TEST(testChromeBuffer) {
         "}                                          ";
 
     JS::SourceText<mozilla::Utf8Unit> srcBuf;
-    CHECK(srcBuf.init(cx, bytes, mozilla::ArrayLength(bytes) - 1,
+    CHECK(srcBuf.init(cx, bytes, js_strlen(bytes),
                       JS::SourceOwnership::Borrowed));
 
     JS::CompileOptions options(cx);
