@@ -31,10 +31,6 @@ struct GCManagedObjectWeakMap : public ObjectWeakMap {
 
 namespace JS {
 
-template <>
-struct DeletePolicy<js::GCManagedObjectWeakMap>
-    : public js::GCManagedDeletePolicy<js::GCManagedObjectWeakMap> {};
-
 template <>
 struct MapTypeToRootKind<js::GCManagedObjectWeakMap*> {
   static const JS::RootKind kind = JS::RootKind::Traceable;
@@ -54,9 +50,7 @@ class AutoNoAnalysisForTest {
 BEGIN_TEST(testGCGrayMarking) {
   AutoNoAnalysisForTest disableAnalysis;
   AutoDisableCompactingGC disableCompactingGC(cx);
-#ifdef JS_GC_ZEAL
   AutoLeaveZeal nozeal(cx);
-#endif /* JS_GC_ZEAL */
 
   CHECK(InitGlobals());
   JSAutoRealm ar(cx, global1);
@@ -64,13 +58,12 @@ BEGIN_TEST(testGCGrayMarking) {
   InitGrayRootTracer();
 
   // Enable incremental GC.
-  JS_SetGCParameter(cx, JSGC_MODE, JSGC_MODE_ZONE_INCREMENTAL);
+  AutoGCParameter param1(cx, JSGC_INCREMENTAL_GC_ENABLED, true);
+  AutoGCParameter param2(cx, JSGC_PER_ZONE_GC_ENABLED, true);
 
   bool ok = TestMarking() && TestJSWeakMaps() && TestInternalWeakMaps() &&
             TestCCWs() && TestGrayUnmarking();
 
-  JS_SetGCParameter(cx, JSGC_MODE, JSGC_MODE_GLOBAL);
-
   global1 = nullptr;
   global2 = nullptr;
   RemoveGrayRootTracer();
@@ -286,7 +279,8 @@ bool TestJSWeakMapWithGrayUnmarking(MarkKeyOrDelegate markKey,
     // Start an incremental GC and run until gray roots have been pushed onto
     // the mark stack.
     JS::PrepareForFullGC(cx);
-    JS::StartIncrementalGC(cx, GC_NORMAL, JS::GCReason::DEBUG_GC, 1000000);
+    JS::StartIncrementalGC(cx, JS::GCOptions::Normal, JS::GCReason::DEBUG_GC,
+                           1000000);
     MOZ_ASSERT(cx->runtime()->gc.state() == gc::State::Sweep);
     MOZ_ASSERT(cx->zone()->gcState() == Zone::MarkBlackAndGray);
 
@@ -413,7 +407,8 @@ bool TestInternalWeakMapWithGrayUnmarking(CellColor keyMarkColor,
     // Start an incremental GC and run until gray roots have been pushed onto
     // the mark stack.
     JS::PrepareForFullGC(cx);
-    JS::StartIncrementalGC(cx, GC_NORMAL, JS::GCReason::DEBUG_GC, 1000000);
+    JS::StartIncrementalGC(cx, JS::GCOptions::Normal, JS::GCReason::DEBUG_GC,
+                           1000000);
     MOZ_ASSERT(cx->runtime()->gc.state() == gc::State::Sweep);
     MOZ_ASSERT(cx->zone()->gcState() == Zone::MarkBlackAndGray);
 
@@ -501,10 +496,13 @@ bool TestCCWs() {
   CHECK(IsMarkedGray(wrapper));
   CHECK(IsMarkedBlack(target));
 
-  JS_SetGCParameter(cx, JSGC_MODE, JSGC_MODE_ZONE_INCREMENTAL);
+  JSRuntime* rt = cx->runtime();
   JS::PrepareForFullGC(cx);
   js::SliceBudget budget(js::WorkBudget(1));
-  cx->runtime()->gc.startDebugGC(GC_NORMAL, budget);
+  rt->gc.startDebugGC(JS::GCOptions::Normal, budget);
+  while (rt->gc.state() == gc::State::Prepare) {
+    rt->gc.debugGCSlice(budget);
+  }
   CHECK(JS::IsIncrementalGCInProgress(cx));
 
   CHECK(!IsMarkedBlack(wrapper));
@@ -527,10 +525,12 @@ bool TestCCWs() {
   CHECK(IsMarkedGray(target));
 
   // Incremental zone GC started: the source is now unmarked.
-  JS_SetGCParameter(cx, JSGC_MODE, JSGC_MODE_ZONE_INCREMENTAL);
   JS::PrepareZoneForGC(cx, wrapper->zone());
   budget = js::SliceBudget(js::WorkBudget(1));
-  cx->runtime()->gc.startDebugGC(GC_NORMAL, budget);
+  rt->gc.startDebugGC(JS::GCOptions::Normal, budget);
+  while (rt->gc.state() == gc::State::Prepare) {
+    rt->gc.debugGCSlice(budget);
+  }
   CHECK(JS::IsIncrementalGCInProgress(cx));
   CHECK(wrapper->zone()->isGCMarkingBlackOnly());
   CHECK(!target->zone()->wasGCStarted());
@@ -613,7 +613,8 @@ struct ColorCheckFunctor {
     }
 
     // Shapes and symbols are never marked gray.
-    jsid id = shape->propid();
+    ShapePropertyIter<NoGC> iter(shape);
+    jsid id = iter->key();
     if (id.isGCThing() &&
         !CheckCellColor(id.toGCCellPtr().asCell(), MarkColor::Black)) {
       return false;
@@ -788,12 +789,9 @@ static bool CheckCellColor(Cell* cell, MarkColor color) {
 void EvictNursery() { cx->runtime()->gc.evictNursery(); }
 
 bool ZoneGC(JS::Zone* zone) {
-  uint32_t oldMode = JS_GetGCParameter(cx, JSGC_MODE);
-  JS_SetGCParameter(cx, JSGC_MODE, JSGC_MODE_ZONE);
   JS::PrepareZoneForGC(cx, zone);
-  cx->runtime()->gc.gc(GC_NORMAL, JS::GCReason::API);
+  cx->runtime()->gc.gc(JS::GCOptions::Normal, JS::GCReason::API);
   CHECK(!cx->runtime()->gc.isFullGc());
-  JS_SetGCParameter(cx, JSGC_MODE, oldMode);
   return true;
 }
 
