@@ -20,9 +20,7 @@ static void MinimizeHeap(JSContext* cx) {
 }
 
 BEGIN_TEST(testGCUID) {
-#ifdef JS_GC_ZEAL
   AutoLeaveZeal nozeal(cx);
-#endif /* JS_GC_ZEAL */
 
   uint64_t uid = 0;
   uint64_t tmp = 0;
@@ -109,7 +107,7 @@ BEGIN_TEST(testGCUID) {
   // Force a compaction to move the object and check that the uid moved to
   // the new tenured heap location.
   JS::PrepareForFullGC(cx);
-  JS::NonIncrementalGC(cx, GC_SHRINK, JS::GCReason::API);
+  JS::NonIncrementalGC(cx, JS::GCOptions::Shrink, JS::GCReason::API);
 
   // There's a very low probability that this check could fail, but it is
   // possible.  If it becomes an annoying intermittent then we should make
