@@ -7,15 +7,21 @@
 #include "js/Proxy.h"
 
 #include "mozilla/Attributes.h"
+#include "mozilla/Maybe.h"
 
 #include <string.h>
 
 #include "jsapi.h"
 
+#include "js/friend/ErrorMessages.h"  // JSMSG_*
+#include "js/friend/StackLimits.h"  // js::AutoCheckRecursionLimit, js::GetNativeStackLimit
+#include "js/friend/WindowProxy.h"  // js::IsWindow, js::IsWindowProxy, js::ToWindowProxyIfWindow
 #include "js/PropertySpec.h"
+#include "js/Value.h"  // JS::ObjectValue
 #include "js/Wrapper.h"
 #include "proxy/DeadObjectProxy.h"
 #include "proxy/ScriptedProxyHandler.h"
+#include "vm/Interpreter.h"  // js::CallGetter
 #include "vm/JSContext.h"
 #include "vm/JSFunction.h"
 #include "vm/JSObject.h"
@@ -28,6 +34,99 @@
 
 using namespace js;
 
+// Used by private fields to manipulate the ProxyExpando:
+// All the following methods are called iff the handler for the proxy
+// returns true for useProxyExpandoObjectForPrivateFields.
+static bool ProxySetOnExpando(JSContext* cx, HandleObject proxy, HandleId id,
+                              HandleValue v, HandleValue receiver,
+                              ObjectOpResult& result) {
+  MOZ_ASSERT(id.isPrivateName());
+
+  // For BaseProxyHandler, private names are stored in the expando object.
+  RootedObject expando(cx, proxy->as<ProxyObject>().expando().toObjectOrNull());
+
+  // SetPrivateElementOperation checks for hasOwn first, which ensures the
+  // expando exsists.
+  MOZ_ASSERT(expando);
+
+  Rooted<mozilla::Maybe<PropertyDescriptor>> ownDesc(cx);
+  if (!GetOwnPropertyDescriptor(cx, expando, id, &ownDesc)) {
+    return false;
+  }
+  MOZ_ASSERT(ownDesc.isSome());
+
+  RootedValue expandoValue(cx, proxy->as<ProxyObject>().expando());
+  return SetPropertyIgnoringNamedGetter(cx, expando, id, v, expandoValue,
+                                        ownDesc, result);
+}
+
+static bool ProxyGetOnExpando(JSContext* cx, HandleObject proxy,
+                              HandleValue receiver, HandleId id,
+                              MutableHandleValue vp) {
+  // For BaseProxyHandler, private names are stored in the expando object.
+  RootedObject expando(cx, proxy->as<ProxyObject>().expando().toObjectOrNull());
+
+  // We must have the expando, or GetPrivateElemOperation didn't call
+  // hasPrivate first.
+  MOZ_ASSERT(expando);
+
+  // Because we controlled the creation of the expando, we know it's not a
+  // proxy, and so can safely call internal methods on it without worrying about
+  // exposing information about private names.
+  Rooted<mozilla::Maybe<PropertyDescriptor>> desc(cx);
+  if (!GetOwnPropertyDescriptor(cx, expando, id, &desc)) {
+    return false;
+  }
+  // We must have the object, same reasoning as the expando.
+  MOZ_ASSERT(desc.isSome());
+
+  // If the private name has a getter, delegate to that.
+  if (desc->hasGetter()) {
+    RootedValue getter(cx, JS::ObjectValue(*desc->getter()));
+    return js::CallGetter(cx, receiver, getter, vp);
+  }
+
+  MOZ_ASSERT(desc->hasValue());
+  MOZ_ASSERT(desc->isDataDescriptor());
+
+  vp.set(desc->value());
+  return true;
+}
+
+static bool ProxyHasOnExpando(JSContext* cx, HandleObject proxy, HandleId id,
+                              bool* bp) {
+  // For BaseProxyHandler, private names are stored in the expando object.
+  RootedObject expando(cx, proxy->as<ProxyObject>().expando().toObjectOrNull());
+
+  // If there is no expando object, then there is no private field.
+  if (!expando) {
+    *bp = false;
+    return true;
+  }
+
+  return HasOwnProperty(cx, expando, id, bp);
+}
+
+static bool ProxyDefineOnExpando(JSContext* cx, HandleObject proxy, HandleId id,
+                                 Handle<PropertyDescriptor> desc,
+                                 ObjectOpResult& result) {
+  MOZ_ASSERT(id.isPrivateName());
+
+  // For BaseProxyHandler, private names are stored in the expando object.
+  RootedObject expando(cx, proxy->as<ProxyObject>().expando().toObjectOrNull());
+
+  if (!expando) {
+    expando = NewObjectWithGivenProto<PlainObject>(cx, nullptr);
+    if (!expando) {
+      return false;
+    }
+
+    proxy->as<ProxyObject>().setExpando(expando);
+  }
+
+  return DefineProperty(cx, expando, id, desc, result);
+}
+
 void js::AutoEnterPolicy::reportErrorIfExceptionIsNotPending(JSContext* cx,
                                                              HandleId id) {
   if (JS_IsExceptionPending(cx)) {
@@ -61,7 +160,7 @@ void js::AutoEnterPolicy::recordLeave() {
   }
 }
 
-JS_FRIEND_API void js::assertEnteredPolicy(JSContext* cx, JSObject* proxy,
+JS_PUBLIC_API void js::assertEnteredPolicy(JSContext* cx, JSObject* proxy,
                                            jsid id,
                                            BaseProxyHandler::Action act) {
   MOZ_ASSERT(proxy->is<ProxyObject>());
@@ -72,27 +171,33 @@ JS_FRIEND_API void js::assertEnteredPolicy(JSContext* cx, JSObject* proxy,
 }
 #endif
 
-bool Proxy::getOwnPropertyDescriptor(JSContext* cx, HandleObject proxy,
-                                     HandleId id,
-                                     MutableHandle<PropertyDescriptor> desc) {
-  if (!CheckRecursionLimit(cx)) {
+bool Proxy::getOwnPropertyDescriptor(
+    JSContext* cx, HandleObject proxy, HandleId id,
+    MutableHandle<mozilla::Maybe<PropertyDescriptor>> desc) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   const BaseProxyHandler* handler = proxy->as<ProxyObject>().handler();
-  desc.object().set(
-      nullptr);  // default result if we refuse to perform this action
+  desc.reset();  // default result if we refuse to perform this action
   AutoEnterPolicy policy(cx, handler, proxy, id,
                          BaseProxyHandler::GET_PROPERTY_DESCRIPTOR, true);
   if (!policy.allowed()) {
     return policy.returnValue();
   }
+
+  // Unless we implment ProxyGetOwnPropertyDescriptorFromExpando,
+  // this would be incorrect.
+  MOZ_ASSERT_IF(handler->useProxyExpandoObjectForPrivateFields(),
+                !id.isPrivateName());
   return handler->getOwnPropertyDescriptor(cx, proxy, id, desc);
 }
 
 bool Proxy::defineProperty(JSContext* cx, HandleObject proxy, HandleId id,
                            Handle<PropertyDescriptor> desc,
                            ObjectOpResult& result) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   const BaseProxyHandler* handler = proxy->as<ProxyObject>().handler();
@@ -103,13 +208,23 @@ bool Proxy::defineProperty(JSContext* cx, HandleObject proxy, HandleId id,
     }
     return result.succeed();
   }
+
+  // Private field accesses have different semantics depending on the kind
+  // of proxy involved, and so take a different path compared to regular
+  // [[Get]] operations. For example, scripted handlers don't fire traps
+  // when accessing private fields (because of the WeakMap semantics)
+  if (id.isPrivateName() && handler->useProxyExpandoObjectForPrivateFields()) {
+    return ProxyDefineOnExpando(cx, proxy, id, desc, result);
+  }
+
   return proxy->as<ProxyObject>().handler()->defineProperty(cx, proxy, id, desc,
                                                             result);
 }
 
 bool Proxy::ownPropertyKeys(JSContext* cx, HandleObject proxy,
                             MutableHandleIdVector props) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   const BaseProxyHandler* handler = proxy->as<ProxyObject>().handler();
@@ -123,7 +238,8 @@ bool Proxy::ownPropertyKeys(JSContext* cx, HandleObject proxy,
 
 bool Proxy::delete_(JSContext* cx, HandleObject proxy, HandleId id,
                     ObjectOpResult& result) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   const BaseProxyHandler* handler = proxy->as<ProxyObject>().handler();
@@ -135,10 +251,15 @@ bool Proxy::delete_(JSContext* cx, HandleObject proxy, HandleId id,
     }
     return ok;
   }
+
+  // Private names shouldn't take this path, as deleting a private name
+  // should be a syntax error.
+  MOZ_ASSERT(!id.isPrivateName());
+
   return proxy->as<ProxyObject>().handler()->delete_(cx, proxy, id, result);
 }
 
-JS_FRIEND_API bool js::AppendUnique(JSContext* cx, MutableHandleIdVector base,
+JS_PUBLIC_API bool js::AppendUnique(JSContext* cx, MutableHandleIdVector base,
                                     HandleIdVector others) {
   RootedIdVector uniqueOthers(cx);
   if (!uniqueOthers.reserve(others.length())) {
@@ -158,14 +279,15 @@ JS_FRIEND_API bool js::AppendUnique(JSContext* cx, MutableHandleIdVector base,
       }
     }
   }
-  return base.appendAll(uniqueOthers);
+  return base.appendAll(std::move(uniqueOthers));
 }
 
 /* static */
 bool Proxy::getPrototype(JSContext* cx, HandleObject proxy,
                          MutableHandleObject proto) {
   MOZ_ASSERT(proxy->hasDynamicPrototype());
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   return proxy->as<ProxyObject>().handler()->getPrototype(cx, proxy, proto);
@@ -175,7 +297,8 @@ bool Proxy::getPrototype(JSContext* cx, HandleObject proxy,
 bool Proxy::setPrototype(JSContext* cx, HandleObject proxy, HandleObject proto,
                          ObjectOpResult& result) {
   MOZ_ASSERT(proxy->hasDynamicPrototype());
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   return proxy->as<ProxyObject>().handler()->setPrototype(cx, proxy, proto,
@@ -186,7 +309,8 @@ bool Proxy::setPrototype(JSContext* cx, HandleObject proxy, HandleObject proto,
 bool Proxy::getPrototypeIfOrdinary(JSContext* cx, HandleObject proxy,
                                    bool* isOrdinary,
                                    MutableHandleObject proto) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   return proxy->as<ProxyObject>().handler()->getPrototypeIfOrdinary(
@@ -196,7 +320,8 @@ bool Proxy::getPrototypeIfOrdinary(JSContext* cx, HandleObject proxy,
 /* static */
 bool Proxy::setImmutablePrototype(JSContext* cx, HandleObject proxy,
                                   bool* succeeded) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   const BaseProxyHandler* handler = proxy->as<ProxyObject>().handler();
@@ -206,7 +331,8 @@ bool Proxy::setImmutablePrototype(JSContext* cx, HandleObject proxy,
 /* static */
 bool Proxy::preventExtensions(JSContext* cx, HandleObject proxy,
                               ObjectOpResult& result) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   const BaseProxyHandler* handler = proxy->as<ProxyObject>().handler();
@@ -215,7 +341,8 @@ bool Proxy::preventExtensions(JSContext* cx, HandleObject proxy,
 
 /* static */
 bool Proxy::isExtensible(JSContext* cx, HandleObject proxy, bool* extensible) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   return proxy->as<ProxyObject>().handler()->isExtensible(cx, proxy,
@@ -223,7 +350,8 @@ bool Proxy::isExtensible(JSContext* cx, HandleObject proxy, bool* extensible) {
 }
 
 bool Proxy::has(JSContext* cx, HandleObject proxy, HandleId id, bool* bp) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   const BaseProxyHandler* handler = proxy->as<ProxyObject>().handler();
@@ -233,6 +361,9 @@ bool Proxy::has(JSContext* cx, HandleObject proxy, HandleId id, bool* bp) {
     return policy.returnValue();
   }
 
+  // Private names shouldn't take this path, but only hasOwn;
+  MOZ_ASSERT(!id.isPrivateName());
+
   if (handler->hasPrototype()) {
     if (!handler->hasOwn(cx, proxy, id, bp)) {
       return false;
@@ -258,7 +389,7 @@ bool Proxy::has(JSContext* cx, HandleObject proxy, HandleId id, bool* bp) {
 bool js::ProxyHas(JSContext* cx, HandleObject proxy, HandleValue idVal,
                   bool* result) {
   RootedId id(cx);
-  if (!ValueToId<CanGC>(cx, idVal, &id)) {
+  if (!ToPropertyKey(cx, idVal, &id)) {
     return false;
   }
 
@@ -266,7 +397,8 @@ bool js::ProxyHas(JSContext* cx, HandleObject proxy, HandleValue idVal,
 }
 
 bool Proxy::hasOwn(JSContext* cx, HandleObject proxy, HandleId id, bool* bp) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   const BaseProxyHandler* handler = proxy->as<ProxyObject>().handler();
@@ -275,13 +407,22 @@ bool Proxy::hasOwn(JSContext* cx, HandleObject proxy, HandleId id, bool* bp) {
   if (!policy.allowed()) {
     return policy.returnValue();
   }
+
+  // Private field accesses have different semantics depending on the kind
+  // of proxy involved, and so take a different path compared to regular
+  // [[Get]] operations. For example, scripted handlers don't fire traps
+  // when accessing private fields (because of the WeakMap semantics)
+  if (id.isPrivateName() && handler->useProxyExpandoObjectForPrivateFields()) {
+    return ProxyHasOnExpando(cx, proxy, id, bp);
+  }
+
   return handler->hasOwn(cx, proxy, id, bp);
 }
 
 bool js::ProxyHasOwn(JSContext* cx, HandleObject proxy, HandleValue idVal,
                      bool* result) {
   RootedId id(cx);
-  if (!ValueToId<CanGC>(cx, idVal, &id)) {
+  if (!ToPropertyKey(cx, idVal, &id)) {
     return false;
   }
 
@@ -301,7 +442,8 @@ MOZ_ALWAYS_INLINE bool Proxy::getInternal(JSContext* cx, HandleObject proxy,
                                           MutableHandleValue vp) {
   MOZ_ASSERT_IF(receiver.isObject(), !IsWindow(&receiver.toObject()));
 
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   const BaseProxyHandler* handler = proxy->as<ProxyObject>().handler();
@@ -311,6 +453,14 @@ MOZ_ALWAYS_INLINE bool Proxy::getInternal(JSContext* cx, HandleObject proxy,
     return policy.returnValue();
   }
 
+  // Private field accesses have different semantics depending on the kind
+  // of proxy involved, and so take a different path compared to regular
+  // [[Get]] operations. For example, scripted handlers don't fire traps
+  // when accessing private fields (because of the WeakMap semantics)
+  if (id.isPrivateName() && handler->useProxyExpandoObjectForPrivateFields()) {
+    return ProxyGetOnExpando(cx, proxy, receiver, id, vp);
+  }
+
   if (handler->hasPrototype()) {
     bool own;
     if (!handler->hasOwn(cx, proxy, id, &own)) {
@@ -348,7 +498,7 @@ bool js::ProxyGetProperty(JSContext* cx, HandleObject proxy, HandleId id,
 bool js::ProxyGetPropertyByValue(JSContext* cx, HandleObject proxy,
                                  HandleValue idVal, MutableHandleValue vp) {
   RootedId id(cx);
-  if (!ValueToId<CanGC>(cx, idVal, &id)) {
+  if (!ToPropertyKey(cx, idVal, &id)) {
     return false;
   }
 
@@ -362,9 +512,11 @@ MOZ_ALWAYS_INLINE bool Proxy::setInternal(JSContext* cx, HandleObject proxy,
                                           ObjectOpResult& result) {
   MOZ_ASSERT_IF(receiver.isObject(), !IsWindow(&receiver.toObject()));
 
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
+
   const BaseProxyHandler* handler = proxy->as<ProxyObject>().handler();
   AutoEnterPolicy policy(cx, handler, proxy, id, BaseProxyHandler::SET, true);
   if (!policy.allowed()) {
@@ -374,6 +526,16 @@ MOZ_ALWAYS_INLINE bool Proxy::setInternal(JSContext* cx, HandleObject proxy,
     return result.succeed();
   }
 
+  // Private field accesses have different semantics depending on the kind
+  // of proxy involved, and so take a different path compared to regular
+  // [[Set]] operations.
+  //
+  // This doesn't interact with hasPrototype, as PrivateFields are always
+  // own propertiers, and so we never deal with prototype traversals.
+  if (id.isPrivateName() && handler->useProxyExpandoObjectForPrivateFields()) {
+    return ProxySetOnExpando(cx, proxy, id, v, receiver, result);
+  }
+
   // Special case. See the comment on BaseProxyHandler::mHasPrototype.
   if (handler->hasPrototype()) {
     return handler->BaseProxyHandler::set(cx, proxy, id, v, receiver, result);
@@ -404,7 +566,7 @@ bool js::ProxySetPropertyByValue(JSContext* cx, HandleObject proxy,
                                  HandleValue idVal, HandleValue val,
                                  bool strict) {
   RootedId id(cx);
-  if (!ValueToId<CanGC>(cx, idVal, &id)) {
+  if (!ToPropertyKey(cx, idVal, &id)) {
     return false;
   }
 
@@ -418,7 +580,8 @@ bool js::ProxySetPropertyByValue(JSContext* cx, HandleObject proxy,
 
 bool Proxy::getOwnEnumerablePropertyKeys(JSContext* cx, HandleObject proxy,
                                          MutableHandleIdVector props) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   const BaseProxyHandler* handler = proxy->as<ProxyObject>().handler();
@@ -432,7 +595,8 @@ bool Proxy::getOwnEnumerablePropertyKeys(JSContext* cx, HandleObject proxy,
 
 bool Proxy::enumerate(JSContext* cx, HandleObject proxy,
                       MutableHandleIdVector props) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
 
@@ -473,7 +637,8 @@ bool Proxy::enumerate(JSContext* cx, HandleObject proxy,
 }
 
 bool Proxy::call(JSContext* cx, HandleObject proxy, const CallArgs& args) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   const BaseProxyHandler* handler = proxy->as<ProxyObject>().handler();
@@ -492,7 +657,8 @@ bool Proxy::call(JSContext* cx, HandleObject proxy, const CallArgs& args) {
 }
 
 bool Proxy::construct(JSContext* cx, HandleObject proxy, const CallArgs& args) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   const BaseProxyHandler* handler = proxy->as<ProxyObject>().handler();
@@ -512,7 +678,8 @@ bool Proxy::construct(JSContext* cx, HandleObject proxy, const CallArgs& args) {
 
 bool Proxy::nativeCall(JSContext* cx, IsAcceptableThis test, NativeImpl impl,
                        const CallArgs& args) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   RootedObject proxy(cx, &args.thisv().toObject());
@@ -524,7 +691,8 @@ bool Proxy::nativeCall(JSContext* cx, IsAcceptableThis test, NativeImpl impl,
 
 bool Proxy::hasInstance(JSContext* cx, HandleObject proxy, MutableHandleValue v,
                         bool* bp) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   const BaseProxyHandler* handler = proxy->as<ProxyObject>().handler();
@@ -538,7 +706,8 @@ bool Proxy::hasInstance(JSContext* cx, HandleObject proxy, MutableHandleValue v,
 }
 
 bool Proxy::getBuiltinClass(JSContext* cx, HandleObject proxy, ESClass* cls) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   return proxy->as<ProxyObject>().handler()->getBuiltinClass(cx, proxy, cls);
@@ -546,7 +715,8 @@ bool Proxy::getBuiltinClass(JSContext* cx, HandleObject proxy, ESClass* cls) {
 
 bool Proxy::isArray(JSContext* cx, HandleObject proxy,
                     JS::IsArrayAnswer* answer) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   return proxy->as<ProxyObject>().handler()->isArray(cx, proxy, answer);
@@ -555,8 +725,8 @@ bool Proxy::isArray(JSContext* cx, HandleObject proxy,
 const char* Proxy::className(JSContext* cx, HandleObject proxy) {
   // Check for unbounded recursion, but don't signal an error; className
   // needs to be infallible.
-  int stackDummy;
-  if (!JS_CHECK_STACK_SIZE(GetNativeStackLimit(cx), &stackDummy)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.checkDontReport(cx)) {
     return "too much recursion";
   }
 
@@ -572,7 +742,8 @@ const char* Proxy::className(JSContext* cx, HandleObject proxy) {
 
 JSString* Proxy::fun_toString(JSContext* cx, HandleObject proxy,
                               bool isToSource) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return nullptr;
   }
   const BaseProxyHandler* handler = proxy->as<ProxyObject>().handler();
@@ -586,7 +757,8 @@ JSString* Proxy::fun_toString(JSContext* cx, HandleObject proxy,
 }
 
 RegExpShared* Proxy::regexp_toShared(JSContext* cx, HandleObject proxy) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return nullptr;
   }
   return proxy->as<ProxyObject>().handler()->regexp_toShared(cx, proxy);
@@ -594,7 +766,8 @@ RegExpShared* Proxy::regexp_toShared(JSContext* cx, HandleObject proxy) {
 
 bool Proxy::boxedValue_unbox(JSContext* cx, HandleObject proxy,
                              MutableHandleValue vp) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   return proxy->as<ProxyObject>().handler()->boxedValue_unbox(cx, proxy, vp);
@@ -605,7 +778,8 @@ JSObject* const TaggedProto::LazyProto = reinterpret_cast<JSObject*>(0x1);
 /* static */
 bool Proxy::getElements(JSContext* cx, HandleObject proxy, uint32_t begin,
                         uint32_t end, ElementAdder* adder) {
-  if (!CheckRecursionLimit(cx)) {
+  AutoCheckRecursionLimit recursion(cx);
+  if (!recursion.check(cx)) {
     return false;
   }
   const BaseProxyHandler* handler = proxy->as<ProxyObject>().handler();
@@ -630,17 +804,17 @@ void Proxy::trace(JSTracer* trc, JSObject* proxy) {
 
 static bool proxy_LookupProperty(JSContext* cx, HandleObject obj, HandleId id,
                                  MutableHandleObject objp,
-                                 MutableHandle<JS::PropertyResult> propp) {
+                                 PropertyResult* propp) {
   bool found;
   if (!Proxy::has(cx, obj, id, &found)) {
     return false;
   }
 
   if (found) {
-    propp.setNonNativeProperty();
+    propp->setProxyProperty();
     objp.set(obj);
   } else {
-    propp.setNotFound();
+    propp->setNotFound();
     objp.set(nullptr);
   }
   return true;
@@ -659,25 +833,36 @@ void ProxyObject::traceEdgeToTarget(JSTracer* trc, ProxyObject* obj) {
   TraceCrossCompartmentEdge(trc, obj, obj->slotOfPrivate(), "proxy target");
 }
 
+#ifdef DEBUG
+static inline void CheckProxyIsInCCWMap(ProxyObject* proxy) {
+  if (proxy->zone()->isGCCompacting()) {
+    // Skip this check during compacting GC since objects' object groups may be
+    // forwarded. It's not impossible to make this work, but requires adding a
+    // parallel lookupWrapper() path for this one case.
+    return;
+  }
+
+  JSObject* referent = MaybeForwarded(proxy->target());
+  if (referent->compartment() != proxy->compartment()) {
+    // Assert that this proxy is tracked in the wrapper map. We maintain the
+    // invariant that the wrapped object is the key in the wrapper map.
+    ObjectWrapperMap::Ptr p = proxy->compartment()->lookupWrapper(referent);
+    MOZ_ASSERT(p);
+    MOZ_ASSERT(*p->value().unsafeGet() == proxy);
+  }
+}
+#endif
+
 /* static */
 void ProxyObject::trace(JSTracer* trc, JSObject* obj) {
   ProxyObject* proxy = &obj->as<ProxyObject>();
 
-  TraceEdge(trc, proxy->shapePtr(), "ProxyObject_shape");
+  TraceNullableEdge(trc, proxy->slotOfExpando(), "expando");
 
 #ifdef DEBUG
   if (TlsContext.get()->isStrictProxyCheckingEnabled() &&
       proxy->is<WrapperObject>()) {
-    JSObject* referent = MaybeForwarded(proxy->target());
-    if (referent->compartment() != proxy->compartment()) {
-      /*
-       * Assert that this proxy is tracked in the wrapper map. We maintain
-       * the invariant that the wrapped object is the key in the wrapper map.
-       */
-      ObjectWrapperMap::Ptr p = proxy->compartment()->lookupWrapper(referent);
-      MOZ_ASSERT(p);
-      MOZ_ASSERT(*p->value().unsafeGet() == proxy);
-    }
+    CheckProxyIsInCCWMap(proxy);
   }
 #endif
 
@@ -771,7 +956,7 @@ const JSClass js::ProxyClass = PROXY_CLASS_DEF_WITH_CLASS_SPEC(
     JSCLASS_HAS_CACHED_PROTO(JSProto_Proxy) | JSCLASS_HAS_RESERVED_SLOTS(2),
     &ProxyClassSpec);
 
-JS_FRIEND_API JSObject* js::NewProxyObject(JSContext* cx,
+JS_PUBLIC_API JSObject* js::NewProxyObject(JSContext* cx,
                                            const BaseProxyHandler* handler,
                                            HandleValue priv, JSObject* proto_,
                                            const ProxyOptions& options) {
@@ -796,30 +981,6 @@ JS_FRIEND_API JSObject* js::NewProxyObject(JSContext* cx,
                           options.clasp());
 }
 
-JS_FRIEND_API JSObject* js::NewSingletonProxyObject(
-    JSContext* cx, const BaseProxyHandler* handler, HandleValue priv,
-    JSObject* proto_, const ProxyOptions& options) {
-  AssertHeapIsIdle();
-  CHECK_THREAD(cx);
-
-  // This can be called from the compartment wrap hooks while in a realm with a
-  // gray global. Trigger the read barrier on the global to ensure this is
-  // unmarked.
-  cx->realm()->maybeGlobal();
-
-  if (proto_ != TaggedProto::LazyProto) {
-    cx->check(proto_);  // |priv| might be cross-compartment.
-  }
-
-  if (options.lazyProto()) {
-    MOZ_ASSERT(!proto_);
-    proto_ = TaggedProto::LazyProto;
-  }
-
-  return ProxyObject::NewSingleton(cx, handler, priv, TaggedProto(proto_),
-                                   options.clasp());
-}
-
 void ProxyObject::renew(const BaseProxyHandler* handler, const Value& priv) {
   MOZ_ASSERT(!IsInsideNursery(this));
   MOZ_ASSERT_IF(IsCrossCompartmentWrapper(this), IsDeadProxyObject(this));
