@@ -18,6 +18,10 @@
 #  include <process.h>
 #  include <string.h>
 #  include <windows.h>
+#elif __wasi__
+#  include <dirent.h>
+#  include <sys/types.h>
+#  include <unistd.h>
 #else
 #  include <dirent.h>
 #  include <sys/types.h>
@@ -32,10 +36,14 @@
 #include "gc/FreeOp.h"
 #include "js/CharacterEncoding.h"
 #include "js/Conversions.h"
+#include "js/experimental/TypedData.h"  // JS_NewUint8Array
+#include "js/Object.h"                  // JS::GetReservedSlot
 #include "js/PropertySpec.h"
+#include "js/Value.h"  // JS::Value
 #include "js/Wrapper.h"
 #include "shell/jsshell.h"
 #include "shell/StringUtils.h"
+#include "util/GetPidProvider.h"  // getpid()
 #include "util/StringBuffer.h"
 #include "util/Text.h"
 #include "util/Windows.h"
@@ -49,6 +57,8 @@
 #    define PATH_MAX (MAX_PATH > _MAX_DIR ? MAX_PATH : _MAX_DIR)
 #  endif
 #  define getcwd _getcwd
+#elif defined(__wasi__)
+// Nothing.
 #else
 #  include <libgen.h>
 #endif
@@ -95,6 +105,9 @@ JSString* ResolvePath(JSContext* cx, HandleString filenameStr,
   if (!filenameStr) {
 #ifdef XP_WIN
     return JS_NewStringCopyZ(cx, "nul");
+#elif defined(__wasi__)
+    MOZ_CRASH("NYI for WASI");
+    return nullptr;
 #else
     return JS_NewStringCopyZ(cx, "/dev/null");
 #endif
@@ -142,9 +155,22 @@ JSString* ResolvePath(JSContext* cx, HandleString filenameStr,
       return nullptr;
     }
 
+#  ifdef __wasi__
+    // dirname() seems not to behave properly with wasi-libc; so we do our own
+    // simple thing here.
+    char* p = buffer + strlen(buffer);
+    while (p > buffer) {
+      if (*p == '/') {
+        *p = '\0';
+        break;
+      }
+      p--;
+    }
+#  else
     // dirname(buffer) might return buffer, or it might return a
     // statically-allocated string
     memmove(buffer, dirname(buffer), strlen(buffer) + 1);
+#  endif
 #endif
   } else {
     const char* cwd = getcwd(buffer, PATH_MAX);
@@ -197,7 +223,7 @@ JSObject* FileAsTypedArray(JSContext* cx, JS::HandleString pathnameStr) {
       }
       JS_ReportErrorUTF8(cx, "can't seek start of %s", pathname.get());
     } else {
-      if (len > ArrayBufferObject::MaxBufferByteLength) {
+      if (len > ArrayBufferObject::maxBufferByteLength()) {
         JS_ReportErrorUTF8(cx, "file %s is too large for a Uint8Array",
                            pathname.get());
         return nullptr;
@@ -322,7 +348,8 @@ static bool osfile_readRelativeToScript(JSContext* cx, unsigned argc,
   return ReadFile(cx, argc, vp, true);
 }
 
-static bool osfile_listDir(JSContext* cx, unsigned argc, Value* vp) {
+static bool ListDir(JSContext* cx, unsigned argc, Value* vp,
+                    PathResolutionMode resolveMode) {
   CallArgs args = CallArgsFromVp(argc, vp);
 
   if (args.length() != 1) {
@@ -337,7 +364,7 @@ static bool osfile_listDir(JSContext* cx, unsigned argc, Value* vp) {
   }
 
   RootedString givenPath(cx, args[0].toString());
-  RootedString str(cx, ResolvePath(cx, givenPath, ScriptRelative));
+  RootedString str(cx, ResolvePath(cx, givenPath, resolveMode));
   if (!str) {
     return false;
   }
@@ -415,6 +442,15 @@ static bool osfile_listDir(JSContext* cx, unsigned argc, Value* vp) {
   return true;
 }
 
+static bool osfile_listDir(JSContext* cx, unsigned argc, Value* vp) {
+  return ListDir(cx, argc, vp, RootRelative);
+}
+
+static bool osfile_listDirRelativeToScript(JSContext* cx, unsigned argc,
+                                           Value* vp) {
+  return ListDir(cx, argc, vp, ScriptRelative);
+}
+
 static bool osfile_writeTypedArrayToFile(JSContext* cx, unsigned argc,
                                          Value* vp) {
   CallArgs args = CallArgsFromVp(argc, vp);
@@ -464,8 +500,8 @@ static bool osfile_writeTypedArrayToFile(JSContext* cx, unsigned argc,
     return false;
   }
   void* buf = obj->dataPointerUnshared();
-  if (fwrite(buf, obj->bytesPerElement(), obj->length(), file) !=
-          obj->length() ||
+  size_t length = obj->length();
+  if (fwrite(buf, obj->bytesPerElement(), length, file) != length ||
       !autoClose.release()) {
     filename = JS_EncodeStringToUTF8(cx, str);
     if (!filename) {
@@ -550,7 +586,7 @@ class FileObject : public NativeObject {
 
   RCFile* rcFile() {
     return reinterpret_cast<RCFile*>(
-        js::GetReservedSlot(this, FILE_SLOT).toPrivate());
+        JS::GetReservedSlot(this, FILE_SLOT).toPrivate());
   }
 };
 
@@ -705,15 +741,20 @@ static const JSFunctionSpecWithHelp osfile_functions[] = {
 "  as the second argument, in which case it returns a Uint8Array. Filename is\n"
 "  relative to the current working directory."),
 
-    JS_FN_HELP("listDir", osfile_listDir, 1, 0,
-"listDir(filename)",
-"  Read entire contents of a directory. The \"filename\" parameter is relate to the\n"
-"  current working directory.Returns a list of filenames within the given directory.\n"
-"  Note that \".\" and \"..\" are also listed."),
-
     JS_FN_HELP("readRelativeToScript", osfile_readRelativeToScript, 1, 0,
 "readRelativeToScript(filename, [\"binary\"])",
 "  Read filename into returned string. Filename is relative to the directory\n"
+"  containing the current script."),
+
+    JS_FN_HELP("listDir", osfile_listDir, 1, 0,
+"listDir(dirname)",
+"  Read entire contents of a directory. The \"dirname\" parameter is relate to the\n"
+"  current working directory. Returns a list of filenames within the given directory.\n"
+"  Note that \".\" and \"..\" are also listed."),
+
+    JS_FN_HELP("listDirRelativeToScript", osfile_listDirRelativeToScript, 1, 0,
+"listDirRelativeToScript(dirname)",
+"  Same as \"listDir\" except that the \"dirname\" is relative to the directory\n"
 "  containing the current script."),
 
     JS_FS_HELP_END
@@ -865,7 +906,8 @@ static bool os_getpid(JSContext* cx, unsigned argc, Value* vp) {
   return true;
 }
 
-#if !defined(XP_WIN)
+#ifndef __wasi__
+#  if !defined(XP_WIN)
 
 // There are two possible definitions of strerror_r floating around. The GNU
 // one returns a char* which may or may not be the buffer you passed in. The
@@ -878,18 +920,18 @@ inline char* strerror_message(int result, char* buffer) {
 
 inline char* strerror_message(char* result, char* buffer) { return result; }
 
-#endif
+#  endif
 
 static void ReportSysError(JSContext* cx, const char* prefix) {
   char buffer[200];
 
-#if defined(XP_WIN)
+#  if defined(XP_WIN)
   strerror_s(buffer, sizeof(buffer), errno);
   const char* errstr = buffer;
-#else
+#  else
   const char* errstr =
       strerror_message(strerror_r(errno, buffer, sizeof(buffer)), buffer);
-#endif
+#  endif
 
   if (!errstr) {
     errstr = "unknown error";
@@ -930,7 +972,7 @@ static bool os_system(JSContext* cx, unsigned argc, Value* vp) {
   return true;
 }
 
-#ifndef XP_WIN
+#  ifndef XP_WIN
 static bool os_spawn(JSContext* cx, unsigned argc, Value* vp) {
   CallArgs args = CallArgsFromVp(argc, vp);
 
@@ -1049,6 +1091,7 @@ static bool os_waitpid(JSContext* cx, unsigned argc, Value* vp) {
   args.rval().setObject(*info);
   return true;
 }
+#  endif  // !__wasi__
 #endif
 
 // clang-format off
@@ -1061,12 +1104,13 @@ static const JSFunctionSpecWithHelp os_functions[] = {
 "getpid()",
 "  Return the current process id."),
 
+#ifndef __wasi__
     JS_FN_HELP("system", os_system, 1, 0,
 "system(command)",
 "  Execute command on the current host, returning result code or throwing an\n"
 "  exception on failure."),
 
-#ifndef XP_WIN
+#  ifndef XP_WIN
     JS_FN_HELP("spawn", os_spawn, 1, 0,
 "spawn(command)",
 "  Start up a separate process running the given command. Returns the pid."),
@@ -1081,7 +1125,8 @@ static const JSFunctionSpecWithHelp os_functions[] = {
 "  Calls waitpid(). 'nohang' is a boolean indicating whether to pass WNOHANG.\n"
 "  The return value is an object containing a 'pid' field, if a process was waitable\n"
 "  and an 'exitStatus' field if a pid exited."),
-#endif
+#  endif
+#endif  // !__wasi__
 
     JS_FS_HELP_END
 };
