@@ -4,10 +4,11 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 427196;
-var summary = 'Do not assert: OBJ_SCOPE(pobj)->object == pobj';
-var actual = '';
-var expect = '';
+var BUGNUMBER = 355052;
+var summary = 'Do not crash with valueOf:gc and __iterator__';
+var actual = 'No Crash';
+var expect = 'No Crash';
+
 
 //-----------------------------------------------------------------------------
 test();
@@ -17,12 +18,17 @@ function test()
 {
   printBugNumber(BUGNUMBER);
   printStatus (summary);
+ 
+  expect = /TypeError: .+ is not a function/;
+  actual = 'No Error';
+  try
+  {
+    ( {valueOf: gc} - [function(){}].__iterator__ )();
+  }
+  catch(ex)
+  {
+    actual = ex + '';
+  }
 
-  var c = {__proto__: []};
-  var a = {__proto__: {__proto__: {}}};
-  c.hasOwnProperty;
-  c.__proto__ = a;
-  c.hasOwnProperty;
-
-  reportCompare(expect, actual, summary);
+  reportMatch(expect, actual, summary);
 }
