@@ -3,14 +3,13 @@
 
 /*---
 esid: sec-array.isarray
-es5id: 15.4.3.2-0-7
-description: Array.isArray returns false if its argument is not an Array
+es5id: 15.4.3.2-0-6
+description: Array.isArray return true if its argument is an Array (new Array())
 ---*/
 
-var o = new Object();
-o[12] = 13;
-var b = Array.isArray(o);
+var a = new Array(10);
+var b = Array.isArray(a);
 
-assert.sameValue(b, false, 'b');
+assert.sameValue(b, true, 'b');
 
 reportCompare(0, 0);
