@@ -3,22 +3,19 @@
 
 /*---
 esid: sec-array.prototype.map
-description: >
-    Array.prototype.map - applied to the Array-like object that
-    'length' property doesn't exist
+description: Array.prototype.map - applied to RegExp object
 ---*/
 
 function callbackfn(val, idx, obj) {
-  return val > 10;
+  return obj instanceof RegExp;
 }
 
-var obj = {
-  0: 11,
-  1: 12
-};
+var obj = new RegExp();
+obj.length = 1;
+obj[0] = 1;
 
 var testResult = Array.prototype.map.call(obj, callbackfn);
 
-assert.sameValue(testResult.length, 0, 'testResult.length');
+assert.sameValue(testResult[0], true, 'testResult[0]');
 
 reportCompare(0, 0);
