@@ -4,20 +4,28 @@
 /*---
 esid: sec-array.prototype.reduce
 description: >
-    Array.prototype.reduce - applied to String object, which
-    implements its own property get method
+    Array.prototype.reduce - when element to be retrieved is own
+    accessor property on an Array
 ---*/
 
 var testResult = false;
 
 function callbackfn(prevVal, curVal, idx, obj) {
   if (idx === 1) {
-    testResult = (prevVal === "0");
+    testResult = (prevVal === 0);
   }
 }
 
-var str = new String("012");
-Array.prototype.reduce.call(str, callbackfn);
+var arr = [, 1, 2];
+
+Object.defineProperty(arr, "0", {
+  get: function() {
+    return 0;
+  },
+  configurable: true
+});
+
+arr.reduce(callbackfn);
 
 assert(testResult, 'testResult !== true');
 
