@@ -4,10 +4,13 @@
 /*---
 esid: sec-date.prototype.toisostring
 description: >
-    Date.prototype.toISOString must exist as a function taking 0
-    parameters
+    Date.prototype.toISOString - TypeError is thrown when this is any
+    primitive values
 ---*/
 
-assert.sameValue(Date.prototype.toISOString.length, 0, 'Date.prototype.toISOString.length');
+
+assert.throws(TypeError, function() {
+  Date.prototype.toISOString.call(15);
+});
 
 reportCompare(0, 0);
