@@ -3,22 +3,21 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.3.5.4_2-23gs
+es5id: 15.3.5.4_2-50gs
 description: >
     Strict mode - checking access to strict function caller from
-    strict function (Anonymous FunctionExpression defined within a
-    FunctionDeclaration inside strict mode)
+    strict function (Literal setter defined within strict mode)
 flags: [onlyStrict]
 ---*/
 
-function f1() {
-  (function() {
+var o = {
+  set foo(stuff) {
     gNonStrict();
-  })();
+  }
 }
 
 assert.throws(TypeError, function() {
-  f1();
+  o.foo = 7;
 });
 
 function gNonStrict() {
