@@ -2,14 +2,14 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.2-2-24
+es5id: 15.2.3.2-2-19
 description: >
     Object.getPrototypeOf returns the [[Prototype]] of its parameter
-    (Number object)
+    (Object object)
 ---*/
 
-var obj = new Number(-3);
+var obj = {};
 
-assert.sameValue(Object.getPrototypeOf(obj), Number.prototype, 'Object.getPrototypeOf(obj)');
+assert.sameValue(Object.getPrototypeOf(obj), Object.prototype, 'Object.getPrototypeOf(obj)');
 
 reportCompare(0, 0);
