@@ -2,12 +2,12 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.13-0-1
-description: Object.isExtensible must exist as a function
+es5id: 15.2.3.13-2-12
+description: Object.isExtensible returns true for all built-in objects (JSON)
 ---*/
 
-var f = Object.isExtensible;
+var e = Object.isExtensible(JSON);
 
-assert.sameValue(typeof(f), "function", 'typeof(f)');
+assert.sameValue(e, true, 'e');
 
 reportCompare(0, 0);
