@@ -6,16 +6,16 @@
 info: |
     PerformPromiseThen
     Ref 25.4.5.3.1
-es6id: S25.4.5.3_A4.2_T1
+es6id: S25.4.5.3_A4.1_T1
 author: Sam Mikes
-description: Promise.prototype.then treats non-callable arg1, arg2 as undefined
+description: Promise.prototype.then accepts 'undefined' as arg1, arg2
 flags: [async]
 ---*/
 
 var obj = {};
 var p = Promise.resolve(obj);
 
-p.then(3, 5)
+p.then(undefined, undefined)
   .then(function(arg) {
     if (arg !== obj) {
       $ERROR("Expected resolution object to be passed through, got " + arg);
