@@ -3,9 +3,9 @@
 // See LICENSE for details.
 
 /*---
-es6id: S25.4.4.3_A6.2_T1
+es6id: S25.4.4.3_A7.2_T1
 author: Sam Mikes
-description: Promise.race([p1]) settles immediately
+description: Promise.race([p1, p2]) settles when first settles
 includes: [promiseHelper.js]
 flags: [async]
 ---*/
@@ -13,16 +13,21 @@ flags: [async]
 var sequence = [];
 
 var p1 = Promise.reject(1),
-  p = Promise.race([p1]);
+  p2 = Promise.resolve(2),
+  p = Promise.race([p1, p2]);
 
 sequence.push(1);
 
 p.then(function() {
-  $ERROR("Should not fulfill.");
-}, function() {
+  $ERROR("Should not be fulfilled - expected rejection.");
+}, function(arg) {
+  if (arg !== 1) {
+    $ERROR("Expected rejection reason to be 1, got " + arg);
+  }
+
   sequence.push(4);
   assert.sameValue(sequence.length, 4);
-checkSequence(sequence, "This happens second");
+  checkSequence(sequence, "This happens second");
 }).catch($DONE);
 
 Promise.resolve().then(function() {
