@@ -2,12 +2,12 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.5.4.20-4-10
+es5id: 15.5.4.20-3-14
 description: >
-    String.prototype.trim handles whitepace and lineterminators
-    (\uFEFFabc)
+    String.prototype.trim - 'S' is a string that has null character in
+    the middle
 ---*/
 
-assert.sameValue("\uFEFFabc".trim(), "abc", '"\uFEFFabc".trim()');
+assert.sameValue("a\0\u0000bc".trim(), "a\0\u0000bc", '"a\0\u0000bc".trim()');
 
 reportCompare(0, 0);
