@@ -1,9 +1,7 @@
-// |reftest| skip-if(!this.hasOwnProperty('BigInt')) -- BigInt is not enabled unconditionally
 // Copyright (C) 2016 the V8 project authors. All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es6id: 22.2.3.16
 esid: sec-%typedarray%.prototype.keys
 description: >
   The prototype of the returned iterator is ArrayIteratorPrototype
