@@ -29,17 +29,15 @@ testWithTypedArrayConstructors(function(TA) {
   var loops = 0;
   var sample = new TA(2);
 
-  assert.throws(TypeError, function() {
-    sample.some(function() {
-      if (loops === 1) {
-        throw new Test262Error("callbackfn called twice");
-      }
+  sample.some(function() {
+    if (loops === 0) {
       $DETACHBUFFER(sample.buffer);
-      loops++;
-    });
+    }
+    loops++;
+    return false;
   });
 
-  assert.sameValue(loops, 1);
+  assert.sameValue(loops, 2);
 });
 
 reportCompare(0, 0);
