@@ -1,16 +1,14 @@
-// |reftest| skip -- Intl.DateTimeFormat-datetimestyle is not supported
 // Copyright 2018 Igalia, S.L. All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
 esid: sec-intl.datetimeformat.prototype.resolvedoptions
 description: Verifies the property order for the object returned by resolvedOptions().
-includes: [compareArray.js]
+includes: [arrayContains.js]
 features: [Intl.DateTimeFormat-datetimestyle]
 ---*/
 
 const options = new Intl.DateTimeFormat([], {
-  "timeStyle": "full",
   "hourCycle": "h24",
   "weekday": "short",
   "era": "short",
@@ -30,9 +28,16 @@ const expected = [
   "timeZone",
   "hourCycle",
   "hour12",
-  "timeStyle",
 ];
 
-assert.compareArray(Object.getOwnPropertyNames(options), expected);
+let actual = Object.getOwnPropertyNames(options);
+
+// Ensure all expected items are in actual and also allow other properties
+// implemented in new proposals.
+assert(arrayContains(actual, expected));
+for (var i = 1; i < expected.length; i++) {
+  // Ensure the order as expected but allow additional new property in between
+  assert(actual.indexOf(expected[i-1]) < actual.indexOf(expected[i]));
+}
 
 reportCompare(0, 0);
