@@ -5,30 +5,30 @@
 info: |
     If Type(Primitive(x)) is not String or Type(Primitive(y)) is not String,
     then operator x > y returns ToNumber(x) > ToNumber(y)
-es5id: 11.8.2_A3.1_T2.4
+es5id: 11.8.2_A3.1_T2.8
 description: >
     Type(Primitive(x)) is different from Type(Primitive(y)) and both
-    types vary between Number (primitive or object) and Undefined
+    types vary between Boolean (primitive or object) and Undefined
 ---*/
 
 //CHECK#1
-if (1 > undefined !== false) {
-  $ERROR('#1: 1 > undefined === false');
+if (true > undefined !== false) {
+  $ERROR('#1: true > undefined === false');
 }
 
 //CHECK#2
-if (undefined > 1 !== false) {
-  $ERROR('#2: undefined > 1 === false');
+if (undefined > true !== false) {
+  $ERROR('#2: undefined > true === false');
 }
 
 //CHECK#3
-if (new Number(1) > undefined !== false) {
-  $ERROR('#3: new Number(1) > undefined === false');
+if (new Boolean(true) > undefined !== false) {
+  $ERROR('#3: new Boolean(true) > undefined === false');
 }
 
 //CHECK#4
-if (undefined > new Number(1) !== false) {
-  $ERROR('#4: undefined > new Number(1) === false');
+if (undefined > new Boolean(true) !== false) {
+  $ERROR('#4: undefined > new Boolean(true) === false');
 }
 
 reportCompare(0, 0);
