@@ -2,15 +2,14 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 10.4.3-1-74gs
+es5id: 10.4.3-1-77gs
 description: >
     checking 'this' from a global scope (strict function declaration called by
-    Function.prototype.call(someObject))
+    Function.prototype.bind(null)())
 ---*/
 
-var o = {};
-function f() { "use strict"; return this===o;};
-if (! f.call(o)){
+function f() { "use strict"; return this===null;};
+if (! (f.bind(null)())){
     throw "'this' had incorrect value!";
 }
 
