@@ -2,8 +2,12 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-esid: prod-RegularExpressionBackslashSequence
+esid: sec-literals-regular-expression-literals
 info: |
+  RegularExpressionChar ::
+    RegularExpressionBackslashSequence
+
+
   RegularExpressionBackslashSequence ::
     \ RegularExpressionNonTerminator
 
@@ -22,12 +26,12 @@ description: >
 
 //CHECK#1
 try {
-   eval("/a\\\u000A/").source;
-   $ERROR('#1.1: RegularExpressionChar :: BackslashSequence :: \\Line Feed is incorrect. Actual: ' + (eval("/a\\\u000A/").source));
+   eval("/a\u000A/").source;
+   $ERROR('#1.1: RegularExpressionChar :: Line Feedis incorrect. Actual: ' + (eval("/a\u000A/").source));
 }
 catch (e) {
   if ((e instanceof SyntaxError) !== true) {
-     $ERROR('#1.2: RegularExpressionChar :: BackslashSequence :: \\Line Feed is incorrect. Actual: ' + (e));
+     $ERROR('#1.2: RegularExpressionChar :: Line Feed is incorrect. Actual: ' + (e));
   }
 }
 
