@@ -5,10 +5,10 @@
 info: |
     The with statement adds a computed object to the front of the
     scope chain of the current execution context
-es5id: 12.10_A1.12_T1
+es5id: 12.10_A1.6_T1
 description: >
-    Calling a function without "with" statement declared within the
-    statement, leading to normal completion
+    Using "with" statement within another "with" statement, leading to
+    normal completion
 flags: [noStrict]
 ---*/
 
@@ -42,7 +42,7 @@ var st_isNaN = "isNaN";
 var st_isFinite = "isFinite";
 
 with(myObj){
-  var f = function(){
+  with(myObj){
     st_p1 = p1;
     st_p2 = p2;
     st_p3 = p3;
@@ -61,7 +61,6 @@ with(myObj){
     var value = 'value';
   }
 }
-f();
 
 if(!(p1 === 1)){
   $ERROR('#1: p1 === 1. Actual:  p1 ==='+ p1  );
@@ -75,11 +74,8 @@ if(!(p3 === 3)){
   $ERROR('#3: p3 === 3. Actual:  p3 ==='+ p3  );
 }
 
-try{
-  p4;
-  $ERROR('#4: p4 doesn\'t exists');
-}
-catch(e){
+if(!(p4 === "x4")){
+  $ERROR('#4: p4 === "x4". Actual:  p4 ==='+ p4  );
 }
 
 if(!(p5 === "x5")){
@@ -134,14 +130,12 @@ if(!(st_isFinite !== isFinite)){
   $ERROR('#17: myObj.isFinite !== isFinite');
 }
 
-try {
-  value;
-  $ERROR('#18: value is not defined');
-} catch(e) {    
+if(!(value === undefined)){
+  $ERROR('#18: value === undefined. Actual:  value ==='+ value  );
 }
 
-if(!(myObj.value === "myObj_value")){
-  $ERROR('#19: myObj.value === "myObj_value". Actual:  myObj.value ==='+ myObj.value  );
+if(!(myObj.value === "value")){
+  $ERROR('#19: myObj.value === "value". Actual:  myObj.value ==='+ myObj.value  );
 }
 
 reportCompare(0, 0);
