@@ -5,10 +5,10 @@
 info: |
     The with statement adds a computed object to the front of the
     scope chain of the current execution context
-es5id: 12.10_A1.5_T5
+es5id: 12.10_A1.7_T5
 description: >
-    Using "with" statement within "for-in" statement, leading to
-    completion by break
+    Calling a function within "with" statement declared within the
+    statement, leading to completion by exception
 flags: [noStrict]
 ---*/
 
@@ -41,32 +41,35 @@ var st_parseFloat = "parseFloat";
 var st_isNaN = "isNaN";
 var st_isFinite = "isFinite";
 
-for(var prop in myObj){
+try {
   with(myObj){
-    break;
-    if(prop === 'p1') {
+    var f = function(){
+      throw value;
       st_p1 = p1;
-      p1 = 'x1';
-    }
-    if(prop === 'p2') {
       st_p2 = p2;
-      this.p2 = 'x2';
-    }
-    if(prop === 'p3') {
       st_p3 = p3;
+      st_parseInt = parseInt;
+      st_NaN = NaN;
+      st_Infinity = Infinity;
+      st_eval = eval;
+      st_parseFloat = parseFloat;
+      st_isNaN = isNaN;
+      st_isFinite = isFinite;
+      p1 = 'x1';
+      this.p2 = 'x2';
       del = delete p3;
+      var p4 = 'x4';
+      p5 = 'x5';
+      var value = 'value';
     }
-    if(prop === 'parseInt') st_parseInt = parseInt;
-    if(prop === 'NaN') st_NaN = NaN;
-    if(prop === 'Infinity') st_Infinity = Infinity;
-    if(prop === 'eval') st_eval = eval;
-    if(prop === 'parseFloat') st_parseFloat = parseFloat;
-    if(prop === 'isNaN') st_isNaN = isNaN;
-    if(prop === 'isFinite') st_isFinite = isFinite;
-    var p4 = 'x4';
-    p5 = 'x5';
-    var value = 'value';
+    f();
   }
+} catch(e){
+  result = e;
+}
+
+if(!(result === undefined)){
+  $ERROR('#0: result === undefined. Actual:  result ==='+ result  );
 }
 
 if(!(p1 === 1)){
@@ -81,8 +84,10 @@ if(!(p3 === 3)){
   $ERROR('#3: p3 === 3. Actual:  p3 ==='+ p3  );
 }
 
-if(!(p4 === undefined)){
-  $ERROR('#4: p4 === undefined. Actual:  p4 ==='+ p4  );
+try {
+  p4;
+  $ERROR('#4: p4 is not defined');
+} catch(e) {    
 }
 
 try {
@@ -139,8 +144,11 @@ if(!(st_isFinite === "isFinite")){
   $ERROR('#17: st_isFinite === "isFinite". Actual:  st_isFinite ==='+ st_isFinite  );
 }
 
-if(!(value === undefined)){
-  $ERROR('#18: value === undefined. Actual:  value ==='+ value  );
+try{
+  value;
+  $ERROR('#18: value is not defined');
+}
+catch(e){
 }
 
 if(!(myObj.value === "myObj_value")){
