@@ -12,9 +12,13 @@
 #include "jsnum.h"
 
 #include "jit/Ion.h"
+#include "js/friend/DumpFunctions.h"
+#include "js/friend/ErrorMessages.h"  // js::GetErrorMessage, JSMSG_*
 #include "vm/ArgumentsObject.h"
 #include "vm/BytecodeUtil.h"  // JSDVG_SEARCH_STACK
 #include "vm/Realm.h"
+#include "vm/SharedStencil.h"  // GCThingIndex
+#include "vm/ThrowMsgKind.h"
 
 #include "vm/EnvironmentObject-inl.h"
 #include "vm/GlobalObject-inl.h"
@@ -26,41 +30,6 @@
 
 namespace js {
 
-/*
- * Every possible consumer of MagicValue(JS_OPTIMIZED_ARGUMENTS) (as determined
- * by ScriptAnalysis::needsArgsObj) must check for these magic values and, when
- * one is received, act as if the value were the function's ArgumentsObject.
- * Additionally, it is possible that, after 'arguments' was copied into a
- * temporary, the arguments object has been created a some other failed guard
- * that called JSScript::argumentsOptimizationFailed. In this case, it is
- * always valid (and necessary) to replace JS_OPTIMIZED_ARGUMENTS with the real
- * arguments object.
- */
-static inline bool IsOptimizedArguments(AbstractFramePtr frame,
-                                        MutableHandleValue vp) {
-  if (vp.isMagic(JS_OPTIMIZED_ARGUMENTS) && frame.script()->needsArgsObj()) {
-    vp.setObject(frame.argsObj());
-  }
-  return vp.isMagic(JS_OPTIMIZED_ARGUMENTS);
-}
-
-/*
- * One optimized consumer of MagicValue(JS_OPTIMIZED_ARGUMENTS) is f.apply.
- * However, this speculation must be guarded before calling 'apply' in case it
- * is not the builtin Function.prototype.apply.
- */
-static inline void GuardFunApplyArgumentsOptimization(JSContext* cx,
-                                                      AbstractFramePtr frame,
-                                                      CallArgs& args) {
-  if (args.length() == 2 && IsOptimizedArguments(frame, args[1])) {
-    if (!IsNativeFunction(args.calleev(), js::fun_apply)) {
-      RootedScript script(cx, frame.script());
-      JSScript::argumentsOptimizationFailed(cx, script);
-      args[1].setObject(frame.argsObj());
-    }
-  }
-}
-
 /*
  * Per ES6, lexical declarations may not be accessed in any fashion until they
  * are initialized (i.e., until the actual declaring statement is
@@ -69,13 +38,13 @@ static inline void GuardFunApplyArgumentsOptimization(JSContext* cx,
  * JS_UNINITIALIZED_LEXICAL.
  */
 static inline bool IsUninitializedLexical(const Value& val) {
-  // Use whyMagic here because JS_OPTIMIZED_ARGUMENTS could flow into here.
+  // Use whyMagic here because JS_OPTIMIZED_OUT could flow into here.
   return val.isMagic() && val.whyMagic() == JS_UNINITIALIZED_LEXICAL;
 }
 
 static inline bool IsUninitializedLexicalSlot(HandleObject obj,
-                                              Handle<PropertyResult> prop) {
-  MOZ_ASSERT(prop);
+                                              const PropertyResult& prop) {
+  MOZ_ASSERT(prop.isFound());
   if (obj->is<WithEnvironmentObject>()) {
     return false;
   }
@@ -85,13 +54,13 @@ static inline bool IsUninitializedLexicalSlot(HandleObject obj,
     return false;
   }
 
-  Shape* shape = prop.shape();
-  if (!shape->isDataProperty()) {
+  PropertyInfo propInfo = prop.propertyInfo();
+  if (!propInfo.isDataProperty()) {
     return false;
   }
 
-  MOZ_ASSERT(obj->as<NativeObject>().containsPure(shape));
-  return IsUninitializedLexical(obj->as<NativeObject>().getSlot(shape->slot()));
+  return IsUninitializedLexical(
+      obj->as<NativeObject>().getSlot(propInfo.slot()));
 }
 
 static inline bool CheckUninitializedLexical(JSContext* cx, PropertyName* name_,
@@ -135,9 +104,9 @@ enum class GetNameMode { Normal, TypeOf };
 
 template <GetNameMode mode>
 inline bool FetchName(JSContext* cx, HandleObject receiver, HandleObject holder,
-                      HandlePropertyName name, Handle<PropertyResult> prop,
+                      HandlePropertyName name, const PropertyResult& prop,
                       MutableHandleValue vp) {
-  if (!prop) {
+  if (prop.isNotFound()) {
     switch (mode) {
       case GetNameMode::Normal:
         ReportIsNotDefined(cx, name);
@@ -149,23 +118,23 @@ inline bool FetchName(JSContext* cx, HandleObject receiver, HandleObject holder,
   }
 
   /* Take the slow path if shape was not found in a native object. */
-  if (!receiver->isNative() || !holder->isNative()) {
+  if (!receiver->is<NativeObject>() || !holder->is<NativeObject>()) {
     Rooted<jsid> id(cx, NameToId(name));
     if (!GetProperty(cx, receiver, receiver, id, vp)) {
       return false;
     }
   } else {
-    RootedShape shape(cx, prop.shape());
-    if (shape->isDataDescriptor() && shape->hasDefaultGetter()) {
+    PropertyInfo propInfo = prop.propertyInfo();
+    if (propInfo.isDataProperty()) {
       /* Fast path for Object instance properties. */
-      MOZ_ASSERT(shape->isDataProperty());
-      vp.set(holder->as<NativeObject>().getSlot(shape->slot()));
+      vp.set(holder->as<NativeObject>().getSlot(propInfo.slot()));
     } else {
       // Unwrap 'with' environments for reasons given in
       // GetNameBoundInEnvironment.
       RootedObject normalized(cx, MaybeUnwrapWithEnvironment(receiver));
+      RootedId id(cx, NameToId(name));
       if (!NativeGetExistingProperty(cx, normalized, holder.as<NativeObject>(),
-                                     shape, vp)) {
+                                     id, propInfo, vp)) {
         return false;
       }
     }
@@ -181,17 +150,17 @@ inline bool FetchName(JSContext* cx, HandleObject receiver, HandleObject holder,
   return CheckUninitializedLexical(cx, name, vp);
 }
 
-inline bool FetchNameNoGC(JSObject* pobj, PropertyResult prop, Value* vp) {
-  if (!prop || !pobj->isNative()) {
+inline bool FetchNameNoGC(NativeObject* pobj, PropertyResult prop, Value* vp) {
+  if (prop.isNotFound()) {
     return false;
   }
 
-  Shape* shape = prop.shape();
-  if (!shape->isDataDescriptor() || !shape->hasDefaultGetter()) {
+  PropertyInfo propInfo = prop.propertyInfo();
+  if (!propInfo.isDataProperty()) {
     return false;
   }
 
-  *vp = pobj->as<NativeObject>().getSlot(shape->slot());
+  *vp = pobj->getSlot(propInfo.slot());
   return !IsUninitializedLexical(*vp);
 }
 
@@ -201,7 +170,7 @@ inline bool GetEnvironmentName(JSContext* cx, HandleObject envChain,
   {
     PropertyResult prop;
     JSObject* obj = nullptr;
-    JSObject* pobj = nullptr;
+    NativeObject* pobj = nullptr;
     if (LookupNameNoGC(cx, name, envChain, &obj, &pobj, &prop)) {
       if (FetchNameNoGC(pobj, prop, vp.address())) {
         return true;
@@ -209,7 +178,7 @@ inline bool GetEnvironmentName(JSContext* cx, HandleObject envChain,
     }
   }
 
-  Rooted<PropertyResult> prop(cx);
+  PropertyResult prop;
   RootedObject obj(cx), pobj(cx);
   if (!LookupName(cx, name, envChain, &obj, &pobj, &prop)) {
     return false;
@@ -223,11 +192,13 @@ inline bool HasOwnProperty(JSContext* cx, HandleValue val, HandleValue idValue,
   // As an optimization, provide a fast path when rooting is not necessary and
   // we can safely retrieve the object's shape.
   jsid id;
-  if (val.isObject() && ValueToId<NoGC>(cx, idValue, &id)) {
+  if (val.isObject() && idValue.isPrimitive() &&
+      PrimitiveValueToId<NoGC>(cx, idValue, &id)) {
     JSObject* obj = &val.toObject();
     PropertyResult prop;
-    if (obj->isNative() && NativeLookupOwnProperty<NoGC>(
-                               cx, &obj->as<NativeObject>(), id, &prop)) {
+    if (obj->is<NativeObject>() &&
+        NativeLookupOwnProperty<NoGC>(cx, &obj->as<NativeObject>(), id,
+                                      &prop)) {
       *result = prop.isFound();
       return true;
     }
@@ -306,20 +277,19 @@ inline bool SetNameOperation(JSContext* cx, JSScript* script, jsbytecode* pc,
   return ok && result.checkStrictModeError(cx, env, id, strict);
 }
 
-inline void InitGlobalLexicalOperation(JSContext* cx,
-                                       LexicalEnvironmentObject* lexicalEnvArg,
-                                       JSScript* script, jsbytecode* pc,
-                                       HandleValue value) {
+inline void InitGlobalLexicalOperation(
+    JSContext* cx, ExtensibleLexicalEnvironmentObject* lexicalEnv,
+    JSScript* script, jsbytecode* pc, HandleValue value) {
   MOZ_ASSERT_IF(!script->hasNonSyntacticScope(),
-                lexicalEnvArg == &cx->global()->lexicalEnvironment());
+                lexicalEnv == &cx->global()->lexicalEnvironment());
   MOZ_ASSERT(JSOp(*pc) == JSOp::InitGLexical);
-  Rooted<LexicalEnvironmentObject*> lexicalEnv(cx, lexicalEnvArg);
-  RootedShape shape(cx, lexicalEnv->lookup(cx, script->getName(pc)));
-  MOZ_ASSERT(shape);
-  MOZ_ASSERT(IsUninitializedLexical(lexicalEnv->getSlot(shape->slot())));
 
-  // Don't treat the initial assignment to global lexicals as overwrites.
-  lexicalEnv->setSlotWithType(cx, shape, value, /* overwriting = */ false);
+  mozilla::Maybe<PropertyInfo> prop =
+      lexicalEnv->lookup(cx, script->getName(pc));
+  MOZ_ASSERT(prop.isSome());
+  MOZ_ASSERT(IsUninitializedLexical(lexicalEnv->getSlot(prop->slot())));
+
+  lexicalEnv->setSlot(prop->slot(), value);
 }
 
 inline bool InitPropertyOperation(JSContext* cx, JSOp op, HandleObject obj,
@@ -388,8 +358,9 @@ static MOZ_ALWAYS_INLINE bool DecOperation(JSContext* cx, HandleValue val,
   return BigInt::decValue(cx, val, res);
 }
 
-static MOZ_ALWAYS_INLINE bool ToIdOperation(JSContext* cx, HandleValue idval,
-                                            MutableHandleValue res) {
+static MOZ_ALWAYS_INLINE bool ToPropertyKeyOperation(JSContext* cx,
+                                                     HandleValue idval,
+                                                     MutableHandleValue res) {
   if (idval.isInt32()) {
     res.set(idval);
     return true;
@@ -407,10 +378,8 @@ static MOZ_ALWAYS_INLINE bool ToIdOperation(JSContext* cx, HandleValue idval,
 static MOZ_ALWAYS_INLINE bool GetObjectElementOperation(
     JSContext* cx, JSOp op, JS::HandleObject obj, JS::HandleValue receiver,
     HandleValue key, MutableHandleValue res) {
-  MOZ_ASSERT(op == JSOp::GetElem || op == JSOp::CallElem ||
-             op == JSOp::GetElemSuper);
-  MOZ_ASSERT_IF(op == JSOp::GetElem || op == JSOp::CallElem,
-                obj == &receiver.toObject());
+  MOZ_ASSERT(op == JSOp::GetElem || op == JSOp::GetElemSuper);
+  MOZ_ASSERT_IF(op == JSOp::GetElem, obj == &receiver.toObject());
 
   do {
     uint32_t index;
@@ -457,10 +426,8 @@ static MOZ_ALWAYS_INLINE bool GetObjectElementOperation(
 }
 
 static MOZ_ALWAYS_INLINE bool GetPrimitiveElementOperation(
-    JSContext* cx, JSOp op, JS::HandleValue receiver, int receiverIndex,
-    HandleValue key, MutableHandleValue res) {
-  MOZ_ASSERT(op == JSOp::GetElem || op == JSOp::CallElem);
-
+    JSContext* cx, JS::HandleValue receiver, int receiverIndex, HandleValue key,
+    MutableHandleValue res) {
   // FIXME: Bug 1234324 We shouldn't be boxing here.
   RootedObject boxed(
       cx, ToObjectFromStackForPropertyAccess(cx, receiver, receiverIndex, key));
@@ -512,35 +479,9 @@ static MOZ_ALWAYS_INLINE bool GetPrimitiveElementOperation(
   return true;
 }
 
-static MOZ_ALWAYS_INLINE bool GetElemOptimizedArguments(
-    JSContext* cx, AbstractFramePtr frame, MutableHandleValue lref,
-    HandleValue rref, MutableHandleValue res, bool* done) {
-  MOZ_ASSERT(!*done);
-
-  if (IsOptimizedArguments(frame, lref)) {
-    if (rref.isInt32()) {
-      int32_t i = rref.toInt32();
-      if (i >= 0 && uint32_t(i) < frame.numActualArgs()) {
-        res.set(frame.unaliasedActual(i));
-        *done = true;
-        return true;
-      }
-    }
-
-    RootedScript script(cx, frame.script());
-    JSScript::argumentsOptimizationFailed(cx, script);
-
-    lref.set(ObjectValue(frame.argsObj()));
-  }
-
-  return true;
-}
-
 static MOZ_ALWAYS_INLINE bool GetElementOperationWithStackIndex(
-    JSContext* cx, JSOp op, HandleValue lref, int lrefIndex, HandleValue rref,
+    JSContext* cx, HandleValue lref, int lrefIndex, HandleValue rref,
     MutableHandleValue res) {
-  MOZ_ASSERT(op == JSOp::GetElem || op == JSOp::CallElem);
-
   uint32_t index;
   if (lref.isString() && IsDefinitelyIndex(rref, &index)) {
     JSString* str = lref.toString();
@@ -556,21 +497,21 @@ static MOZ_ALWAYS_INLINE bool GetElementOperationWithStackIndex(
 
   if (lref.isPrimitive()) {
     RootedValue thisv(cx, lref);
-    return GetPrimitiveElementOperation(cx, op, thisv, lrefIndex, rref, res);
+    return GetPrimitiveElementOperation(cx, thisv, lrefIndex, rref, res);
   }
 
   RootedObject obj(cx, &lref.toObject());
   RootedValue thisv(cx, lref);
-  return GetObjectElementOperation(cx, op, obj, thisv, rref, res);
+  return GetObjectElementOperation(cx, JSOp::GetElem, obj, thisv, rref, res);
 }
 
 // Wrapper for callVM from JIT.
-static MOZ_ALWAYS_INLINE bool GetElementOperation(JSContext* cx, JSOp op,
+static MOZ_ALWAYS_INLINE bool GetElementOperation(JSContext* cx,
                                                   HandleValue lref,
                                                   HandleValue rref,
                                                   MutableHandleValue res) {
-  return GetElementOperationWithStackIndex(cx, op, lref, JSDVG_SEARCH_STACK,
-                                           rref, res);
+  return GetElementOperationWithStackIndex(cx, lref, JSDVG_SEARCH_STACK, rref,
+                                           res);
 }
 
 static MOZ_ALWAYS_INLINE JSString* TypeOfOperation(const Value& v,
@@ -591,55 +532,74 @@ static MOZ_ALWAYS_INLINE bool InitElemOperation(JSContext* cx, jsbytecode* pc,
   }
 
   unsigned flags = GetInitDataPropAttrs(JSOp(*pc));
+  if (id.isPrivateName()) {
+    // Clear enumerate flag off of private names.
+    flags &= ~JSPROP_ENUMERATE;
+  }
   return DefineDataProperty(cx, obj, id, val, flags);
 }
 
-static MOZ_ALWAYS_INLINE bool InitArrayElemOperation(JSContext* cx,
-                                                     jsbytecode* pc,
-                                                     HandleObject obj,
-                                                     uint32_t index,
-                                                     HandleValue val) {
-  JSOp op = JSOp(*pc);
-  MOZ_ASSERT(op == JSOp::InitElemArray || op == JSOp::InitElemInc);
+static MOZ_ALWAYS_INLINE bool CheckPrivateFieldOperation(JSContext* cx,
+                                                         jsbytecode* pc,
+                                                         HandleValue val,
+                                                         HandleValue idval,
+                                                         bool* result) {
+  // Result had better not be a nullptr.
+  MOZ_ASSERT(result);
 
-  MOZ_ASSERT(obj->is<ArrayObject>());
+  ThrowCondition condition;
+  ThrowMsgKind msgKind;
+  GetCheckPrivateFieldOperands(pc, &condition, &msgKind);
 
-  // The JITs depend on InitElemArray's index not exceeding the dense element
-  // capacity.
-  MOZ_ASSERT_IF(op == JSOp::InitElemArray,
-                index < obj->as<ArrayObject>().getDenseCapacity());
+  // When we are using OnlyCheckRhs, we are implementing PrivateInExpr
+  // This requires we throw if the rhs is not an object;
+  //
+  // The InlineCache for CheckPrivateField already checks for a
+  // non-object rhs and refuses to attach in that circumstance.
+  if (condition == ThrowCondition::OnlyCheckRhs) {
+    if (!val.isObject()) {
+      ReportInNotObjectError(cx, idval, val);
+      return false;
+    }
+  }
 
-  if (op == JSOp::InitElemInc && index == INT32_MAX) {
+  // js::DumpValue(idval.get());
+  // js::DumpValue(val.get());
+
+  MOZ_ASSERT(idval.isSymbol());
+  MOZ_ASSERT(idval.toSymbol()->isPrivateName());
+
+  if (!HasOwnProperty(cx, val, idval, result)) {
+    return false;
+  }
+
+  if (!CheckPrivateFieldWillThrow(condition, *result)) {
+    return true;
+  }
+
+  // Throw!
+  JS_ReportErrorNumberASCII(cx, GetErrorMessage, nullptr,
+                            ThrowMsgKindToErrNum(msgKind));
+  return false;
+}
+
+inline bool InitElemIncOperation(JSContext* cx, HandleArrayObject arr,
+                                 uint32_t index, HandleValue val) {
+  if (index == INT32_MAX) {
     JS_ReportErrorNumberASCII(cx, GetErrorMessage, nullptr,
                               JSMSG_SPREAD_TOO_LARGE);
     return false;
   }
 
-  /*
-   * If val is a hole, do not call DefineElement.
-   *
-   * Furthermore, if the current op is JSOp::InitElemInc, always call
-   * SetLengthProperty even if it is not the last element initialiser, because
-   * it may be followed by a SpreadElement loop, which will not set the array
-   * length if nothing is spread.
-   *
-   * Alternatively, if the current op is JSOp::InitElemArray, the length will
-   * have already been set by the earlier JSOp::NewArray; JSOp::InitElemArray
-   * cannot follow SpreadElements.
-   */
+  // If val is a hole, do not call DefineDataElement.
   if (val.isMagic(JS_ELEMENTS_HOLE)) {
-    if (op == JSOp::InitElemInc) {
-      if (!SetLengthProperty(cx, obj, index + 1)) {
-        return false;
-      }
-    }
-  } else {
-    if (!DefineDataElement(cx, obj, index, val, JSPROP_ENUMERATE)) {
-      return false;
-    }
+    // Always call SetLengthProperty even if this is not the last element
+    // initialiser, because this may be followed by a SpreadElement loop,
+    // which will not set the array length if nothing is spread.
+    return SetLengthProperty(cx, arr, index + 1);
   }
 
-  return true;
+  return DefineDataElement(cx, arr, index, val, JSPROP_ENUMERATE);
 }
 
 static inline ArrayObject* ProcessCallSiteObjOperation(JSContext* cx,
@@ -650,7 +610,7 @@ static inline ArrayObject* ProcessCallSiteObjOperation(JSContext* cx,
   RootedArrayObject cso(cx, &script->getObject(pc)->as<ArrayObject>());
 
   if (cso->isExtensible()) {
-    RootedObject raw(cx, script->getObject(GET_UINT32_INDEX(pc) + 1));
+    RootedObject raw(cx, script->getObject(GET_GCTHING_INDEX(pc).next()));
     MOZ_ASSERT(raw->is<ArrayObject>());
 
     RootedValue rawValue(cx, ObjectValue(*raw));
