@@ -85,12 +85,13 @@ class MOZ_TRIVIAL_CTOR_DTOR MMPolicyInProcessPrimitive {
   /**
    * @return true if the page that hosts aVAddress is accessible.
    */
-  bool IsPageAccessible(void* aVAddress) const {
+  bool IsPageAccessible(uintptr_t aVAddress) const {
     MEMORY_BASIC_INFORMATION mbi;
-    SIZE_T result = nt::VirtualQuery(aVAddress, &mbi, sizeof(mbi));
+    SIZE_T result = nt::VirtualQuery(reinterpret_cast<LPCVOID>(aVAddress), &mbi,
+                                     sizeof(mbi));
 
-    return result && mbi.AllocationProtect && (mbi.Type & MEM_IMAGE) &&
-           mbi.State == MEM_COMMIT && mbi.Protect != PAGE_NOACCESS;
+    return result && mbi.AllocationProtect && mbi.State == MEM_COMMIT &&
+           mbi.Protect != PAGE_NOACCESS;
   }
 };
 
@@ -110,7 +111,33 @@ class MOZ_TRIVIAL_CTOR_DTOR MMPolicyBase {
     return aUnaligned + ((-aUnaligned) & (aAlignTo - 1));
   }
 
+  static PVOID AlignUpToRegion(PVOID aUnaligned, uintptr_t aAlignTo,
+                               size_t aLen, size_t aDesiredLen) {
+    uintptr_t unaligned = reinterpret_cast<uintptr_t>(aUnaligned);
+    uintptr_t aligned = AlignUp(unaligned, aAlignTo);
+    MOZ_ASSERT(aligned >= unaligned);
+
+    if (aLen < aligned - unaligned) {
+      return nullptr;
+    }
+
+    aLen -= (aligned - unaligned);
+    return reinterpret_cast<PVOID>((aLen >= aDesiredLen) ? aligned : 0);
+  }
+
  public:
+#if defined(NIGHTLY_BUILD)
+  Maybe<DetourError> mLastError;
+  const Maybe<DetourError>& GetLastDetourError() const { return mLastError; }
+  template <typename... Args>
+  void SetLastDetourError(Args&&... aArgs) {
+    mLastError = Some(DetourError(std::forward<Args>(aArgs)...));
+  }
+#else
+  template <typename... Args>
+  void SetLastDetourError(Args&&... aArgs) {}
+#endif  // defined(NIGHTLY_BUILD)
+
   DWORD ComputeAllocationSize(const uint32_t aRequestedSize) const {
     MOZ_ASSERT(aRequestedSize);
     DWORD result = aRequestedSize;
@@ -247,8 +274,8 @@ class MOZ_TRIVIAL_CTOR_DTOR MMPolicyBase {
     }
 
     // Return the result as a Span
-    return Some(MakeSpan(reinterpret_cast<const uint8_t*>(lowerBound.value()),
-                         upperBound.value() - lowerBound.value()));
+    return Some(Span(reinterpret_cast<const uint8_t*>(lowerBound.value()),
+                     upperBound.value() - lowerBound.value()));
   }
 
   /**
@@ -258,15 +285,21 @@ class MOZ_TRIVIAL_CTOR_DTOR MMPolicyBase {
    * large.
    */
   PVOID FindRegion(HANDLE aProcess, const size_t aDesiredBytesLen,
-                   const uint8_t* aRangeMin, const uint8_t* aRangeMax) const {
+                   const uint8_t* aRangeMin, const uint8_t* aRangeMax) {
+    // Convert the given pointers to uintptr_t because we should not
+    // compare two pointers unless they are from the same array or object.
+    uintptr_t rangeMin = reinterpret_cast<uintptr_t>(aRangeMin);
+    uintptr_t rangeMax = reinterpret_cast<uintptr_t>(aRangeMax);
+
     const DWORD kGranularity = GetAllocGranularity();
-    MOZ_ASSERT(aDesiredBytesLen >= kGranularity);
     if (!aDesiredBytesLen) {
+      SetLastDetourError(MMPOLICY_RESERVE_FINDREGION_INVALIDLEN);
       return nullptr;
     }
 
-    MOZ_ASSERT(aRangeMin < aRangeMax);
-    if (aRangeMin >= aRangeMax) {
+    MOZ_ASSERT(rangeMin < rangeMax);
+    if (rangeMin >= rangeMax) {
+      SetLastDetourError(MMPOLICY_RESERVE_FINDREGION_INVALIDRANGE);
       return nullptr;
     }
 
@@ -277,31 +310,68 @@ class MOZ_TRIVIAL_CTOR_DTOR MMPolicyBase {
 
     // Reduce rnd to a value that falls within the acceptable range
     uintptr_t maxOffset =
-        (aRangeMax - aRangeMin - aDesiredBytesLen) / kGranularity;
-    uintptr_t offset = (uintptr_t(rnd) % maxOffset) * kGranularity;
+        (rangeMax - rangeMin - aDesiredBytesLen) / kGranularity;
+    // Divide by maxOffset + 1 because maxOffset * kGranularity is acceptable.
+    uintptr_t offset = (uintptr_t(rnd) % (maxOffset + 1)) * kGranularity;
 
     // Start searching at this address
-    const uint8_t* address = aRangeMin + offset;
+    const uintptr_t searchStart = rangeMin + offset;
     // The max address needs to incorporate the desired length
-    const uint8_t* const kMaxPtr = aRangeMax - aDesiredBytesLen;
+    const uintptr_t kMaxPtr = rangeMax - aDesiredBytesLen;
 
-    MOZ_DIAGNOSTIC_ASSERT(address <= kMaxPtr);
+    MOZ_DIAGNOSTIC_ASSERT(searchStart <= kMaxPtr);
 
     MEMORY_BASIC_INFORMATION mbi;
     SIZE_T len = sizeof(mbi);
 
     // Scan the range for a free chunk that is at least as large as
     // aDesiredBytesLen
-    while (address <= kMaxPtr &&
-           nt::VirtualQueryEx(aProcess, address, &mbi, len)) {
-      if (mbi.State == MEM_FREE && mbi.RegionSize >= aDesiredBytesLen) {
-        return mbi.BaseAddress;
+    // Scan [searchStart, kMaxPtr]
+    for (uintptr_t address = searchStart; address <= kMaxPtr;) {
+      if (nt::VirtualQueryEx(aProcess, reinterpret_cast<uint8_t*>(address),
+                             &mbi, len) != len) {
+        SetLastDetourError(MMPOLICY_RESERVE_FINDREGION_VIRTUALQUERY_ERROR,
+                           ::GetLastError());
+        return nullptr;
       }
 
-      address =
-          reinterpret_cast<const uint8_t*>(mbi.BaseAddress) + mbi.RegionSize;
+      if (mbi.State == MEM_FREE) {
+        // |mbi.BaseAddress| is aligned with the page granularity, but may not
+        // be aligned with the allocation granularity.  VirtualAlloc does not
+        // accept such a non-aligned address unless the corresponding allocation
+        // region is free.  So we get the next boundary's start address.
+        PVOID regionStart = AlignUpToRegion(mbi.BaseAddress, kGranularity,
+                                            mbi.RegionSize, aDesiredBytesLen);
+        if (regionStart) {
+          return regionStart;
+        }
+      }
+
+      address = reinterpret_cast<uintptr_t>(mbi.BaseAddress) + mbi.RegionSize;
     }
 
+    // Scan [aRangeMin, searchStart)
+    for (uintptr_t address = rangeMin; address < searchStart;) {
+      if (nt::VirtualQueryEx(aProcess, reinterpret_cast<uint8_t*>(address),
+                             &mbi, len) != len) {
+        SetLastDetourError(MMPOLICY_RESERVE_FINDREGION_VIRTUALQUERY_ERROR,
+                           ::GetLastError());
+        return nullptr;
+      }
+
+      if (mbi.State == MEM_FREE) {
+        PVOID regionStart = AlignUpToRegion(mbi.BaseAddress, kGranularity,
+                                            mbi.RegionSize, aDesiredBytesLen);
+        if (regionStart) {
+          return regionStart;
+        }
+      }
+
+      address = reinterpret_cast<uintptr_t>(mbi.BaseAddress) + mbi.RegionSize;
+    }
+
+    SetLastDetourError(MMPOLICY_RESERVE_FINDREGION_NO_FREE_REGION,
+                       ::GetLastError());
     return nullptr;
   }
 
@@ -324,10 +394,15 @@ class MOZ_TRIVIAL_CTOR_DTOR MMPolicyBase {
   PVOID Reserve(HANDLE aProcess, const uint32_t aSize,
                 const ReserveFnT& aReserveFn,
                 const ReserveRangeFnT& aReserveRangeFn,
-                const Maybe<Span<const uint8_t>>& aBounds) const {
+                const Maybe<Span<const uint8_t>>& aBounds) {
     if (!aBounds) {
       // No restrictions, let the OS choose the base address
-      return aReserveFn(aProcess, nullptr, aSize);
+      PVOID ret = aReserveFn(aProcess, nullptr, aSize);
+      if (!ret) {
+        SetLastDetourError(MMPOLICY_RESERVE_NOBOUND_RESERVE_ERROR,
+                           ::GetLastError());
+      }
+      return ret;
     }
 
     const uint8_t* lowerBound = GetLowerBound(aBounds.ref());
@@ -365,8 +440,12 @@ class MOZ_TRIVIAL_CTOR_DTOR MMPolicyBase {
     // If we run out of attempts, we fall through to the default case where
     // the system chooses any base address it wants. In that case, the hook
     // will be set on a best-effort basis.
-
-    return aReserveFn(aProcess, nullptr, aSize);
+    PVOID ret = aReserveFn(aProcess, nullptr, aSize);
+    if (!ret) {
+      SetLastDetourError(MMPOLICY_RESERVE_FINAL_RESERVE_ERROR,
+                         ::GetLastError());
+    }
+    return ret;
   }
 };
 
@@ -464,8 +543,8 @@ class MOZ_TRIVIAL_CTOR_DTOR MMPolicyInProcess
     auto reserveWithinRangeFn =
         [](HANDLE aProcess, uint32_t aSize, const uint8_t* aRangeMin,
            const uint8_t* aRangeMaxExcl) -> Maybe<PVOID> {
-      static const StaticDynamicallyLinkedFunctionPtr<decltype(
-          &::VirtualAlloc2)>
+      static const StaticDynamicallyLinkedFunctionPtr<
+          decltype(&::VirtualAlloc2)>
           pVirtualAlloc2(L"kernelbase.dll", "VirtualAlloc2");
       if (!pVirtualAlloc2) {
         return Nothing();
@@ -535,6 +614,7 @@ class MMPolicyInProcessEarlyStage : public MMPolicyInProcessPrimitive {
  public:
   struct Kernel32Exports {
     decltype(&::FlushInstructionCache) mFlushInstructionCache;
+    decltype(&::GetModuleHandleW) mGetModuleHandleW;
     decltype(&::GetSystemInfo) mGetSystemInfo;
     decltype(&::VirtualProtect) mVirtualProtect;
   };
@@ -729,12 +809,13 @@ class MMPolicyOutOfProcess : public MMPolicyBase {
   /**
    * @return true if the page that hosts aVAddress is accessible.
    */
-  bool IsPageAccessible(void* aVAddress) const {
+  bool IsPageAccessible(uintptr_t aVAddress) const {
     MEMORY_BASIC_INFORMATION mbi;
-    SIZE_T result = nt::VirtualQueryEx(mProcess, aVAddress, &mbi, sizeof(mbi));
+    SIZE_T result = nt::VirtualQueryEx(
+        mProcess, reinterpret_cast<LPCVOID>(aVAddress), &mbi, sizeof(mbi));
 
-    return result && mbi.AllocationProtect && (mbi.Type & MEM_IMAGE) &&
-           mbi.State == MEM_COMMIT && mbi.Protect != PAGE_NOACCESS;
+    return result && mbi.AllocationProtect && mbi.State == MEM_COMMIT &&
+           mbi.Protect != PAGE_NOACCESS;
   }
 
   bool FlushInstructionCache() const {
@@ -762,26 +843,30 @@ class MMPolicyOutOfProcess : public MMPolicyBase {
   uint32_t Reserve(const uint32_t aSize,
                    const Maybe<Span<const uint8_t>>& aBounds) {
     if (!aSize || !mProcess) {
+      SetLastDetourError(MMPOLICY_RESERVE_INVALIDARG);
       return 0;
     }
 
     if (mRemoteView) {
       MOZ_ASSERT(mReservationSize >= aSize);
+      SetLastDetourError(MMPOLICY_RESERVE_ZERO_RESERVATIONSIZE);
       return mReservationSize;
     }
 
     mReservationSize = ComputeAllocationSize(aSize);
 
-    mMapping = ::CreateFileMapping(INVALID_HANDLE_VALUE, nullptr,
-                                   PAGE_EXECUTE_READWRITE | SEC_RESERVE, 0,
-                                   mReservationSize, nullptr);
+    mMapping = ::CreateFileMappingW(INVALID_HANDLE_VALUE, nullptr,
+                                    PAGE_EXECUTE_READWRITE | SEC_RESERVE, 0,
+                                    mReservationSize, nullptr);
     if (!mMapping) {
+      SetLastDetourError(MMPOLICY_RESERVE_CREATEFILEMAPPING, ::GetLastError());
       return 0;
     }
 
     mLocalView = static_cast<uint8_t*>(
         ::MapViewOfFile(mMapping, FILE_MAP_WRITE, 0, 0, 0));
     if (!mLocalView) {
+      SetLastDetourError(MMPOLICY_RESERVE_MAPVIEWOFFILE, ::GetLastError());
       return 0;
     }
 
@@ -795,8 +880,8 @@ class MMPolicyOutOfProcess : public MMPolicyBase {
         [mapping = mMapping](HANDLE aProcess, uint32_t aSize,
                              const uint8_t* aRangeMin,
                              const uint8_t* aRangeMaxExcl) -> Maybe<PVOID> {
-      static const StaticDynamicallyLinkedFunctionPtr<decltype(
-          &::MapViewOfFile3)>
+      static const StaticDynamicallyLinkedFunctionPtr<
+          decltype(&::MapViewOfFile3)>
           pMapViewOfFile3(L"kernelbase.dll", "MapViewOfFile3");
       if (!pMapViewOfFile3) {
         return Nothing();
