@@ -16,7 +16,7 @@ import os
 import sys
 import traceback
 import uuid
-from collections import Iterable
+from collections.abc import Iterable
 
 from six import string_types
 
@@ -33,62 +33,77 @@ from .config import ConfigSettings
 from .dispatcher import CommandAction
 from .logging import LoggingManager
 from .registrar import Registrar
-from .util import setenv
+from .sentry import register_sentry, NoopErrorReporter
+from .telemetry import report_invocation_metrics, create_telemetry_from_environment
+from .util import setenv, UserError
 
-SUGGEST_MACH_BUSTED = r'''
+SUGGEST_MACH_BUSTED_TEMPLATE = r"""
 You can invoke |./mach busted| to check if this issue is already on file. If it
-isn't, please use |./mach busted file| to report it. If |./mach busted| is
+isn't, please use |./mach busted file %s| to report it. If |./mach busted| is
 misbehaving, you can also inspect the dependencies of bug 1543241.
-'''.lstrip()
+""".lstrip()
 
-MACH_ERROR = r'''
+MACH_ERROR_TEMPLATE = (
+    r"""
 The error occurred in mach itself. This is likely a bug in mach itself or a
 fundamental problem with a loaded module.
 
-'''.lstrip() + SUGGEST_MACH_BUSTED
+""".lstrip()
+    + SUGGEST_MACH_BUSTED_TEMPLATE
+)
 
-ERROR_FOOTER = r'''
+ERROR_FOOTER = r"""
 If filing a bug, please include the full output of mach, including this error
 message.
 
 The details of the failure are as follows:
-'''.lstrip()
+""".lstrip()
 
-COMMAND_ERROR = r'''
+USER_ERROR = r"""
+This is a user error and does not appear to be a bug in mach.
+""".lstrip()
+
+COMMAND_ERROR_TEMPLATE = (
+    r"""
 The error occurred in the implementation of the invoked mach command.
 
 This should never occur and is likely a bug in the implementation of that
 command.
-'''.lstrip() + SUGGEST_MACH_BUSTED
+""".lstrip()
+    + SUGGEST_MACH_BUSTED_TEMPLATE
+)
 
-MODULE_ERROR = r'''
+MODULE_ERROR_TEMPLATE = (
+    r"""
 The error occurred in code that was called by the mach command. This is either
 a bug in the called code itself or in the way that mach is calling it.
-'''.lstrip() + SUGGEST_MACH_BUSTED
+""".lstrip()
+    + SUGGEST_MACH_BUSTED_TEMPLATE
+)
 
-NO_COMMAND_ERROR = r'''
+NO_COMMAND_ERROR = r"""
 It looks like you tried to run mach without a command.
 
 Run |mach help| to show a list of commands.
-'''.lstrip()
+""".lstrip()
 
-UNKNOWN_COMMAND_ERROR = r'''
+UNKNOWN_COMMAND_ERROR = r"""
 It looks like you are trying to %s an unknown mach command: %s
 %s
 Run |mach help| to show a list of commands.
-'''.lstrip()
+""".lstrip()
 
-SUGGESTED_COMMANDS_MESSAGE = r'''
+SUGGESTED_COMMANDS_MESSAGE = r"""
 Did you want to %s any of these commands instead: %s?
-'''
+"""
 
-UNRECOGNIZED_ARGUMENT_ERROR = r'''
+UNRECOGNIZED_ARGUMENT_ERROR = r"""
 It looks like you passed an unrecognized argument into mach.
 
 The %s command does not accept the arguments: %s
-'''.lstrip()
+""".lstrip()
 
-INVALID_ENTRY_POINT = r'''
+INVALID_ENTRY_POINT = r"""
 Entry points should return a list of command providers or directories
 containing command providers. The following entry point is invalid:
 
@@ -97,7 +112,7 @@ containing command providers. The following entry point is invalid:
 You are seeing this because there is an error in an external module attempting
 to implement a mach command. Please fix the error, or uninstall the module from
 your system.
-'''.lstrip()
+""".lstrip()
 
 
 class ArgumentParser(argparse.ArgumentParser):
@@ -105,11 +120,11 @@ class ArgumentParser(argparse.ArgumentParser):
 
     def error(self, message):
         """Custom error reporter to give more helpful text on bad commands."""
-        if not message.startswith('argument command: invalid choice'):
+        if not message.startswith("argument command: invalid choice"):
             argparse.ArgumentParser.error(self, message)
             assert False
 
-        print('Invalid command specified. The list of commands is below.\n')
+        print("Invalid command specified. The list of commands is below.\n")
         self.print_help()
         sys.exit(1)
 
@@ -122,15 +137,15 @@ class ArgumentParser(argparse.ArgumentParser):
         #   {foo,bar}
         #     foo  Do foo.
         #     bar  Do bar.
-        search = 'Commands:\n  {'
+        search = "Commands:\n  {"
         start = text.find(search)
 
         if start != -1:
-            end = text.find('}\n', start)
+            end = text.find("}\n", start)
             assert end != -1
 
-            real_start = start + len('Commands:\n')
-            real_end = end + len('}\n')
+            real_start = start + len("Commands:\n")
+            real_end = end + len("}\n")
 
             text = text[0:real_start] + text[real_end:]
 
@@ -139,15 +154,15 @@ class ArgumentParser(argparse.ArgumentParser):
 
 class ContextWrapper(object):
     def __init__(self, context, handler):
-        object.__setattr__(self, '_context', context)
-        object.__setattr__(self, '_handler', handler)
+        object.__setattr__(self, "_context", context)
+        object.__setattr__(self, "_handler", handler)
 
     def __getattribute__(self, key):
         try:
-            return getattr(object.__getattribute__(self, '_context'), key)
+            return getattr(object.__getattribute__(self, "_context"), key)
         except AttributeError as e:
             try:
-                ret = object.__getattribute__(self, '_handler')(self, key)
+                ret = object.__getattribute__(self, "_handler")(key)
             except (AttributeError, TypeError):
                 # TypeError is in case the handler comes from old code not
                 # taking a key argument.
@@ -156,7 +171,7 @@ class ContextWrapper(object):
             return ret
 
     def __setattr__(self, key, value):
-        setattr(object.__getattribute__(self, '_context'), key, value)
+        setattr(object.__getattribute__(self, "_context"), key, value)
 
 
 class Mach(object):
@@ -170,14 +185,11 @@ class Mach(object):
 
         populate_context_handler -- If defined, it must be a callable. The
             callable signature is the following:
-                populate_context_handler(context, key=None)
+                populate_context_handler(key=None)
             It acts as a fallback getter for the mach.base.CommandContext
             instance.
             This allows to augment the context instance with arbitrary data
             for use in command handlers.
-            For backwards compatibility, it is also called before command
-            dispatch without a key, allowing the context handler to add
-            attributes to the context instance.
 
         require_conditions -- If True, commands that do not have any condition
             functions applied will be skipped. Defaults to False.
@@ -213,21 +225,12 @@ To see more help for a specific command, run:
         self.settings = ConfigSettings()
         self.settings_paths = []
 
-        if 'MACHRC' in os.environ:
-            self.settings_paths.append(os.environ['MACHRC'])
+        if "MACHRC" in os.environ:
+            self.settings_paths.append(os.environ["MACHRC"])
 
         self.log_manager.register_structured_logger(self.logger)
-        self.global_arguments = []
         self.populate_context_handler = None
 
-    def add_global_argument(self, *args, **kwargs):
-        """Register a global argument with the argument parser.
-
-        Arguments are proxied to ArgumentParser.add_argument()
-        """
-
-        self.global_arguments.append((args, kwargs))
-
     def load_commands_from_directory(self, path):
         """Scan for mach commands from modules in a directory.
 
@@ -235,11 +238,11 @@ To see more help for a specific command, run:
         registers and found mach command providers with this mach instance.
         """
         for f in sorted(os.listdir(path)):
-            if not f.endswith('.py') or f == '__init__.py':
+            if not f.endswith(".py") or f == "__init__.py":
                 continue
 
             full_path = os.path.join(path, f)
-            module_name = 'mach.commands.%s' % f[0:-3]
+            module_name = "mach.commands.%s" % f[0:-3]
 
             self.load_commands_from_file(full_path, module_name=module_name)
 
@@ -253,11 +256,11 @@ To see more help for a specific command, run:
         if module_name is None:
             # Ensure parent module is present otherwise we'll (likely) get
             # an error due to unknown parent.
-            if 'mach.commands' not in sys.modules:
-                mod = imp.new_module('mach.commands')
-                sys.modules['mach.commands'] = mod
+            if "mach.commands" not in sys.modules:
+                mod = imp.new_module("mach.commands")
+                sys.modules["mach.commands"] = mod
 
-            module_name = 'mach.commands.%s' % uuid.uuid4().hex
+            module_name = "mach.commands.%s" % uuid.uuid4().hex
 
         try:
             imp.load_source(module_name, path)
@@ -265,9 +268,9 @@ To see more help for a specific command, run:
             if e.errno != errno.ENOENT:
                 raise
 
-            raise MissingFileError('%s does not exist' % path)
+            raise MissingFileError("%s does not exist" % path)
 
-    def load_commands_from_entry_point(self, group='mach.providers'):
+    def load_commands_from_entry_point(self, group="mach.providers"):
         """Scan installed packages for mach command provider entry points. An
         entry point is a function that returns a list of paths to files or
         directories containing command providers.
@@ -278,8 +281,10 @@ To see more help for a specific command, run:
         try:
             import pkg_resources
         except ImportError:
-            print("Could not find setuptools, ignoring command entry points",
-                  file=sys.stderr)
+            print(
+                "Could not find setuptools, ignoring command entry points",
+                file=sys.stderr,
+            )
             return
 
         for entry in pkg_resources.iter_entry_points(group=group, name=None):
@@ -315,6 +320,7 @@ To see more help for a specific command, run:
         Returns the integer exit code that should be used. 0 means success. All
         other values indicate failure.
         """
+        sentry = NoopErrorReporter()
 
         # If no encoding is defined, we default to UTF-8 because without this
         # Python 2.7 will assume the default encoding of ASCII. This will blow
@@ -336,26 +342,32 @@ To see more help for a specific command, run:
         orig_env = dict(os.environ)
 
         try:
+            # Load settings as early as possible so things in dispatcher.py
+            # can use them.
+            for provider in Registrar.settings_providers:
+                self.settings.register_provider(provider)
+            self.load_settings(self.settings_paths)
+
             if sys.version_info < (3, 0):
                 if stdin.encoding is None:
-                    sys.stdin = codecs.getreader('utf-8')(stdin)
+                    sys.stdin = codecs.getreader("utf-8")(stdin)
 
                 if stdout.encoding is None:
-                    sys.stdout = codecs.getwriter('utf-8')(stdout)
+                    sys.stdout = codecs.getwriter("utf-8")(stdout)
 
                 if stderr.encoding is None:
-                    sys.stderr = codecs.getwriter('utf-8')(stderr)
+                    sys.stderr = codecs.getwriter("utf-8")(stderr)
 
             # Allow invoked processes (which may not have a handle on the
             # original stdout file descriptor) to know if the original stdout
             # is a TTY. This provides a mechanism to allow said processes to
             # enable emitting code codes, for example.
             if os.isatty(orig_stdout.fileno()):
-                setenv('MACH_STDOUT_ISATTY', '1')
+                setenv("MACH_STDOUT_ISATTY", "1")
 
             return self._run(argv)
         except KeyboardInterrupt:
-            print('mach interrupted by signal or user action. Stopping.')
+            print("mach interrupted by signal or user action. Stopping.")
             return 1
 
         except Exception:
@@ -365,12 +377,15 @@ To see more help for a specific command, run:
             # bug in mach (or a loaded command module being silly) and thus
             # should be reported differently.
             self._print_error_header(argv, sys.stdout)
-            print(MACH_ERROR)
+            print(MACH_ERROR_TEMPLATE % "general")
 
             exc_type, exc_value, exc_tb = sys.exc_info()
             stack = traceback.extract_tb(exc_tb)
 
-            self._print_exception(sys.stdout, exc_type, exc_value, stack)
+            sentry_event_id = sentry.report_exception(exc_value)
+            self._print_exception(
+                sys.stdout, exc_type, exc_value, stack, sentry_event_id=sentry_event_id
+            )
 
             return 1
 
@@ -383,21 +398,25 @@ To see more help for a specific command, run:
             sys.stderr = orig_stderr
 
     def _run(self, argv):
-        # Load settings as early as possible so things in dispatcher.py
-        # can use them.
-        for provider in Registrar.settings_providers:
-            self.settings.register_provider(provider)
-        self.load_settings(self.settings_paths)
+        topsrcdir = None
+        if self.populate_context_handler:
+            topsrcdir = self.populate_context_handler("topdir")
+            sentry = register_sentry(argv, self.settings, topsrcdir)
+        else:
+            sentry = NoopErrorReporter()
 
-        context = CommandContext(cwd=self.cwd,
-                                 settings=self.settings, log_manager=self.log_manager,
-                                 commands=Registrar)
+        context = CommandContext(
+            cwd=self.cwd,
+            settings=self.settings,
+            log_manager=self.log_manager,
+            commands=Registrar,
+        )
 
         if self.populate_context_handler:
-            self.populate_context_handler(context)
             context = ContextWrapper(context, self.populate_context_handler)
 
         parser = self.get_argument_parser(context)
+        context.global_parser = parser
 
         if not len(argv):
             # We don't register the usage until here because if it is globally
@@ -408,36 +427,35 @@ To see more help for a specific command, run:
             return 0
 
         try:
-            try:
-                args = parser.parse_args(argv)
-            except NoCommandError as e:
-                if e.namespace.print_command:
-                    context.get_command = True
-                    args = parser.parse_args(e.namespace.print_command)
-                    if args.command == 'mach-completion':
-                        args = parser.parse_args(e.namespace.print_command[2:])
-                    print(args.command)
-                    return 0
-                else:
-                    raise
+            args = parser.parse_args(argv)
         except NoCommandError:
             print(NO_COMMAND_ERROR)
             return 1
         except UnknownCommandError as e:
-            suggestion_message = SUGGESTED_COMMANDS_MESSAGE % (
-                e.verb, ', '.join(e.suggested_commands)) if e.suggested_commands else ''
-            print(UNKNOWN_COMMAND_ERROR %
-                  (e.verb, e.command, suggestion_message))
+            suggestion_message = (
+                SUGGESTED_COMMANDS_MESSAGE % (e.verb, ", ".join(e.suggested_commands))
+                if e.suggested_commands
+                else ""
+            )
+            print(UNKNOWN_COMMAND_ERROR % (e.verb, e.command, suggestion_message))
             return 1
         except UnrecognizedArgumentError as e:
-            print(UNRECOGNIZED_ARGUMENT_ERROR % (e.command,
-                                                 ' '.join(e.arguments)))
+            print(UNRECOGNIZED_ARGUMENT_ERROR % (e.command, " ".join(e.arguments)))
             return 1
 
-        if not hasattr(args, 'mach_handler'):
-            raise MachError('ArgumentParser result missing mach handler info.')
+        if not hasattr(args, "mach_handler"):
+            raise MachError("ArgumentParser result missing mach handler info.")
 
-        handler = getattr(args, 'mach_handler')
+        context.is_interactive = (
+            args.is_interactive
+            and sys.__stdout__.isatty()
+            and sys.__stderr__.isatty()
+            and not os.environ.get("MOZ_AUTOMATION", None)
+        )
+        context.telemetry = create_telemetry_from_environment(self.settings)
+
+        handler = getattr(args, "mach_handler")
+        report_invocation_metrics(context.telemetry, handler.name)
 
         # Add JSON logging to a file if requested.
         if args.logfile:
@@ -448,17 +466,17 @@ To see more help for a specific command, run:
         if args.verbose:
             log_level = logging.DEBUG
 
-        self.log_manager.register_structured_logger(logging.getLogger('mach'))
+        self.log_manager.register_structured_logger(logging.getLogger("mach"))
 
         write_times = True
-        if args.log_no_times or 'MACH_NO_WRITE_TIMES' in os.environ:
+        if args.log_no_times or "MACH_NO_WRITE_TIMES" in os.environ:
             write_times = False
 
         # Always enable terminal logging. The log manager figures out if we are
         # actually in a TTY or are a pipe and does the right thing.
-        self.log_manager.add_terminal_logging(level=log_level,
-                                              write_interval=args.log_interval,
-                                              write_times=write_times)
+        self.log_manager.add_terminal_logging(
+            level=log_level, write_interval=args.log_interval, write_times=write_times
+        )
 
         if args.settings_file:
             # Argument parsing has already happened, so settings that apply
@@ -467,15 +485,28 @@ To see more help for a specific command, run:
 
         try:
             return Registrar._run_command_handler(
-                handler, context, debug_command=args.debug_command,
-                **vars(args.command_args))
+                handler,
+                context,
+                debug_command=args.debug_command,
+                **vars(args.command_args)
+            )
         except KeyboardInterrupt as ki:
             raise ki
         except FailedCommandError as e:
-            print(e.message)
+            print(e)
             return e.exit_code
+        except UserError:
+            # We explicitly don't report UserErrors to Sentry.
+            exc_type, exc_value, exc_tb = sys.exc_info()
+            # The first two frames are us and are never used.
+            stack = traceback.extract_tb(exc_tb)[2:]
+            self._print_error_header(argv, sys.stdout)
+            print(USER_ERROR)
+            self._print_exception(sys.stdout, exc_type, exc_value, stack)
+            return 1
         except Exception:
             exc_type, exc_value, exc_tb = sys.exc_info()
+            sentry_event_id = sentry.report_exception(exc_value)
 
             # The first two frames are us and are never used.
             stack = traceback.extract_tb(exc_tb)[2:]
@@ -487,9 +518,14 @@ To see more help for a specific command, run:
             # argument on the method. We handle that here until the module
             # loader grows the ability to validate better.
             if not len(stack):
-                print(COMMAND_ERROR)
-                self._print_exception(sys.stdout, exc_type, exc_value,
-                                      traceback.extract_tb(exc_tb))
+                print(COMMAND_ERROR_TEMPLATE % handler.name)
+                self._print_exception(
+                    sys.stdout,
+                    exc_type,
+                    exc_value,
+                    traceback.extract_tb(exc_tb),
+                    sentry_event_id=sentry_event_id,
+                )
                 return 1
 
             # Split the frames into those from the module containing the
@@ -512,36 +548,42 @@ To see more help for a specific command, run:
             self._print_error_header(argv, sys.stdout)
 
             if len(other_frames):
-                print(MODULE_ERROR)
+                print(MODULE_ERROR_TEMPLATE % handler.name)
             else:
-                print(COMMAND_ERROR)
+                print(COMMAND_ERROR_TEMPLATE % handler.name)
 
-            self._print_exception(sys.stdout, exc_type, exc_value, stack)
+            self._print_exception(
+                sys.stdout, exc_type, exc_value, stack, sentry_event_id=sentry_event_id
+            )
 
             return 1
 
     def log(self, level, action, params, format_str):
         """Helper method to record a structured log event."""
-        self.logger.log(level, format_str,
-                        extra={'action': action, 'params': params})
+        self.logger.log(level, format_str, extra={"action": action, "params": params})
 
     def _print_error_header(self, argv, fh):
-        fh.write('Error running mach:\n\n')
-        fh.write('    ')
+        fh.write("Error running mach:\n\n")
+        fh.write("    ")
         fh.write(repr(argv))
-        fh.write('\n\n')
+        fh.write("\n\n")
 
-    def _print_exception(self, fh, exc_type, exc_value, stack):
+    def _print_exception(self, fh, exc_type, exc_value, stack, sentry_event_id=None):
         fh.write(ERROR_FOOTER)
-        fh.write('\n')
+        fh.write("\n")
 
         for l in traceback.format_exception_only(exc_type, exc_value):
             fh.write(l)
 
-        fh.write('\n')
+        fh.write("\n")
         for l in traceback.format_list(stack):
             fh.write(l)
 
+        if not sentry_event_id:
+            return
+
+        fh.write("\nSentry event ID: {}\n".format(sentry_event_id))
+
     def load_settings(self, paths):
         """Load the specified settings files.
 
@@ -553,7 +595,7 @@ To see more help for a specific command, run:
         if isinstance(paths, string_types):
             paths = [paths]
 
-        valid_names = ('machrc', '.machrc')
+        valid_names = ("machrc", ".machrc")
 
         def find_in_dir(base):
             if os.path.isfile(base):
@@ -572,50 +614,92 @@ To see more help for a specific command, run:
     def get_argument_parser(self, context):
         """Returns an argument parser for the command-line interface."""
 
-        parser = ArgumentParser(add_help=False,
-                                usage='%(prog)s [global arguments] '
-                                'command [command arguments]')
+        parser = ArgumentParser(
+            add_help=False,
+            usage="%(prog)s [global arguments] " "command [command arguments]",
+        )
 
+        # WARNING!!! If you add a global argument here, also add it to the
+        # global argument handling in the top-level `mach` script.
         # Order is important here as it dictates the order the auto-generated
         # help messages are printed.
-        global_group = parser.add_argument_group('Global Arguments')
+        global_group = parser.add_argument_group("Global Arguments")
 
-        global_group.add_argument('-v', '--verbose', dest='verbose',
-                                  action='store_true', default=False,
-                                  help='Print verbose output.')
-        global_group.add_argument('-l', '--log-file', dest='logfile',
-                                  metavar='FILENAME', type=argparse.FileType('a'),
-                                  help='Filename to write log data to.')
-        global_group.add_argument('--log-interval', dest='log_interval',
-                                  action='store_true', default=False,
-                                  help='Prefix log line with interval from last message rather '
-                                  'than relative time. Note that this is NOT execution time '
-                                  'if there are parallel operations.')
+        global_group.add_argument(
+            "-v",
+            "--verbose",
+            dest="verbose",
+            action="store_true",
+            default=False,
+            help="Print verbose output.",
+        )
+        global_group.add_argument(
+            "-l",
+            "--log-file",
+            dest="logfile",
+            metavar="FILENAME",
+            type=argparse.FileType("a"),
+            help="Filename to write log data to.",
+        )
+        global_group.add_argument(
+            "--log-interval",
+            dest="log_interval",
+            action="store_true",
+            default=False,
+            help="Prefix log line with interval from last message rather "
+            "than relative time. Note that this is NOT execution time "
+            "if there are parallel operations.",
+        )
+        global_group.add_argument(
+            "--no-interactive",
+            dest="is_interactive",
+            action="store_false",
+            help="Automatically selects the default option on any "
+            "interactive prompts. If the output is not a terminal, "
+            "then --no-interactive is assumed.",
+        )
         suppress_log_by_default = False
-        if 'INSIDE_EMACS' in os.environ:
+        if "INSIDE_EMACS" in os.environ:
             suppress_log_by_default = True
-        global_group.add_argument('--log-no-times', dest='log_no_times',
-                                  action='store_true', default=suppress_log_by_default,
-                                  help='Do not prefix log lines with times. By default, '
-                                  'mach will prefix each output line with the time since '
-                                  'command start.')
-        global_group.add_argument('-h', '--help', dest='help',
-                                  action='store_true', default=False,
-                                  help='Show this help message.')
-        global_group.add_argument('--debug-command', action='store_true',
-                                  help='Start a Python debugger when command is dispatched.')
-        global_group.add_argument('--settings', dest='settings_file',
-                                  metavar='FILENAME', default=None,
-                                  help='Path to settings file.')
-        global_group.add_argument('--print-command', nargs=argparse.REMAINDER,
-                                  help=argparse.SUPPRESS)
-
-        for args, kwargs in self.global_arguments:
-            global_group.add_argument(*args, **kwargs)
+        global_group.add_argument(
+            "--log-no-times",
+            dest="log_no_times",
+            action="store_true",
+            default=suppress_log_by_default,
+            help="Do not prefix log lines with times. By default, "
+            "mach will prefix each output line with the time since "
+            "command start.",
+        )
+        global_group.add_argument(
+            "-h",
+            "--help",
+            dest="help",
+            action="store_true",
+            default=False,
+            help="Show this help message.",
+        )
+        global_group.add_argument(
+            "--debug-command",
+            action="store_true",
+            help="Start a Python debugger when command is dispatched.",
+        )
+        global_group.add_argument(
+            "--profile-command",
+            action="store_true",
+            help="Capture a Python profile of the mach process as command is dispatched.",
+        )
+        global_group.add_argument(
+            "--settings",
+            dest="settings_file",
+            metavar="FILENAME",
+            default=None,
+            help="Path to settings file.",
+        )
 
         # We need to be last because CommandAction swallows all remaining
         # arguments and argparse parses arguments in the order they were added.
-        parser.add_argument('command', action=CommandAction,
-                            registrar=Registrar, context=context)
+        parser.add_argument(
+            "command", action=CommandAction, registrar=Registrar, context=context
+        )
 
         return parser
