@@ -16,12 +16,12 @@ from mozbuild.base import MachCommandBase
 
 @CommandProvider
 class TestCommandProvider(MachCommandBase):
-    @Command('throw', category='testing')
-    @CommandArgument('--message', '-m', default='General Error')
-    def throw(self, message):
+    @Command("throw", category="testing")
+    @CommandArgument("--message", "-m", default="General Error")
+    def throw(self, command_context, message):
         raise Exception(message)
 
-    @Command('throw_deep', category='testing')
-    @CommandArgument('--message', '-m', default='General Error')
-    def throw_deep(self, message):
+    @Command("throw_deep", category="testing")
+    @CommandArgument("--message", "-m", default="General Error")
+    def throw_deep(self, command_context, message):
         throw2.throw_deep(message)
