@@ -4,12 +4,12 @@
 
 from __future__ import absolute_import, print_function, unicode_literals
 
-LINUX_SCCACHE = 'linux64-sccache'
-MACOS_SCCACHE = 'macosx64-sccache'
-WIN64_SCCACHE = 'win64-sccache'
+LINUX_SCCACHE = "linux64-sccache"
+MACOS_SCCACHE = "macosx64-sccache"
+WIN64_SCCACHE = "win64-sccache"
 
 # sccache-dist currently expects clients to provide toolchains when
 # distributing from macOS or Windows, so we download linux binaries capable
 # of cross-compiling for these cases.
-RUSTC_DIST_TOOLCHAIN = 'rustc-dist-toolchain'
-CLANG_DIST_TOOLCHAIN = 'clang-dist-toolchain'
+RUSTC_DIST_TOOLCHAIN = "rustc-dist-toolchain"
+CLANG_DIST_TOOLCHAIN = "clang-dist-toolchain"
