@@ -15,8 +15,9 @@ def main():
     print("Downloading wpt manifest")
     sys.path.insert(0, buildconfig.topsrcdir)
     import manifestupdate
+
     return 0 if manifestupdate.run(buildconfig.topsrcdir, buildconfig.topobjdir) else 1
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     sys.exit(main())
