@@ -10,8 +10,8 @@ from mozversioncontrol import get_repository_object
 
 
 def test_context_manager(repo):
-    is_git = repo.vcs == 'git'
-    cmd = ['show', '--no-patch'] if is_git else ['tip']
+    is_git = repo.vcs == "git"
+    cmd = ["show", "--no-patch"] if is_git else ["tip"]
 
     vcs = get_repository_object(repo.strpath)
     output_subprocess = vcs._run(*cmd)
@@ -26,5 +26,5 @@ def test_context_manager(repo):
     assert output_subprocess == output_client
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     mozunit.main()
