@@ -18,31 +18,27 @@ from mozprofile.profile import Profile
 def test_nonce(tmpdir):
     # make a profile with one preference
     path = tmpdir.strpath
-    profile = Profile(path,
-                      preferences={'foo': 'bar'},
-                      restore=False)
-    user_js = os.path.join(profile.profile, 'user.js')
+    profile = Profile(path, preferences={"foo": "bar"}, restore=False)
+    user_js = os.path.join(profile.profile, "user.js")
     assert os.path.exists(user_js)
 
     # ensure the preference is correct
     prefs = Preferences.read_prefs(user_js)
-    assert dict(prefs) == {'foo': 'bar'}
+    assert dict(prefs) == {"foo": "bar"}
 
     del profile
 
     # augment the profile with a second preference
-    profile = Profile(path,
-                      preferences={'fleem': 'baz'},
-                      restore=True)
+    profile = Profile(path, preferences={"fleem": "baz"}, restore=True)
     prefs = Preferences.read_prefs(user_js)
-    assert dict(prefs) == {'foo': 'bar', 'fleem': 'baz'}
+    assert dict(prefs) == {"foo": "bar", "fleem": "baz"}
 
     # cleanup the profile;
     # this should remove the new preferences but not the old
     profile.cleanup()
     prefs = Preferences.read_prefs(user_js)
-    assert dict(prefs) == {'foo': 'bar'}
+    assert dict(prefs) == {"foo": "bar"}
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     mozunit.main()
