@@ -16,7 +16,7 @@ from six import string_types
 here = os.path.abspath(os.path.dirname(__file__))
 build = MozbuildObject.from_environment(cwd=here)
 
-sys.path.insert(0, os.path.join(build.topsrcdir, 'testing', 'mozharness'))
+sys.path.insert(0, os.path.join(build.topsrcdir, "testing", "mozharness"))
 from mozharness.base.log import INFO
 from mozharness.base.errors import BaseErrorList
 from mozharness.mozilla.structuredlog import StructuredOutputParser
@@ -26,8 +26,8 @@ from mozharness.mozilla.testing.errors import HarnessErrorList
 def get_mozharness_status(suite, lines, status, formatter=None, buf=None):
     """Given list of log lines, determine what the mozharness status would be."""
     parser = StructuredOutputParser(
-        config={'log_level': INFO},
-        error_list=BaseErrorList+HarnessErrorList,
+        config={"log_level": INFO},
+        error_list=BaseErrorList + HarnessErrorList,
         strict=False,
         suite_category=suite,
     )
@@ -38,7 +38,7 @@ def get_mozharness_status(suite, lines, status, formatter=None, buf=None):
     # Processing the log with mozharness will re-print all the output to stdout
     # Since this exact same output has already been printed by the actual test
     # run, temporarily redirect stdout to devnull.
-    buf = buf or open(os.devnull, 'w')
+    buf = buf or open(os.devnull, "w")
     orig = sys.stdout
     sys.stdout = buf
     for line in lines:
@@ -50,4 +50,5 @@ def get_mozharness_status(suite, lines, status, formatter=None, buf=None):
 def filter_action(actions, lines):
     if isinstance(actions, string_types):
         actions = (actions,)
-    return filter(lambda x: x['action'] in actions, lines)
+    # pylint --py3k: W1639
+    return list(filter(lambda x: x["action"] in actions, lines))
